// action.cpp: actions done on ObjectList and PointList
// also Undo()

#include "defs.h"
#include "render2d.h"

class UndoHandler {
public:
  int lastAction;
  double dx, dy;
  QList<customObject> undoObjects;
  QList<DPoint> undoPoints;
  QString oldtext;
};

UndoHandler HowToUndo;

// Action(int):  Executes action specified by (int)a.  Stores action
// in (class UndoHandler)HowToUndo in case user wants to undo.
void Render2D::Action(int a)
{
  customObject *tmp;

  // only allow one undo step -- 'forget' previous actions
  HowToUndo.undoObjects.clear();		
    
  switch (a) {
  case DELETE_SELECTED:
    // delete a single object selected using erase tool
    HowToUndo.lastAction = a;
    highlighted_line->Unhighlight();
    HowToUndo.undoObjects.append(highlighted_line);
    objectList.remove(highlighted_line);
    highlighted_line = 0;
    break;

  case DELETE_MULTIPLE:
    HowToUndo.lastAction = a;
    for (tmp = selectList.first(); tmp != 0; tmp = selectList.next() ) {
      tmp->Unhighlight();
      HowToUndo.undoObjects.append(tmp);
      objectList.remove(tmp);
    }
    break;

  case ADD_OBJECT:
    HowToUndo.lastAction = a;
    HowToUndo.undoObjects.append(tmp_obj);
    objectList.append(tmp_obj);
    break;

  case UPDATE_TEXT:
    HowToUndo.lastAction = a;
    HowToUndo.undoObjects.append(tmp_obj);
    HowToUndo.oldtext = tmp_obj->Label();
    tmp_obj->setLabel(tmp_text);
    break;
  }
}

// Undo(): undo last action
void Render2D::Undo()
{
  int rf = false;

  switch (HowToUndo.lastAction) {
  case DELETE_SELECTED:
    objectList.append(HowToUndo.undoObjects.first());
    rf = true;
    break;
  case DELETE_MULTIPLE:
    for (tmp_obj = HowToUndo.undoObjects.first(); tmp_obj != 0;
	 tmp_obj = HowToUndo.undoObjects.next() ) {
      objectList.append(tmp_obj);
    }
    rf = true;
    break;
  case ADD_OBJECT:
    objectList.remove(HowToUndo.undoObjects.first());
    rf = true;    
    break;
  case UPDATE_TEXT:
    tmp_obj = HowToUndo.undoObjects.first();
    tmp_obj->setLabel(HowToUndo.oldtext);
    repaint();
    break;

  default:
    emit UpdateStatusBar("Sorry, can't undo!");
    break;
  }
  
  // if an Undo occured, clear Undo structure and redraw
  if (rf) {
    repaint();
    HowToUndo.lastAction = 0;
    HowToUndo.undoObjects.clear();		
  }
}
