// application.cpp:  XDrawChem main window.

#include <qmime.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qmultilineedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qscrollview.h>
#include <qwhatsthis.h>
#include <qpushbutton.h>

#include "application.h"
#include "defs.h"
#include "render2d.h"
#include "config.h"
#include "ringdialog.h"
#include "symboldialog.h"
#include "pagesetupdialog.h"
#include "dpoint.h"

// XPM files define the icons
#include "filesave.xpm"
#include "fileopen.xpm"
#include "fileprint.xpm"
#include "savepic.xpm"
#include "selecttool.xpm"
#include "edittool.xpm"
#include "erasetool.xpm"
#include "linetool.xpm"
#include "dashtool.xpm"
#include "texttool.xpm"
#include "arrowtool.xpm"
#include "ringtool.xpm"
#include "symboltool.xpm"
#include "downtool.xpm"
#include "uptool.xpm"
#include "brackettool.xpm"
#include "cuttool.xpm"
#include "copytool.xpm"
#include "pastetool.xpm"

// What's This? for menus and file/edit toolbar
const char * fileOpenText =
"Click this button to open a <em>new file</em>. <br><br>"
"You can also select the <b>Open command</b> from the File menu.";
const char * fileSaveText = "Click this button to save the file you are "
"editing.  You will be prompted for a file name.\n\n"
"You can also select the Save command from the File menu.\n\n";
const char * filePrintText = "Click this button to print the file you "
"are editing.\n\n"
"You can also select the Print command from the File menu.";
const char * savePicText = "Click this button to save a bitmap image "
"of what is on the screen.  These bitmap images can be imported "
"into other applications.\n\nIf something is selected, only the selection will "
"be saved in the image.";

// What's This? for drawing toolbar
const char * selectToolText =
"Select tool<br><br>Click this button to select objects.<br><br>"
"This tool is used for moving, cut, copy, and paste.  "
"You can click and drag objects around the screen with this tool.";
const char * editToolText =
"Edit tool<br><br>Click this button to edit objects.<br><br>"
"This tool allows you to change text, labels, format, color, etc.";
const char * eraseToolText =
"Eraser<br><br>Click this to erase objects.";
const char * lineToolText =
"Line tool<br><br>Click this button to draw lines.";
const char * bracketToolText =
"Bracket tool<br><br>Click this button to draw brackets. "
"Only [ ] may be drawn at this time.";
const char * lineStereoToolText =
"Special line tool<br><br>Click this button to draw "
"a stereospecific bond.";
const char * textToolText =
"Text tool<br><br>Click this button to add text.<br><br>"
"You can also label points and arrows with this tool.";
const char * arrowToolText =
"Arrow tool<br><br>Click this button to draw straight arrows.";
const char * ringToolText =
"Ring tool<br><br>Click this button to insert rings.<br><br>"
"A dialog will appear with a list of rings.  Select a ring by clicking "
"on it.";
const char * symbolToolText =
"Symbol tool<br><br>Click this button to insert symbols.<br><br>"
"A dialog will appear with a list of symbols.  Select a symbol by clicking "
"on it.";

ApplicationWindow::ApplicationWindow()
    : QMainWindow( 0, "XDrawChem main window", WDestructiveClose )
{
    int id;
		QMimeSourceFactory *myfactory = QMimeSourceFactory::defaultFactory();

    printer = new QPrinter;
    QPixmap openIcon, saveIcon, printIcon, drawLineIcon, drawBoxIcon,
			selectIcon, textIcon, editBoxIcon, ringIcon, arrowIcon, dashIcon, 
			downIcon, upIcon, savePicIcon, bracketIcon, cutIcon, copyIcon, pasteIcon,
			symbolIcon;

		setCaption("XDrawChem - New Document");

		sv = new QScrollView(this, "Scroller");
    r = new Render2D( sv->viewport(), "2D render window" );
//		r->resize(750,1000);
		sv->addChild(r);

    fileTools = new QToolBar( this, "file operations" );

    openIcon = QPixmap( fileopen );
		myfactory->setImage(QString("fileopenicon"), QImage(fileopen) );
    QToolButton * fileOpen
	= new QToolButton( openIcon, "Open File", QString::null,
			   this, SLOT(load()), fileTools, "open file" );

    saveIcon = QPixmap( filesave );
		myfactory->setImage(QString("filesaveicon"), QImage(filesave) );
    QToolButton * fileSave
	= new QToolButton( saveIcon, "Save File", QString::null,
			   this, SLOT(save()), fileTools, "save file" );

		savePicIcon = QPixmap( savepic );
		myfactory->setImage(QString("savepicicon"), QImage(savepic) );
    QToolButton * fileSavePic
	= new QToolButton( savePicIcon, "Save Image", QString::null,
			   this, SLOT(savePicture()), fileTools, "save image" );

    printIcon = QPixmap( fileprint );
		myfactory->setImage(QString("fileprinticon"), QImage(fileprint) );
    QToolButton * filePrint
	= new QToolButton( printIcon, "Print File", QString::null,
			   r, SLOT(Print()), fileTools, "print file" );

		cutIcon = QPixmap(cuttool);
		myfactory->setImage(QString("cuttoolicon"), QImage(cuttool) );
    QToolButton * editCut
	= new QToolButton( cutIcon, "Cut", QString::null,
			   r, SLOT(Cut()), fileTools, "cut" );

		copyIcon = QPixmap(copytool);
		myfactory->setImage(QString("copytoolicon"), QImage(copytool) );
    QToolButton * editCopy
	= new QToolButton( copyIcon, "Copy", QString::null,
			   r, SLOT(Copy()), fileTools, "copy" );

		pasteIcon = QPixmap(pastetool);
		myfactory->setImage(QString("pastetoolicon"), QImage(pastetool) );
    QToolButton * editPaste
	= new QToolButton( pasteIcon, "Paste", QString::null,
			   r, SLOT(Paste()), fileTools, "paste" );

    QToolButton *wt = QWhatsThis::whatsThisButton( fileTools );
    myfactory->setPixmap( "whatsthisicon", *(wt->pixmap()) );

    QWhatsThis::add( fileOpen, fileOpenText );
    QWhatsThis::add( fileSave, fileSaveText );
    QWhatsThis::add( filePrint, filePrintText );
    QWhatsThis::add( fileSavePic, savePicText );

	drawTools = new QToolBar(this, "draw operations");
	moveToolBar(drawTools, QMainWindow::Left);
	selectIcon = QPixmap( selecttool );
  QToolButton * selectObjects
		= new QToolButton( selectIcon, "Select", QString::null,
			this, SLOT(setModeSelect()), drawTools, "select" );
	QWhatsThis::add(selectObjects, selectToolText);
	myfactory->setImage(QString("selecttoolicon"), QImage(selecttool) );

  drawBoxIcon = QPixmap( eraseIcon );
  QToolButton * drawBox
	  = new QToolButton( drawBoxIcon, "Eraser", QString::null,
			this, SLOT(setErase()), drawTools, "erase" );
	QWhatsThis::add(drawBox, eraseToolText);
	myfactory->setImage(QString("erasetoolicon"), QImage(eraseIcon) );

  drawLineIcon = QPixmap( linetool );
  QToolButton * drawLine
	  = new QToolButton( drawLineIcon, "Draw Line", QString::null,
			this, SLOT(setDrawLine()), drawTools, "draw line" );
	QWhatsThis::add(drawLine, lineToolText);
	myfactory->setImage(QString("linetoolicon"), QImage(linetool) );

  dashIcon = QPixmap( dashtool );
  QToolButton * drawDash
	  = new QToolButton( dashIcon, "Draw Dashed Line", QString::null,
			this, SLOT(setDashLine()), drawTools, "draw dashed line" );
	QWhatsThis::add(drawLine, lineToolText);
	myfactory->setImage(QString("dashtoolicon"), QImage(dashtool) );

  upIcon = QPixmap( uptool );
  QToolButton * upLine
	  = new QToolButton( upIcon, "Draw Line", QString::null,
			this, SLOT(setUpLine()), drawTools, "draw line" );
	QWhatsThis::add(upLine, lineStereoToolText);
	myfactory->setImage(QString("uplinetoolicon"), QImage(uptool) );

  downIcon = QPixmap( downtool );
  QToolButton * downLine
	  = new QToolButton( downIcon, "Draw Line", QString::null,
		  this, SLOT(setDownLine()), drawTools, "draw line" );
	QWhatsThis::add(downLine, lineStereoToolText);
	myfactory->setImage(QString("downlinetoolicon"), QImage(downtool) );

  arrowIcon = QPixmap( arrowtool );
  QToolButton * drawArrow
	  = new QToolButton( arrowIcon, "Draw Arrow", QString::null,
			this, SLOT(setArrow()), drawTools, "draw arrow" );
	QWhatsThis::add(drawArrow, arrowToolText);
	myfactory->setImage(QString("arrowtoolicon"), QImage(arrowtool) );

	bracketIcon = QPixmap( brackettool );
  QToolButton * drawBracket
	  = new QToolButton( bracketIcon, "Draw brackets", QString::null,
			this, SLOT( setBrackets() ), drawTools, "draw brackets" );
	QWhatsThis::add(drawBracket, bracketToolText);
	myfactory->setImage(QString("brackettoolicon"), QImage(brackettool) );

  textIcon = QPixmap( texttool );
  QToolButton * drawTextBox
	  = new QToolButton( textIcon, "Draw Text", QString::null,
			this, SLOT(setText()), drawTools, "draw text" );
	QWhatsThis::add(drawTextBox, textToolText);
	myfactory->setImage(QString("texttoolicon"), QImage(texttool) );

	ringIcon = QPixmap(ringtool);	
  ringButton
	  = new QToolButton( ringIcon, "Draw Ring", QString::null,
			this, SLOT(OpenRingSubmenu()), drawTools, "draw ring" );
	QWhatsThis::add(ringButton, ringToolText);
	myfactory->setImage(QString("ringtoolicon"), QImage(ringtool) );

	symbolIcon = QPixmap(symboltool);	
  QToolButton *symButton
	  = new QToolButton( symbolIcon, "Draw Symbol", QString::null,
			this, SLOT(OpenSymbolSubmenu()), drawTools, "draw symbol" );
	QWhatsThis::add(symButton, symbolToolText);
	myfactory->setImage(QString("symboltoolicon"), QImage(symboltool) );

    QPopupMenu * file = new QPopupMenu( this );
    menuBar()->insertItem( "&File", file );

    file->insertItem( "&New", this, SLOT(newDoc()), CTRL+Key_N );

    id = file->insertItem( openIcon, "&Open",
			   this, SLOT(load()), CTRL+Key_O );
    file->setWhatsThis( id, fileOpenText );

    id = file->insertItem( saveIcon, "&Save",
			   this, SLOT(save()), CTRL+Key_S );
    file->setWhatsThis( id, fileSaveText );
    id = file->insertItem( "Save &as...", this, SLOT(saveAs()) );
    file->setWhatsThis( id, fileSaveText );
    id = file->insertItem( savePicIcon, "Save &image...", this, 
			   SLOT(savePicture()) );
    id = file->insertItem( "Save ring template", r, SLOT(saveRing()) );
    file->insertItem( "Info...", this, SLOT(showInfo()), CTRL+Key_I );
    file->insertSeparator();
    id = file->insertItem( "Pa&ge setup...", this, SLOT(pageSetup()) );
    id = file->insertItem( printIcon, "&Print",
			   r, SLOT(Print()), CTRL+Key_P );
    file->setWhatsThis( id, filePrintText );
    file->insertSeparator();
    file->insertItem( "&Close", this, SLOT(close()), CTRL+Key_W );
//    file->insertItem( "&Quit", qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );
    file->insertItem( "&Quit", qApp, SLOT( quit() ), CTRL+Key_Q );

    QPopupMenu * edit = new QPopupMenu( this );
    menuBar()->insertItem("&Edit", edit);
    
    id = edit->insertItem( "&Undo", r, SLOT(Undo()), CTRL+Key_Z );		
    edit->insertSeparator();
    id = edit->insertItem( cutIcon, "Cu&t", r, SLOT(Cut()), CTRL+Key_X );
    id = edit->insertItem( copyIcon, "&Copy", r, SLOT(Copy()), CTRL+Key_C );
    id = edit->insertItem( pasteIcon, "&Paste", r, SLOT(Paste()), CTRL+Key_V );
    id = edit->insertItem( "Clea&r", r, SLOT(Clear()), Key_Delete );
    edit->insertSeparator();
    id = edit->insertItem( "Select &all", r, SLOT(selectAll()), CTRL+Key_A );
    edit->insertSeparator();
    id = edit->insertItem( "Flip &horizontal", r, SLOT(FlipH()) );
    id = edit->insertItem( "Flip &vertical", r, SLOT(FlipV()) );
    //    edit->insertSeparator();
    //    id = edit->insertItem( "Preferences...", this, SLOT(about()), 
    //			   CTRL+Key_Enter );
    
    QPopupMenu * fllist = new QPopupMenu(this);
    id = fllist->insertItem("25 pixels", r, SLOT(setFL25()) );
    id = fllist->insertItem("50 pixels", r, SLOT(setFL50()) );
    //id = fllist->insertItem("1/4 inch", r, SLOT(setFL_Quarter()) );
    //id = fllist->insertItem("1/2 inch", r, SLOT(setFL_Half()) );
    
    QPopupMenu * falist = new QPopupMenu(this);
    id = falist->insertItem("10 degrees", r, SLOT(setFA10()) );
    id = falist->insertItem("15 degrees", r, SLOT(setFA15()) );
    id = falist->insertItem("30 degrees", r, SLOT(setFA30()) );
    
    QPopupMenu * lwlist = new QPopupMenu(this);
    id = lwlist->insertItem("1", r, SLOT(setLW1()) );
    id = lwlist->insertItem("2", r, SLOT(setLW2()) );
    id = lwlist->insertItem("3", r, SLOT(setLW3()) );
    id = lwlist->insertItem("4", r, SLOT(setLW4()) );
    id = lwlist->insertItem("5", r, SLOT(setLW5()) );
    
    fontlist = new QPopupMenu(this);
    fontlist->setCheckable(true);
    f1 = fontlist->insertItem("Courier", r, SLOT(setFontCourier()) );
    f2 = fontlist->insertItem("Helvetica", r, SLOT(setFontHelv()) );
    fontlist->setItemChecked(f2, true);
    f3 = fontlist->insertItem("Times", r, SLOT(setFontTimes()) );

    fontsizelist = new QPopupMenu(this);
    fontsizelist->setCheckable(true);
    fs1 = fontsizelist->insertItem("8", r, SLOT(setFont8()) );
    fs2 = fontsizelist->insertItem("10", r, SLOT(setFont10()) );
    fs3 = fontsizelist->insertItem("12", r, SLOT(setFont12()) );
    fontsizelist->setItemChecked(fs3, true);
    fs4 = fontsizelist->insertItem("14", r, SLOT(setFont14()) );
    fs5 = fontsizelist->insertItem("18", r, SLOT(setFont18()) );
    fs6 = fontsizelist->insertItem("24", r, SLOT(setFont24()) );

    QPopupMenu * alignlist = new QPopupMenu(this);
    id = alignlist->insertItem( "Align on &horizontal line", this, 
				SLOT(AlignHCenter()) );
    id = alignlist->insertItem( "Align on &vertical line", this, 
				SLOT(AlignVCenter()) );

    QPopupMenu * colorlist = new QPopupMenu(this);
    id = colorlist->insertItem("Black", r, SLOT(setColorBlack()) );
    id = colorlist->insertItem("Red", r, SLOT(setColorRed()) );
    id = colorlist->insertItem("Green", r, SLOT(setColorGreen()) );
    id = colorlist->insertItem("Blue", r, SLOT(setColorBlue()) );
    
    format = new QPopupMenu(this);
    format->setCheckable(true);
    menuBar()->insertItem("Forma&t", format);
    id = format->insertItem( "Auto Format...", this, SLOT(doAutoFormat()), CTRL+Key_F );
    format->insertItem( "Format &options...", this, SLOT(formatOptions()) );
    format->insertSeparator();
    fixlen = format->insertItem( "Fixed &length/angles", this, SLOT(setFixed()) );
    id = format->insertItem( "Set fixed length", fllist );
    id = format->insertItem( "Set fixed angle", falist );
    format->insertSeparator();
    id = format->insertItem( "Color", colorlist);
    id = format->insertItem( "Line &thickness", lwlist );
    format->insertSeparator();
    id = format->insertItem( "&Font", fontlist );
    id = format->insertItem( "Font &size", fontsizelist );

    QPopupMenu * toolMenu = new QPopupMenu(this);
    menuBar()->insertItem( "T&ools", toolMenu );
    toolMenu->insertItem( "&Alignment", alignlist );
    toolMenu->insertItem( "&Calculate mass...", r, SLOT(FindMW()) );
    //    toolMenu->insertItem( "Test functionality", r, SLOT(TEST()) );
    
    QPopupMenu * help = new QPopupMenu( this );
    menuBar()->insertSeparator();
    menuBar()->insertItem( "&Help", help );

    help->insertItem( "&Manual", this, SLOT(Manual()), Key_F1);
    help->insertItem( "&About", this, SLOT(blurb()) );
    help->insertSeparator();
    help->insertItem( "What's &This", this, SLOT(whatsThis()), SHIFT+Key_F1);

//    r = new Render2D( this, "2D render window" );
		connect( r, SIGNAL(UpdateStatusBar(QString)), 
			this, SLOT(SetStatusBar(QString)) );
		connect( r, SIGNAL(ChooseFont(int)), this, SLOT(CheckFont(int)) );
		connect( r, SIGNAL(ChooseSize(int)), this, SLOT(CheckSize(int)) );
    setCentralWidget( sv );
    statusBar()->message( "Ready" );
    resize( 450, 600 );
}

ApplicationWindow::~ApplicationWindow()
{
    delete printer;
}

void ApplicationWindow::newDoc()
{
    ApplicationWindow *ed = new ApplicationWindow;
    ed->show();
}

void ApplicationWindow::setFixed()
{
  if (format->isItemChecked(fixlen)) {
    format->setItemChecked(fixlen, false);
    r->setParameter(FREE_DRAWING);
  } else {
    format->setItemChecked(fixlen, true);	
    r->setParameter(FIXED_DRAWING);
  }
}

void ApplicationWindow::CheckSize(int which) {
  fontsizelist->setItemChecked(fs1, false);
  fontsizelist->setItemChecked(fs2, false);
  fontsizelist->setItemChecked(fs3, false);
  fontsizelist->setItemChecked(fs4, false);
  fontsizelist->setItemChecked(fs5, false);
  fontsizelist->setItemChecked(fs6, false);
  if (which == FONT_8)
    fontsizelist->setItemChecked(fs1, true);
  if (which == FONT_10)
    fontsizelist->setItemChecked(fs2, true);
  if (which == FONT_12)
    fontsizelist->setItemChecked(fs3, true);
  if (which == FONT_14)
    fontsizelist->setItemChecked(fs4, true);
  if (which == FONT_18)
    fontsizelist->setItemChecked(fs5, true);
  if (which == FONT_24)
    fontsizelist->setItemChecked(fs6, true);
}

void ApplicationWindow::CheckFont(int which) {
  fontlist->setItemChecked(f1, false);
  fontlist->setItemChecked(f2, false);
  fontlist->setItemChecked(f3, false);
  if (which == FONT_COURIER)
    fontlist->setItemChecked(f1, true);
  if (which == FONT_HELVETICA)
    fontlist->setItemChecked(f2, true);
  if (which == FONT_TIMES)
    fontlist->setItemChecked(f3, true);
}

void ApplicationWindow::load()
{
  if (r->changed()) {
    switch (QMessageBox::warning(this, "Save?",
				 "Document changed since last save!\n"
				 "Save now?\n",
				 "Yes", "No", 0, 0, 1)) {
    case 0:  // Yes
      save();
      break;
    case 1:
      break;
    }
  }

  QFileDialog fd(QString::null, QString::null, 0, 0, TRUE);
  fd.setCaption("Load file");
  fd.setMode(QFileDialog::ExistingFile);
  QStringList filters;
  filters.append("All files (*)");
  filters.append("XDrawChem native (*.xdc)");
  filters.append("MDL Molfile (*.mol)");
  filters.append("CML - Chemical Markup Language (*.cml)");
  fd.setFilters(filters);  
  if ( fd.exec() == QDialog::Accepted ) {
    load(fd.selectedFile().ascii());
  } else {
    statusBar()->message( "Loading aborted" );
  }
}


void ApplicationWindow::load( const char *fileName )
{
    QString s;
    if (r->load(fileName)) {
      setCaption( QString("XDrawChem - ") + QString(fileName) );
      filename = fileName;
      statusBar()->message( QString("Loaded file ") + filename);
    }
}


void ApplicationWindow::save()
{
  if ( filename.isEmpty() ) {
    saveAs();
    return;
  }

  if ( r->save(filename.ascii()) ) {
    setCaption( QString("XDrawChem - ") + QString(filename) );
    statusBar()->message( QString( "Saved %1" ).arg( filename ), 2000 );
  } else {
    statusBar()->message( QString( "Failed to save %1" ).arg( filename ), 2000 );
    filename = QString("");
  }
}

void ApplicationWindow::saveAs()
{
  saveAs(QString(""));
}

void ApplicationWindow::saveAs(QString fn1)
{
  QFileDialog fd(QString::null, QString::null, 0, 0, TRUE);
  fd.setCaption("Save file as...");
  fd.setMode(QFileDialog::AnyFile);
  QStringList filters;
  filters.append("XDrawChem native (*.xdc)");
  filters.append("MDL Molfile (*.mol)");
  filters.append("CML - Chemical Markup Language (*.cml)");
  //filters.append("Encapsulated PostScript (*.eps)");
  fd.setFilters(filters);  
  fd.setSelection(fn1);
  if ( fd.exec() == QDialog::Accepted ) {
    filename = fd.selectedFile();
    QString tmpx = filename.right(4);
    if (tmpx.left(1) != QString(".")) {
      QMessageBox::warning(this, "Invalid file name", "You must explicitly type the file extension.\n*.xdc = XDrawChem format\n*.mol = MDL Molfile\n*.cml - Chemical Markup Language (CML)");
      saveAs(filename);
      return;
    }
    save();
  } else {
    statusBar()->message( "Save cancelled" );
  }
}

void ApplicationWindow::savePicture()
{
  int ftype = 0;
  QString ftypestr;

  QFileDialog fd(QString::null, QString::null, 0, 0, TRUE);
  fd.setCaption("Save file as...");
  fd.setMode(QFileDialog::AnyFile);
  QStringList filters;
  filters.append("Portable network graphics (*.png)");
  filters.append("Windows bitmap (*.bmp)");
  //filters.append("EPS - Encapsulated PostScript (*.eps)");
  fd.setFilters(filters);  
  if ( fd.exec() == QDialog::Accepted ) {
    QString fn = fd.selectedFile();
    QString rightend = fn.right(4);
    if (rightend.left(1) != QString(".")) fn.append(QString(".png"));
    if (fn.right(4) == QString(".bmp")) {
      ftype = IMG_BMP; ftypestr = QString("BMP");
    }
    if (fn.right(4) == QString(".png")) {
      ftype = IMG_PNG; ftypestr = QString("PNG");
    }
    if (fn.right(4) == QString(".eps")) {
      ftype = IMG_EPS; ftypestr = QString("EPS");
    }
    if (ftype == 0) {
      QMessageBox::warning(this, "Invalid file name", "You should type the file extension (lowercase, please).\n*.bmp = Windows bitmap format\n*.png = Portable network graphic (PNG)\n*.eps = Encapsulated PostScript (EPS)\n(if no extension, default is PNG)");		
      statusBar()->message( "Save cancelled" );
      return;
    }
    // true = error; false = ok
    if (r->savePicture(fn, ftype)) {
      statusBar()->message( "Error saving " + ftypestr + " image: " + fn );
    } else {
      statusBar()->message( "Saved " + ftypestr + " image: " + fn );
    }
  } else {
    statusBar()->message( "Save cancelled" );
  }
}

void ApplicationWindow::closeEvent(QCloseEvent *ce)
{
  if (r->changed()) {
    switch (QMessageBox::warning(this, "Save?",
				 "Document changed since last save!\n"
				 "Save now?\n",
				 "Yes", "No", "Cancel", 0, 1)) {
    case 0:  // Yes
      save();
      ce->accept();
      break;
    case 1:
      ce->accept();
      break;
    case 2:
    default:
      ce->ignore();
      break;
    }
  } else {
    ce->accept();
    return;
  }
}

void ApplicationWindow::SetStatusBar(QString msg)
{
  statusBar()->message( msg );
}

void ApplicationWindow::about()
{
  QMessageBox::about( this, "Information",
		      "Feature not implemented...");
}

void ApplicationWindow::blurb()
{
  QMessageBox::about( this, "About XDrawChem",
		      "XDrawChem 0.84a\n\nby Bryan Herger\nGeorgia Institute of Technology\n"
		      "herger@chemistry.gatech.edu");
}

void ApplicationWindow::setModeSelect() {	r->setMode(MODE_SELECT); }

void ApplicationWindow::setDrawLine() {	r->setMode(MODE_DRAW_LINE); }

void ApplicationWindow::setDashLine() {	r->setMode(MODE_DASH_LINE); }

void ApplicationWindow::setErase() {	r->setMode(MODE_ERASE); }

void ApplicationWindow::setText() {	r->setMode(MODE_TEXT); }

void ApplicationWindow::setArrow() {	r->setMode(MODE_ARROW); }

void ApplicationWindow::setUpLine() { r->setMode(MODE_UPLINE); }

void ApplicationWindow::setDownLine() { r->setMode(MODE_DOWNLINE); }

void ApplicationWindow::setBrackets() { r->setMode(MODE_BRACKETS); }

void ApplicationWindow::AlignHCenter() { r->Align(ALIGN_HCENTER); }

void ApplicationWindow::AlignVCenter() { r->Align(ALIGN_VCENTER); }

void ApplicationWindow::doAutoFormat() { r->AutoFormat(); }

void ApplicationWindow::pageSetup(void)
{
  PageSetupDialog t(this, QString("mine!"));
  t.setPageSize(r->getPaperSize());
  t.setOrientation(r->getOrientation());
  t.setWH(r->pageWidth(), r->pageHeight());
  if ( !t.exec() ) return;
  if (t.getPageSize() != PAGE_USERDEFINE)
    r->setSizeAndOrientation(t.getPageSize(), t.getOrientation());
  else
    r->setPaperSize(t.getWidth(), t.getHeight());
}

void ApplicationWindow::OpenRingSubmenu(void)
{
  RingDialog t(this, QString("mine!"));
  if ( !t.exec() ) return;
  int w = sv->viewport()->width();	
  int h = sv->viewport()->height();
  w /= 2; h /= 2;
  int x, y;
  sv->viewportToContents(w, h, x, y);	
  r->addRing(t.getRing(), x, y, t.getFile());
}

void ApplicationWindow::OpenSymbolSubmenu(void)
{
  SymbolDialog t(this, QString("mine!"));
  if ( !t.exec() ) return;
  int w = sv->viewport()->width();	
  int h = sv->viewport()->height();
  w /= 2; h /= 2;
  int x, y;
  sv->viewportToContents(w, h, x, y);	
  r->addSymbol(t.getSymbol(), x, y);
}

void ApplicationWindow::showInfo() {
  r->showInfo();
}

void ApplicationWindow::formatOptions() {
  qWarning("Opening format options window - in Render2D widget");
}
