#include <qpainter.h>
#include <qprinter.h>
#include <qcolor.h>
#include <qstring.h>
#include <math.h>
#include <qlist.h>
#include <qtextstream.h>
#include <iostream.h>

#include "render2d.h"
#include "defs.h"
#include "qmath.h"
#include "dpoint.h"
#include "customObject.h"

// defined in main.cpp
extern QList<QPixmap> DSymbolTable;

customObject::customObject()
{
	currentColor = QColor(0,0,0);
	thickness = SINGLE_BOND;
	dashed = false;
	lw = 1;
	hl = false;
	npoints = 0;
}

customObject::customObject(customObject *dup)
{
  currentColor = dup->currentColor;
  thickness = dup->thickness;
  label = dup->label;
  type = dup->type;
  font = dup->font;
  lw = dup->lw;
  dashed = dup->dashed;
  npoints = 0;
}

void customObject::setType(int x)
{
	type = x;
}

void customObject::setFont(QFont x)
{
	font = x;
}

void customObject::setThickness(int x)
{
	if (x > TRIPLE_BOND) return;
	thickness = x;
}

int customObject::Thickness(void)
{
	return thickness;
}

void customObject::setLabel(QString l)
{
	label = l;
}

QString customObject::Label(void)
{
	return label;
}

void customObject::Highlight(void)
{
	hl = true;
	currentColor = QColor(255,0,0);
}

void customObject::Unhighlight(void)
{
	hl = false;
	currentColor = color;
}

void customObject::addPoint(DPoint *l)
{
	if (npoints == 0) { p1 = l; }
	if (npoints == 1) { p2 = l; }
	npoints++;
}

DPoint * customObject::getPoint1(void)
{
	return p1;
}

DPoint * customObject::getPoint2(void)
{
	return p2;
}

int customObject::withinBounds(int l, int t, int r, int b)
{
	if (p1->x() < l) return false;
	if (p1->x() > r) return false;
	if (p1->y() > t) return false;
	if (p1->y() < b) return false;
	if ((type == MLINE) || (type == MARROW) || (type == MUPLINE) ||
		(type == MDOWNLINE) || (type == MBRACKET) ) {
		if (p2->x() < l) return false;
		if (p2->x() > r) return false;
		if (p2->y() > t) return false;
		if (p2->y() < b) return false;
	}
	return true;
}

int customObject::onText(DPoint n)
{
	QPixmap l(100,100);
	QPainter p(&l);
	p.setFont(font);
	QFontMetrics fm = p.fontMetrics();
	QRect m = fm.boundingRect(label);
	QRect r(p1->xint() - 4, p1->yint() - m.height()/2, 
				  m.width(), m.height());

	return r.contains(n.toQPoint(), true);
}

void Render2D::Render(int dest)
{        
  QMath c;
  QPainter p;
  customObject *o;
  if (dest == 0)
    p.begin(this);
  if (dest == 1)
    p.begin(rendermap);
  if (dest == 2)
    p.begin(printer);
  for (o = objectList.first(); o != 0; o = objectList.next()) {
    p.setPen(QPen(o->currentColor, o->lw - 1));
    double myangle;
    int x1, y1, x2, y2;
    if (o->type == MLINE) {
      if (o->thickness == SINGLE_BOND) {
	if (o->Dash()) p.setPen(QPen(o->currentColor, o->lw - 1, DashLine));
	p.drawLine( o->p1->toQPoint(), o->p2->toQPoint() );
	p.setPen(QPen(o->currentColor, o->lw - 1));
      }
      if (o->thickness == TRIPLE_BOND)
	p.drawLine( o->p1->toQPoint(), o->p2->toQPoint() );
      if (o->thickness == SINGLE_BOND) continue;
      myangle = c.getAngle(*(o->p1), *(o->p2));
      myangle += 90.0;
      myangle /= MOL_ARAD;
      double offs = 1.0;
      if (o->lw > 3) offs = 1.5;
      x1 = (int)(cos(myangle) * 3.0 * offs);
      y1 = (int)(sin(myangle) * 3.0 * offs);
      QPoint tn(x1, y1);
      x2 = (int)(cos(myangle) * 2.0 * offs);
      y2 = (int)(sin(myangle) * 2.0 * offs);
      QPoint tp(x2, y2);
      if (o->thickness == DOUBLE_BOND) {
	QPoint t1(o->p1->xint()+tp.x(), o->p1->yint()+tp.y()); 
	QPoint t2(o->p2->xint()+tp.x(), o->p2->yint()+tp.y()); 
	p.drawLine(t1, t2);
	QPoint t3(o->p1->xint()-tp.x(), o->p1->yint()-tp.y()); 
	QPoint t4(o->p2->xint()-tp.x(), o->p2->yint()-tp.y()); 
	if (o->Dash()) p.setPen(QPen(o->currentColor, o->lw - 1, DashLine));
	p.drawLine(t3, t4);
	p.setPen(QPen(o->currentColor, o->lw - 1));
      }
      if (o->thickness == TRIPLE_BOND) {
	QPoint t1(o->p1->xint()+tn.x(), o->p1->yint()+tn.y()); 
	QPoint t2(o->p2->xint()+tn.x(), o->p2->yint()+tn.y()); 
	p.drawLine(t1, t2);
	QPoint t3(o->p1->xint()-tn.x(), o->p1->yint()-tn.y()); 
	QPoint t4(o->p2->xint()-tn.x(), o->p2->yint()-tn.y()); 
	if (o->Dash()) p.setPen(QPen(o->currentColor, o->lw - 1, DashLine));
	p.drawLine(t3, t4);
	p.setPen(QPen(o->currentColor, o->lw - 1));
      }
    }
	
    if (o->type == MUPLINE) {
      double ang = getAngle(o->p1, o->p2);
      double dx = o->p2->x() - o->p1->x();
      double dy = o->p2->y() - o->p1->y();
      double newang1 = ang + 8.0;
      double newang2 = ang - 8.0;
      double newlen = sqrt(dx*dx + dy*dy);
      double mysign = 1.0;
      dx = mysign * newlen * cos(newang1 / MOL_ARAD);
      dy = mysign * newlen * sin(newang1 / MOL_ARAD);
      QPoint t1( (int)(o->p1->x() + dx), (int)(o->p1->y() + dy) );
      dx = mysign * newlen * cos(newang2 / MOL_ARAD);
      dy = mysign * newlen * sin(newang2 / MOL_ARAD);
      QPoint t2( (int)(o->p1->x() + dx), (int)(o->p1->y() + dy) );

      p.setBrush(QBrush(o->currentColor));
      QPointArray triangle(3);
      triangle.setPoint(0, t1);
      triangle.setPoint(1, t2);
      triangle.setPoint(2, o->p1->xint(), o->p1->yint());
      p.drawPolygon(triangle);
    }

    if (o->type == MDOWNLINE) {
      double ang = getAngle(o->p1, o->p2);
      double dx = o->p2->x() - o->p1->x();
      double dy = o->p2->y() - o->p1->y();
      double newang1 = ang + 8.0;
      double newang2 = ang - 8.0;
      double newlen = sqrt(dx*dx + dy*dy);

      for (double scalefactor = 1.0; scalefactor < newlen; scalefactor = 
	     scalefactor + 4.0) {
	dx = scalefactor * cos(newang1 / MOL_ARAD);
	dy = scalefactor * sin(newang1 / MOL_ARAD);
	QPoint t1( (int)(o->p1->x() + dx), (int)(o->p1->y() + (int)dy) );
	dx = scalefactor * cos(newang2 / MOL_ARAD);
	dy = scalefactor * sin(newang2 / MOL_ARAD);
	QPoint t2( (int)(o->p1->x() + dx), (int)(o->p1->y() + dy) );
	p.drawLine(t1, t2);
      }
      dx = cos(newang1 / MOL_ARAD);
      dy = sin(newang1 / MOL_ARAD);
      QPoint t1( (int)(o->p1->x() + dx), (int)(o->p1->y() + dy) );
      dx = cos(newang2 / MOL_ARAD);
      dy = sin(newang2 / MOL_ARAD);
      QPoint t2( (int)(o->p1->x() + dx), (int)(o->p1->y() + dy) );
      p.drawLine(t1, t2);
    }

    if (o->type == MARROW) {
      p.drawLine(o->p1->toQPoint(), o->p2->toQPoint());
      // Draw arrowhead
      double dx = o->p1->x() - o->p2->x();
      double dy = o->p1->y() - o->p2->y();
      int mysign = 1;
      if (dx < 0.0) mysign = -1;
      double ang = atan(dy/dx)*MOL_ARAD;
      if (ang < 0.0) ang *= -1.0;

      if (o->p1->x() > o->p2->x() && o->p1->y() > o->p2->y()) ang = 0.0 + ang;
      if (o->p1->x() < o->p2->x() && o->p1->y() > o->p2->y()) ang = 180.0 - ang;
      if (o->p1->x() < o->p2->x() && o->p1->y() < o->p2->y()) ang = 180.0 + ang;
      if (o->p1->x() > o->p2->x() && o->p1->y() < o->p2->y()) ang = 360.0 - ang;
      if (dx == 0.0) {
	if (dy < 0.0)
	  ang = 270.0;
	else
	  ang = 90.0;
      }
      if (dy == 0.0) {
	if (dx < 0.0)
	  ang = 180.0;
	else
	  ang = 0.0;
      }

      double newang1 = ang + 30.0;
      double newang2 = ang - 30.0;

      QPoint a1(o->p2->x() + (cos(newang1/MOL_ARAD) * 10.0),
		o->p2->y() + (sin(newang1/MOL_ARAD) * 10.0));
      QPoint a2(o->p2->x() + (cos(newang2/MOL_ARAD) * 10.0),
		o->p2->y() + (sin(newang2/MOL_ARAD) * 10.0));
      p.drawLine(o->p2->toQPoint(),a1);
      p.drawLine(o->p2->toQPoint(),a2);
    }
	
    // If type is MTEXT or MLABEL, draw the text in QString label
    if (o->type == MTEXT) {
      int yoff = 0, voff, off, newsize;
      p.setFont(o->font);
      QFontMetrics fm = p.fontMetrics();
      int xpos = (int)o->p1->x() - 4;
      int init_xpos = xpos;
      int ypos = (int)o->p1->y() + 6;
      QTextStream t(&(o->label), IO_ReadOnly);
      switch (o->font.pointSize()) {
      case 8:
	newsize = 8; off = 2; voff = -1;
	break;
      case 10:
	newsize = 8; off = 2; voff = -3;
	break;
      case 12:
	newsize = 8; off = 2; voff = -4;
	break;
      case 14:
	newsize = 10; off = 2; voff = -4;
	break;
      case 18:
	newsize = 12; off = 3; voff = -5;
	break;
      case 24:
	newsize = 12; off = 3; voff = -10;
	break;
      default:
	newsize = 8; off = 4; voff = -2;
	break;
      }
      QFont newfont(o->font.family(), newsize);

      do {
	QString l = t.readLine();
	int superflag = 0, subflag = 0;
	if (l.contains(QString("<super>"), FALSE) > 0) {
	  superflag = 1;
	  ypos += 1;
	  if (newsize == 12) ypos += 1;
	}
	if (l.contains(QString("<sub>"), FALSE) > 0) subflag = 1;
	    
	xpos = init_xpos;

	for (int i = 0; i < l.length(); i++) {
	  QChar n1 = l.at(i);
	  yoff = 0;
	  // handle super/sub
	  if (n1 == "<") {
	    QString s1;
	    int lflag = 0;
	    if (l.at(i+3).upper() == "P") {
	      yoff = voff;
	      i += 7;
	      lflag = 1;
	    } else {
	      yoff = off;
	      i += 5;
	    }
	    do {
	      s1.append(l.at(i));
	      i++;
	      n1 = l.at(i);
	    } while (n1 != "<");
	    p.setFont(newfont);
	    p.drawText(xpos, ypos + yoff, s1);
	    xpos += fm.width(s1);
	    if (lflag == 1)
	      i += 7;
	    else
	      i += 5;
	  } else {
	    p.setFont(o->font);
	    p.drawText(xpos, ypos, QString(n1));
	    xpos += fm.width(n1);
	  }
	}
	    
	if (subflag > 0) 
	  ypos += 1;
	if ( (newsize == 12) && (subflag > 0) ) ypos += 1;
	ypos += o->font.pointSize();
      } while (!t.atEnd());
    }

    if (o->type == MBRACKET) {
      QPoint b1(o->p1->xint(), o->p2->yint());
      QPoint b2(o->p2->xint(), o->p1->yint());
      int s = 1;
      if (o->p2->xint() < o->p1->xint()) s = -1;
      p.drawLine(o->p1->toQPoint(), b1);
      p.drawLine(b2, o->p2->toQPoint());
      p.drawLine(o->p1->toQPoint(), QPoint(o->p1->x() + s*5, o->p1->y()) );
      p.drawLine(QPoint(b2.x() - s*5, b2.y()), b2);
      p.drawLine(b1, QPoint(b1.x() + s*5, b1.y()) );
      p.drawLine(QPoint(o->p2->x() - s*5, o->p2->y()), o->p2->toQPoint() );
    }
    if (o->type == MSYMBOL) {
      p.drawPixmap(o->p1->toQPoint(), *(DSymbolTable.at(o->symbol)));
      if (o->hl == true) {
	p.setPen(QColor(255,0,0));
	p.drawEllipse(o->p1->xint() - 3, o->p1->yint(), 12, 12);
	p.setPen(o->currentColor);
      }
    }
  } // for(...)

  // Draw labels
  for (o = objectList.first(); o != 0; o = objectList.next()) {
    // If type is MTEXT or MLABEL, draw the text in QString label
    if (o->type == MLABEL) {
      p.setPen(o->currentColor);
      p.setFont(o->font);
      QFontMetrics fm = p.fontMetrics();
      QRect m = fm.boundingRect(o->label);
      int w = m.width();
      int h = m.height();
      int xi = (int)o->p1->xint() - 4;
      int yi = (int)o->p1->yint() + (h/2);
      p.fillRect(o->p1->xint() - 6, o->p1->yint() - (h/2) - 1, w + 4, h + 2, 
		 QBrush(QColor(255,255,255)));
      int offset = 0, firstletter = false, lastletter = false;
      for (int z = 0; z < o->label.length(); z++) {
	QChar n1 = o->label.at(z);
	if (z < (o->label.length() - 1)) {
	  if (o->label.at(z+1).isSpace()) lastletter = true;
	}
	if (z == (o->label.length() - 1)) lastletter = true;
	if ( n1.isSpace() ) firstletter = false;
	if ( n1.isLetter() ) firstletter = true;
	offset = 0;
	if (n1.isNumber() && firstletter)
	  offset = fm.height() / 3;
	if ((n1 == '+') && firstletter && lastletter)
	  offset = -(fm.height() / 3);
	if ((n1 == '-') && firstletter && lastletter)
	  offset = -(fm.height() / 3);
	p.drawText( xi, yi + offset, QString(n1) );
	xi += fm.width(n1);
	lastletter = false;
      }
    }
  }  // for(...)
}

void Render2D::Render(customObject *o, int dest)
{        
	QMath c;
  QPainter p;
	if (dest == 0)
		p.begin(this);
	if (dest == 1)
		p.begin(rendermap);
	p.setPen(QPen(o->currentColor, o->lw - 1));
	double myangle;
	int x1, y1, x2, y2;
	if (o->type == MLINE) {
		if (o->thickness == SINGLE_BOND) {
			if (o->Dash()) p.setPen(QPen(o->currentColor, o->lw - 1, DashLine));
		  p.drawLine( o->p1->toQPoint(), o->p2->toQPoint() );
			p.setPen(QPen(o->currentColor, o->lw - 1));
		}
		if (o->thickness == TRIPLE_BOND)
		  p.drawLine( o->p1->toQPoint(), o->p2->toQPoint() );
		if (o->thickness == SINGLE_BOND) return;
		myangle = c.getAngle(*(o->p1), *(o->p2));
		myangle += 90.0;
		myangle /= MOL_ARAD;
		double offs = 1.0;
		if (o->lw > 3) offs = 1.5;
		x1 = (int)(cos(myangle) * 3.0 * offs);
		y1 = (int)(sin(myangle) * 3.0 * offs);
		QPoint tn(x1, y1);
		x2 = (int)(cos(myangle) * 2.0 * offs);
		y2 = (int)(sin(myangle) * 2.0 * offs);
		QPoint tp(x2, y2);
		if (o->thickness == DOUBLE_BOND) {
			QPoint t1(o->p1->xint()+tp.x(), o->p1->yint()+tp.y()); 
			QPoint t2(o->p2->xint()+tp.x(), o->p2->yint()+tp.y()); 
			p.drawLine(t1, t2);
			QPoint t3(o->p1->xint()-tp.x(), o->p1->yint()-tp.y()); 
			QPoint t4(o->p2->xint()-tp.x(), o->p2->yint()-tp.y()); 
			if (o->Dash()) p.setPen(QPen(o->currentColor, o->lw - 1, DashLine));
			p.drawLine(t3, t4);
			p.setPen(QPen(o->currentColor, o->lw - 1));
		}
		if (o->thickness == TRIPLE_BOND) {
			QPoint t1(o->p1->xint()+tn.x(), o->p1->yint()+tn.y()); 
			QPoint t2(o->p2->xint()+tn.x(), o->p2->yint()+tn.y()); 
			p.drawLine(t1, t2);
			QPoint t3(o->p1->xint()-tn.x(), o->p1->yint()-tn.y()); 
			QPoint t4(o->p2->xint()-tn.x(), o->p2->yint()-tn.y()); 
			if (o->Dash()) p.setPen(QPen(o->currentColor, o->lw - 1, DashLine));
			p.drawLine(t3, t4);
			p.setPen(QPen(o->currentColor, o->lw - 1));
		}
	}
	
	if (o->type == MUPLINE) {
		double ang = getAngle(o->p1, o->p2);
		double dx = o->p2->x() - o->p1->x();
		double dy = o->p2->y() - o->p1->y();
		double newang1 = ang + 8.0;
		double newang2 = ang - 8.0;
		double newlen = sqrt(dx*dx + dy*dy);
		double mysign = 1.0;
		dx = mysign * newlen * cos(newang1 / MOL_ARAD);
		dy = mysign * newlen * sin(newang1 / MOL_ARAD);
		QPoint t1( (int)(o->p1->x() + dx), (int)(o->p1->y() + dy) );
		dx = mysign * newlen * cos(newang2 / MOL_ARAD);
		dy = mysign * newlen * sin(newang2 / MOL_ARAD);
		QPoint t2( (int)(o->p1->x() + dx), (int)(o->p1->y() + dy) );

		p.setBrush(QBrush(o->currentColor));
		QPointArray triangle(3);
		triangle.setPoint(0, t1);
		triangle.setPoint(1, t2);
		triangle.setPoint(2, o->p1->xint(), o->p1->yint());
		p.drawPolygon(triangle);
	}

	if (o->type == MDOWNLINE) {
		double ang = getAngle(o->p1, o->p2);
		double dx = o->p2->x() - o->p1->x();
		double dy = o->p2->y() - o->p1->y();
		double newang1 = ang + 8.0;
		double newang2 = ang - 8.0;
		double newlen = sqrt(dx*dx + dy*dy);

		for (double scalefactor = 1.0; scalefactor < newlen; scalefactor = 
			scalefactor + 4.0) {
			dx = scalefactor * cos(newang1 / MOL_ARAD);
			dy = scalefactor * sin(newang1 / MOL_ARAD);
			QPoint t1( (int)(o->p1->x() + dx), (int)(o->p1->y() + (int)dy) );
			dx = scalefactor * cos(newang2 / MOL_ARAD);
			dy = scalefactor * sin(newang2 / MOL_ARAD);
			QPoint t2( (int)(o->p1->x() + dx), (int)(o->p1->y() + dy) );
			p.drawLine(t1, t2);
		}
		dx = cos(newang1 / MOL_ARAD);
		dy = sin(newang1 / MOL_ARAD);
		QPoint t1( (int)(o->p1->x() + dx), (int)(o->p1->y() + dy) );
		dx = cos(newang2 / MOL_ARAD);
		dy = sin(newang2 / MOL_ARAD);
		QPoint t2( (int)(o->p1->x() + dx), (int)(o->p1->y() + dy) );
		p.drawLine(t1, t2);
	}

	if (o->type == MARROW) {
	  p.drawLine(o->p1->toQPoint(), o->p2->toQPoint());
		// Draw arrowhead
		double dx = o->p1->x() - o->p2->x();
		double dy = o->p1->y() - o->p2->y();
		int mysign = 1;
		if (dx < 0.0) mysign = -1;
		double ang = atan(dy/dx)*MOL_ARAD;
		if (ang < 0.0) ang *= -1.0;

	if (o->p1->x() > o->p2->x() && o->p1->y() > o->p2->y()) ang = 0.0 + ang;
	if (o->p1->x() < o->p2->x() && o->p1->y() > o->p2->y()) ang = 180.0 - ang;
	if (o->p1->x() < o->p2->x() && o->p1->y() < o->p2->y()) ang = 180.0 + ang;
	if (o->p1->x() > o->p2->x() && o->p1->y() < o->p2->y()) ang = 360.0 - ang;
		if (dx == 0.0) {
			if (dy < 0.0)
				ang = 270.0;
			else
				ang = 90.0;
		}
		if (dy == 0.0) {
			if (dx < 0.0)
				ang = 180.0;
			else
				ang = 0.0;
		}

		double newang1 = ang + 30.0;
		double newang2 = ang - 30.0;

		QPoint a1(o->p2->x() + (cos(newang1/MOL_ARAD) * 10.0),
							o->p2->y() + (sin(newang1/MOL_ARAD) * 10.0));
		QPoint a2(o->p2->x() + (cos(newang2/MOL_ARAD) * 10.0),
							o->p2->y() + (sin(newang2/MOL_ARAD) * 10.0));
		p.drawLine(o->p2->toQPoint(),a1);
		p.drawLine(o->p2->toQPoint(),a2);
	}
	
	if (o->type == MBRACKET) {
		QPoint b1(o->p1->xint(), o->p2->yint());
		QPoint b2(o->p2->xint(), o->p1->yint());
		int s = 1;
		if (o->p2->xint() < o->p1->xint()) s = -1;
		p.drawLine(o->p1->toQPoint(), b1);
		p.drawLine(b2, o->p2->toQPoint());
		p.drawLine(o->p1->toQPoint(), QPoint(o->p1->x() + s*5, o->p1->y()) );
		p.drawLine(QPoint(b2.x() - s*5, b2.y()), b2);
		p.drawLine(b1, QPoint(b1.x() + s*5, b1.y()) );
		p.drawLine(QPoint(o->p2->x() - s*5, o->p2->y()), o->p2->toQPoint() );
	}
	if (o->type == MSYMBOL) {
		p.drawPixmap(o->p1->toQPoint(), *(DSymbolTable.at(o->symbol)));
		if (o->hl == true) {
			p.setPen(QColor(255,0,0));
			p.drawEllipse(o->p1->xint() - 3, o->p1->yint(), 12, 12);
			p.setPen(o->currentColor);
		}
	}

	// draw MTEXT here (copy above)

	// If type is MLABEL, draw the text in QString label
	if (o->type == MLABEL) {
		p.setPen(o->currentColor);
		p.setFont(o->font);
		QFontMetrics fm = p.fontMetrics();
		QRect m = fm.boundingRect(o->label);
		int w = m.width();
		int h = m.height();
		int xi = (int)o->p1->xint() - 4;
		int yi = (int)o->p1->yint() + (h/2);
		p.fillRect(o->p1->xint() - 6, o->p1->yint() - (h/2) - 1, w + 4, h + 2, 
			QBrush(QColor(255,255,255)));
		int offset = 0, firstletter = false, lastletter = false;
		for (int z = 0; z < o->label.length(); z++) {
			QChar n1 = o->label.at(z);
      if (z < (o->label.length() - 1)) {
        if (o->label.at(z+1).isSpace()) lastletter = true;
      }
      if (z == (o->label.length() - 1)) lastletter = true;
			if ( n1.isSpace() ) firstletter = false;
			if ( n1.isLetter() ) firstletter = true;
      offset = 0;
			if (n1.isNumber() && firstletter)
				offset = fm.height() / 3;
			if ((n1 == '+') && firstletter && lastletter)
				offset = -(fm.height() / 3);
			if ((n1 == '-') && firstletter && lastletter)
				offset = -(fm.height() / 3);
			p.drawText( xi, yi + offset, QString(n1) );
			xi += fm.width(n1);
      lastletter = false;
		}
	}
}
