// manual.cpp - online help.

#include <qlayout.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qmultilineedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qscrollview.h>
#include <qwhatsthis.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qtextview.h>
#include <qbrush.h>
#include <qlabel.h>
#include <qcombobox.h>

#include "application.h"
#include "defs.h"
#include "config.h"
#include "manual.h"

// Manual pages.  Rich text format.
static const char *manpages[] = {
"<center>"
"<b>Welcome to XDrawChem!</b><br>"
"Bryan Herger<br>"
"herger@chemistry.gatech.edu<br>"
"</center>"
"<p><b>Navigating the manual:</b>"
"<p>At the bottom of the window there are three buttons.  These are:"
"<p>Close - close the manual.<br>"
"Prev - go to the previous manual page.<br>"
"Next - go to the next manual page."
"<p>The number between Prev and Next tells what page you are viewing.<br>"
"You can jump to any page by selecting the page title from "
"the combo box at the top."
"<p>A note on keyboard shortcuts:  key names are abbreviated.  For example, "
"\"Ctrl+O\" means hold the control (Ctrl) key and press O.",

"<b>Overview</b>"
"<p>The application window consists of five major elements:<br>"
"The menu bar at the top;<br>"
"The file/edit toolbar below the menu bar;<br>"
"The drawing toolbar, on the left;<br>"
"The status bar, along the bottom;<br>"
"and the drawing area, the large area in the middle."
"<p>The File/Edit toolbar shows commonly used items which are also found "
"under the File and Edit menus."
"<p>The Drawing toolbar shows available drawing tools."
"<p>To get help on a toolbar button, click on the "
"<img source=\"whatsthisicon\"><b>What's This?</b> "
"button, then the button you want help with."
"<p>The status bar shows error messages, information, and usually a tip "
"on how to use the current drawing tool."
"<p>The following pages describe all the buttons and menu options.",

"<b>The File Menu and Toolbar Buttons</b>"
"<p><img source=\"fileopenicon\"><b>Open File</b> (Ctrl+O) Open a file. "
"A dialog will appear asking you to select a file."
"<p><img source=\"filesaveicon\"><b>Save File</b> (Ctrl+S) Save the current "
"file.  If it is a new file, you will be prompted for a directory and "
"filename to save to."
"<p><b>Save as...</b> Save the current file with a different name.<br>"
"NOTE: You must explicitly specify an extension (*.xdc or *.mol) or "
"XChemDraw will not save the drawing."
"<p><img source=\"savepicicon\"><b>Save image</b> Saves the current "
"drawing as a Windows bitmap image (*.bmp). If something is selected, only the "
"selection will be converted to an image."
"<p><b>Page setup...</b> Choose paper size and orientation "
"for printing."
"<p><img source=\"fileprinticon\"><b>Print</b> (Ctrl+P) Prints the "
"current drawing.  A dialog will appear, allowing you to select destination "
"printer or file, paper size, orientation, etc."
"<p><b>Close</b> (Ctrl+W) Closes this window."
"<p><b>Quit</b> (Ctrl+Q) Closes all windows.",

"<b>The Edit menu and Toolbar buttons</b>"
"<p>Important note:  you can only cut, copy, and paste between XDrawChem "
"windows.  To import drawings into other applications, use "
"<img source=\"savepicicon\">Save image to create an image, "
"or save as an MDL Molfile."
"<p><img source=\"cuttoolicon\"><b>Cut</b> (Ctrl+X) Cut a selection from this drawing."
"<p><img source=\"copytoolicon\"><b>Copy</b> (Ctrl+C) Copy a selection from this "
"drawing."
"<p><img source=\"pastetoolicon\"><b>Paste</b> (Ctrl+V) Paste something that was cut or "
"copied."
"<p><b>Clear</b> (Del) Erases the current selection permanently."
"<p><b>Select all</b> (Ctrl+A) Select all objects in the drawing."
"<p>Edit functions not on the menu:<br>"
"<b>Rotate</b> To rotate a selection, right-click (cursor will change) "
" and drag.<br>"
"<b>Resize</b> To resize or stretch a selection, left-click on the resize "
"handle (cursor will change) and drag.",

"<b>The Drawing Toolbar</b>"
"<p><img source=\"selecttoolicon\"><b>Select tool</b> Select objects in the "
"drawing.  Click on individual objects to select, or left-click and drag to select "
"multiple objects.<br>"
"After selecting objects, you can move them by dragging with the left mouse "
"button, or roatet them by dragging with the right mouse button."
"<p><img source=\"erasetoolicon\"><b>Erase tool</b> Erase individual objects. "
"Move mouse over object to highlight and left-click to delete."
"<p><img source=\"linetoolicon\"><b>Line tool</b> Draw lines (bonds).  This "
"tool will automatically snap endpoints of lines together. Start or finish "
"drawing near the endpoint of another line.  The highlight square will "
"indicate which line it will snap to. Create double and triple bonds by "
"drawing over an existing line."
"<p><img source=\"uplinetoolicon\"><b>Up Line tool</b> Draw up-facing "
"stereospecific bonds."
"<p><img source=\"downlinetoolicon\"><b>Down Line tool</b> Draw down-facing "
"stereospecific bonds."
"<p><img source=\"arrowtoolicon\"><b>Arrow tool</b> Draw arrows.  Arrows "
"cannot snap to any other object."
"<p><img source=\"brackettoolicon\"><b>Bracket tool</b> Draw square brackets. "
"Left-click and drag to draw brackets."
"<p><img source=\"texttoolicon\"><b>Text tool</b> Add text.  Click on the "
"endpoint of a line or an intersection to label it.<br>"
"Note:  XDrawChem will automatically super- and subscript text."
"<p><img source=\"ringtoolicon\"><b>Ring tool</b> Add rings. A dialog box "
"will appear with rings XChemDraw is able to draw."
"<p><img source=\"symboltoolicon\"><b>Symbol tool</b> Add symbols. A dialog "
"box will appear with symbols XChemDraw is able to draw.",

"<b>Formatting</b>"
"<p><b>Fixed line length and angles</b> To set fixed line length and angles, "
"check the 'Fixed length/angles' option under the Format menu.  The fixed "
"length may be changed by "
"selecting a different entry under the 'Fixed Length' submenu under the"
"Format menu.  The fixed angle may be changed by selecting another angle from "
"the Fixed Angle submenu under the Format menu.<br>"
"Note:  1/4 inch and 1/2 inch designations have not been thoroughly tested. "
"Changing line length does not affect lines already drawn."
"<p><b>Changing line thickness</b> To change line thickness, change the setting "
"'Line thickness' under the Format menu.  Line thickness is in pixels."
"<p><b>Changing font and font size</b> To change font parameters, select new font "
"style and size under the Format menu."
"<p><b>Alignment</b> To align objects, select the objects you wish to line up. "
"Under the Alignment submenu of the Format menu, select Align Horizontal - "
"Center to align along a horizontal line, or select Align Vertical - Center to "
"align on a vertical line.",

"<b>Quick tour: Creating new files, opening files</b>"
"<p>XDrawChem starts with a blank drawing area if no file is specified on the "
"command line.  Later pages describe page setup and drawing."
"<p>To open a new application window with blank drawing area, press Ctrl+N "
"or select New under the File menu."
"<p><b>Loading files:</b>  you may specify the file name on the command "
"line:  xdrawchem [filename]<br>"
"Or, go to the Open File dialog by any of the following: "
"click on the <img source=\"fileopenicon\">file open icon on the top toolbar, "
"select Open under the File menu, or press Ctrl+O."
"<p>Opening a file will replace any drawing in the current window!  To open "
"multiple drawings, first open windows by selecting New under the File menu "
"or press Ctrl+N.  Then open a drawing in each window.",

"<b>Quick tour: page setup</b>"
"<p>First, choose a page size and orientation.  Go to Page Setup... under the"
" File menu."
"<p>Which options you choose depends on what you want to do.<br>"
"<b>Create a full-page drawing:</b> Select a page size and orientation and "
"click OK.<br>"
"<b>Create an image to insert into a document:</b> Select User defined as the "
"page size.  The width and height text fields will turn color.  Type in a width "
"and height for your drawing.  TIP:  make a 100-dpi (dots per inch) image. "
"For example, make a 6\" wide by 8\" tall image by entering 600 as the width "
"and 800 as the height.  This will allow other applications to correctly size "
"your image."
"<p>You can later change page size and orientation as needed.",

"<b>Quick tour: drawing (lines/bonds)</b>"
"<p>Draw lines with the <img source=\"linetoolicon\">Line tool. Left-click to start a line, drag to where "
"you want the line to end, and release the button to finish drawing.  To "
"connect lines, move the mouse near the endpoint of another line.  A square "
"will highlight the endpoint of the existing line.  If you click and draw "
"when an endpoint is highlighted, the line you draw will be attached to that "
"point.  To attach the end of the new line to a point, move the mouse near the "
"endpoint of the line you want to attach to.  A square will highlight the end "
"of the target point.  Release the button and the line will be attached to that "
"point.<br>"
"If you draw over an existing line, that line will become a double bond. "
"If you draw over an existing double bond, it will become a triple bond."
"<p>Draw stereo bonds with the <img source=\"uplinetoolicon\">Up bond and "
"<img source=\"downlinetoolicon\">Down bond tools.  They work exactly like "
"the Line tool.",

"<b>Quick tour: drawing (text/rings/symbols)</b>"
"<p>Add text by selecting the <img source=\"texttoolicon\">Text tool.  The "
"cursor will change to an I-beam text cursor."
"Click on a point to label the point (e.g., to "
"label a nitrogen atom with N).  "
"A dialog box will appear.  Type in your text and press Enter or click OK.<br>"
"Click anywhere in the drawing to add text.  A different dialog will appear.  "
"You may enter multiple lines of text, and add super- and subscripting.  "
"XDrawChem recognizes these HTML tags: super /super sub /sub (in arrow brackets, of course). "
"Click Preview to see what your text will look like.  "
"To change font style and size, exit the text entry dialog, select the text, "
"and use the Font submenus undr Format."
"<p>Draw rings with the <img source=\"ringtoolicon\">Ring tool.  When you "
"choose this tool, a dialog will appear.  To pick a ring, click on the "
"image button (rings sorted by increasing number of carbons).  The ring will "
"be inserted in the drawing.  The new ring will be selected automatically "
"so you can move it."
"<p>Add symbols with the <img source=\"symboltoolicon\">Symbol tool.  This "
"tool allows you to add charges, radicals, and so on.  When you select this "
"tool, a dialog box will appear showing available symbols.  Choose a symbol "
"by clicking on it and the symbol will appear in your drawing.  You will be "
"able to move it to the appropriate position after adding.",

"<b>Quick tour: Saving files and images</b>"
"<p>If the drawing you are working on is already on disk (its name will appear "
"on the title bar if it has already been saved), you can choose Save under the "
"File menu or press Ctrl+S.  This will save any changes to the drawing to the "
"existing file.<br>"
"To save your drawing to a new file, or change the file format, select "
"Save As... under the File menu.  You must explicitly specify the extension "
"of the file to choose a format:<br>"
"<b>(*.xdc) XDrawChem format:</b> saves all features of the drawing.  However, "
"you will only be able to open this drawing in XDrawChem.<br>"
"<b>(*.mol) MDL Molfile:</b> most applications can read this format.  However, "
"only the structures will be saved.  Features and formatting such as arrows, "
"brackets, and color will not be saved.<br>"
"<b>(*.cml) CML - Chemical Markup Language:</b> XML for chemistry.  Growing support for this, XDrawChem saves in MDL Molfile style.<br>"
"NOTE:  To save a file as XDrawChem, it must end in .xdc, and to save as "
"Molfile, it must end in .mol, and for CML, .cml"
"<p><b>Saving images</b><br>"
"To save the drawing as an image file, select Save image... under the File "
"menu or press the <img source=\"savepicicon\">Save image button on the "
"toolbar.  The image will be saved as a Windows bitmap file (.bmp).  If items "
"on the screen are selected, only those items will be in the image.  Otherwise, "
"the entire drawing is saved as an image.<br>"  
"NOTE: XDrawChem will automatically crop the image."
};

// Show the manual.  Called by Help/Manual or pressing F1.
void ApplicationWindow::Manual()
{
  XManual *m = new XManual;
  m->show();
}

// Manual dialog box.  Start at page 1.  Pages defined above in manpages[]
XManual::XManual(QWidget *parent , const char *name ) : 
  QDialog(parent, name)
{
  resize(400,450);
  setCaption("XDrawChem manual");
  qvb = new QVBox(this, "Manual dialog box");
  qvb->setMinimumSize(400,450);
  qvb->setMargin(3);
    
  pagelist = new QComboBox( false, qvb );
  pagelist->insertItem("Title page");
  pagelist->insertItem("Overview");
  pagelist->insertItem("File Menu/Toolbar");
  pagelist->insertItem("Edit Menu/Toolbar");
  pagelist->insertItem("Drawing Toolbar");
  pagelist->insertItem("Formatting");
  pagelist->insertItem("Tour: new files, opening files");
  pagelist->insertItem("Tour: page setup");
  pagelist->insertItem("Tour: drawing lines and bonds");
  pagelist->insertItem("Tour: drawing text, rings, symbols");
  pagelist->insertItem("Tour: loading and saving");
  connect( pagelist, SIGNAL( activated(int) ), this, SLOT( gotoPage(int) ) );

  pages = new QTextView( qvb );
  pages->setText( manpages[0] );
  QBrush paper( QColor(255,255,255) );
  pages->setPaper( paper );

  QHBox *buttons = new QHBox( qvb );
  buttons->setMargin( 3 );

  bClose = new QPushButton( "&Close", buttons );
  QWidget *w = new QWidget( buttons );
  w->setMaximumHeight( bClose->sizeHint().height() );
  bPrev = new QPushButton( "<< &Prev", buttons );
  bLabel = new QLabel( "Page 1", buttons );
	bLabel->setAlignment(AlignCenter);
  bNext = new QPushButton( "&Next >>", buttons );
  buttons->setMaximumHeight( bNext->sizeHint().height() + 10 );

  bPrev->setEnabled( FALSE );

  connect( bClose, SIGNAL( clicked() ), this, SLOT( close() ) );
  connect( bPrev, SIGNAL( clicked() ), this, SLOT( prevPage() ) );
  connect( bNext, SIGNAL( clicked() ), this, SLOT( nextPage() ) );

  pagenum = 0;
  lastpage = 10;
}

// Turn to previous manual page.
void XManual::prevPage()
{
	if (pagenum == 0) return;
	pagenum--;
	pages->setText(manpages[pagenum]);
	if (pagenum == 0) bPrev->setEnabled(false);
	bNext->setEnabled(true);

	QString q;
	q.setNum(pagenum + 1);
	bLabel->setText( QString("Page ") + q );

	pagelist->setCurrentItem(pagenum);
}

// Turn to next manual page.
void XManual::nextPage()
{
	if (pagenum == lastpage) return;
	pagenum++;
	pages->setText(manpages[pagenum]);
	if (pagenum == lastpage) bNext->setEnabled(false);
	bPrev->setEnabled(true);

	QString q;
	q.setNum(pagenum + 1);
	bLabel->setText( QString("Page ") + q );

	pagelist->setCurrentItem(pagenum);
}

// Goto a specific page by number.
// NOTE: gotoPage assumes first page is 0.
void XManual::gotoPage(int thispage)
{
	if (thispage > lastpage) return;
	pagenum = thispage;
	bNext->setEnabled(true);
	bPrev->setEnabled(true);
	if (pagenum == 0) bPrev->setEnabled(false);
	if (pagenum == lastpage) bNext->setEnabled(false);
	pages->setText( manpages[thispage] );
	QString q;
	q.setNum(thispage+1);
	bLabel->setText( QString("Page ") + q );
}

// Handle resize.  Propagates the event to QVBox* qvb, which handles
// resize for its children.
void XManual::resizeEvent(QResizeEvent *re)
{
	qvb->resize( re->size() );
}
