#include <qwidget.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include <qcolor.h>
#include <qfont.h>
#include <qnamespace.h>
#include <qpixmap.h>
#include <qprinter.h>

#include "pagesetupdialog.h"
#include "defs.h"

PageSetupDialog::PageSetupDialog(QWidget *parent, QString name):
	QDialog(parent, name, TRUE)
{
	setCaption("Page Setup");
	QLabel *l1 = new QLabel(this, "Paper size label");
	l1->setGeometry(10,10,80,30);
	l1->setText("Paper size:");
	psize = new QComboBox(false, this, "Paper size");
	psize->setGeometry(100,10,180,30);
	psize->insertItem("Letter (8.5\" x 11\")");
	psize->insertItem("Legal (8.5\" x 14\")");
	psize->insertItem("A4 (210 mm x 297 mm)");
	psize->insertItem("User define (for images)");
	connect(psize, SIGNAL(activated(int)), this, SLOT(comboChanged(int)));
	QLabel *l2 = new QLabel(this, "Paper size label");
	l2->setGeometry(10,50,80,30);
	l2->setText("Orientation:");
	porient = new QComboBox(false, this, "Paper orientation");
	porient->setGeometry(100,50,180,30);
	porient->insertItem("Portriat");
	porient->insertItem("Landscape");
	QLabel *l3 = new QLabel(this, "Width x height (pixels):");
	l3->setGeometry(10,90,200,30);
	l3->setText("Width x height (pixels):");
	w = new QLineEdit(this);
	w->setGeometry(10,125,60,30);
	w->setEnabled(false);
	QLabel *l4 = new QLabel(this, "x");
	l4->setGeometry(80,125,20,30);
	l4->setText("X");
	h = new QLineEdit(this);
	h->setGeometry(110,125,60,30);
	h->setEnabled(false);
	QPushButton *ok, *dismiss;
  ok = new QPushButton("OK", this);
  ok->setGeometry(50,170,80,30);
  ok->setPalette(QPalette(lightGray));
  connect(ok, SIGNAL(clicked()), SLOT(accept()) );
  dismiss = new QPushButton("Cancel", this);
  dismiss->setGeometry(140,170,80,30);
  dismiss->setPalette(QPalette(lightGray));
  connect(dismiss, SIGNAL(clicked()), SLOT(reject()) );
}

void PageSetupDialog::setPageSize(int n)
{
	if (n == PAGE_LETTER)
		psize->setCurrentItem(0);
	if (n == PAGE_LEGAL)
		psize->setCurrentItem(1);
	if (n == PAGE_A4)
		psize->setCurrentItem(2);
	if (n == PAGE_USERDEFINE) {
		psize->setCurrentItem(3);
		w->setEnabled(true);
		h->setEnabled(true);
	}
}

int PageSetupDialog::getPageSize(void)
{
	if (psize->currentItem() == 0)
		return PAGE_LETTER;
	if (psize->currentItem() == 1)
		return PAGE_LEGAL;
	if (psize->currentItem() == 2)
		return PAGE_A4;
	if (psize->currentItem() == 3)
		return PAGE_USERDEFINE;
}

void PageSetupDialog::setOrientation(int n)
{
	if (n == PAGE_PORTRAIT)
		porient->setCurrentItem(0);
	if (n == PAGE_LANDSCAPE)
		porient->setCurrentItem(1);
}

int PageSetupDialog::getOrientation(void)
{
	if (porient->currentItem() == 0)
		return PAGE_PORTRAIT;
	if (porient->currentItem() == 1)
		return PAGE_LANDSCAPE;
}

void PageSetupDialog::setWH(int w1, int h1)
{
	QString w2, h2;
	w2.setNum(w1);
	h2.setNum(h1);
	w->setText(w2);
	h->setText(h2);
}

void PageSetupDialog::comboChanged(int ww)
{
	if (ww == 3) {
		w->setEnabled(true);
		h->setEnabled(true);
	} else {
		w->setEnabled(false);
		h->setEnabled(false);
	}
		
}
