// processing.cpp -- stuff under Tools and Format menus.
// clean up internal data structures, etc.
// Auto Format is in the file autoformat.cpp (but you knew that)

#include <iostream.h>
#include <qpainter.h>
#include <qimage.h>
#include <qevent.h>
#include <qlist.h>
#include <math.h>
#include <qbitmap.h>
#include <qcursor.h>
#include <qdialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpointarray.h>
#include <qprinter.h>
#include <qpaintdevicemetrics.h>
#include <qnamespace.h>
#include <qmessagebox.h>
#include <qstring.h>
#include <qfont.h>
#include <qfontmetrics.h>

#include "defs.h"
#include "customObject.h"
#include "dpoint.h"
#include "render2d.h"
#include "data.h"
#include "groupdata.h"

// Cleanup(void): cleans up internal data structure QList pointList.
// This is necessary because the Erase tool deletes the object but not the
// points because other objects may be attached to these points.  Therefore,
// Cleanup() checks to make sure objects are connected to the points, and if
// not, it deletes the points permanently.
void Render2D::Cleanup(void)
{
  int flag = false;

  // to avoid trouble later...
  deselectAll();

  DPoint *tmppt;
  customObject *tmpobj;
  for (tmppt = pointList.first(); tmppt != 0; tmppt = pointList.next()) {
    flag = false;
    for (tmpobj = objectList.first(); tmpobj!=0; tmpobj = objectList.next()) {
      if (tmppt == tmpobj->getPoint1()) { flag = true; continue; }
      if (tmppt == tmpobj->getPoint2()) { flag = true; continue; }
    }
    if (flag == false) pointList.remove(tmppt);
  }
}

// Group(void): collect objects which are connected into groups.  The code
// creates two kinds of groups:  lines, so each distinct structure becomes
// a group; and arrows and text near arrows.  
// Return: the number of groups in the selection.
int Render2D::Group(void)
{
  int currentgid = 1, iflag = false;

  // "Degroup" all items
  DPoint *tmppt1, *tmppt2;

  for (tmppt1 = selectPointList.first(); tmppt1 != 0; 
       tmppt1 = selectPointList.next() ) {
    tmppt1->gid = 0;
  }

  // Assign groups
  customObject *tmpobj;
  do {
    iflag = false;
    // First, find object in a new group.
    for (tmpobj = selectList.first(); tmpobj!=0; tmpobj = selectList.next()) {
      if (tmpobj->Type() == MTEXT) continue;
      if (tmpobj->Type() == MLABEL) continue;
      if (tmpobj->Type() == MBRACKET) continue;
      if (tmpobj->Type() == MSYMBOL) continue;
      tmppt1 = tmpobj->getPoint1();
      tmppt2 = tmpobj->getPoint2();
      // If neither point is part of a group, create a new group
      if ((tmppt1->gid == 0) && (tmppt2->gid == 0)) {
	tmppt1->gid = currentgid; tmppt2->gid = currentgid;
	break;
      }
    }
    // Next, label all objects in that group.
    for (tmpobj = selectList.first(); tmpobj!=0; tmpobj = selectList.next()) {
      if (tmpobj->Type() == MTEXT) continue;
      if (tmpobj->Type() == MLABEL) continue;
      if (tmpobj->Type() == MBRACKET) continue;
      if (tmpobj->Type() == MSYMBOL) continue;
      tmppt1 = tmpobj->getPoint1();
      tmppt2 = tmpobj->getPoint2();
      // If one point is part of a group, the other one is in same group...
      if ((tmppt1->gid == currentgid) && (tmppt2->gid == 0)) {
	tmppt1->gid = currentgid; tmppt2->gid = currentgid;
	tmpobj = selectList.first();  // rewind
      }
      if ((tmppt1->gid == 0) && (tmppt2->gid == currentgid)) {
	tmppt1->gid = currentgid; tmppt2->gid = currentgid;
	tmpobj = selectList.first();  // rewind
      }
    }
    // Have we grouped everything?  If so, quit
    for (tmpobj = selectList.first(); tmpobj!=0; tmpobj = selectList.next()) {
      if (tmpobj->Type() == MTEXT) continue;
      if (tmpobj->Type() == MLABEL) continue;
      if (tmpobj->Type() == MBRACKET) continue;
      if (tmpobj->Type() == MSYMBOL) continue;
      tmppt1 = tmpobj->getPoint1();
      tmppt2 = tmpobj->getPoint2();
      // If there is an ungrouped object, we need to continue
      if ((tmppt1->gid == 0) || (tmppt2->gid == 0)) {
	iflag = true;
	currentgid++;
	break;
      }
    }
  } while (iflag == true);
  
  return currentgid;
}

// GroupAll(void): collect objects which are connected into groups.  The code
// creates two kinds of groups:  lines, so each distinct structure becomes
// a group; and arrows and text near arrows.  
// GroupAll ignores arrows and collects entire structure.
// Return: the number of groups in the selection.
int Render2D::GroupAll(void)
{
  int currentgid = 1, iflag = false;

  // "Degroup" all items
  DPoint *tmppt1, *tmppt2;

  for (tmppt1 = pointList.first(); tmppt1 != 0; 
       tmppt1 = pointList.next() ) {
    tmppt1->gid = 0;
  }

  // Assign groups
  customObject *tmpobj;
  do {
    iflag = false;
    // First, find object in a new group.
    for (tmpobj = objectList.first(); tmpobj!=0; tmpobj = objectList.next()) {
      if (tmpobj->Type() == MTEXT) continue;
      if (tmpobj->Type() == MLABEL) continue;
      if (tmpobj->Type() == MBRACKET) continue;
      if (tmpobj->Type() == MSYMBOL) continue;
      if (tmpobj->Type() == MARROW) continue;
      tmppt1 = tmpobj->getPoint1();
      tmppt2 = tmpobj->getPoint2();
      // If neither point is part of a group, create a new group
      if ((tmppt1->gid == 0) && (tmppt2->gid == 0)) {
	tmppt1->gid = currentgid; tmppt2->gid = currentgid;
	break;
      }
    }
    // Next, label all objects in that group.
    for (tmpobj = objectList.first(); tmpobj!=0; tmpobj = objectList.next()) {
      if (tmpobj->Type() == MTEXT) continue;
      if (tmpobj->Type() == MLABEL) continue;
      if (tmpobj->Type() == MBRACKET) continue;
      if (tmpobj->Type() == MSYMBOL) continue;
      if (tmpobj->Type() == MARROW) continue;
      tmppt1 = tmpobj->getPoint1();
      tmppt2 = tmpobj->getPoint2();
      // If one point is part of a group, the other one is in same group...
      if ((tmppt1->gid == currentgid) && (tmppt2->gid == 0)) {
	tmppt1->gid = currentgid; tmppt2->gid = currentgid;
	tmpobj = objectList.first();  // rewind
      }
      if ((tmppt1->gid == 0) && (tmppt2->gid == currentgid)) {
	tmppt1->gid = currentgid; tmppt2->gid = currentgid;
	tmpobj = objectList.first();  // rewind
      }
    }
    // Have we grouped everything?  If so, quit
    for (tmpobj = objectList.first(); tmpobj!=0; tmpobj = objectList.next()) {
      if (tmpobj->Type() == MTEXT) continue;
      if (tmpobj->Type() == MLABEL) continue;
      if (tmpobj->Type() == MBRACKET) continue;
      if (tmpobj->Type() == MSYMBOL) continue;
      if (tmpobj->Type() == MARROW) continue;
      tmppt1 = tmpobj->getPoint1();
      tmppt2 = tmpobj->getPoint2();
      // If there is an ungrouped object, we need to continue
      if ((tmppt1->gid == 0) || (tmppt2->gid == 0)) {
	iflag = true;
	currentgid++;
	break;
      }
    }
  } while (iflag == true);
  
  return currentgid;
}

// Align(int dir):  Align selection as specified by dir.
void Render2D::Align(int dir)
{
  int ngroups=255;
  // this only works on selections
  if (selectList.count() == 0) {
    emit UpdateStatusBar("Select something to align first.");
    return;
  }

  // Make groups
  ngroups = Group();

  GroupData g[255];
  DPoint *tmppt;
  for (	tmppt = selectPointList.first(); tmppt != 0; 
	tmppt = selectPointList.next() ) {
    if (tmppt->gid > 0) {
      double xa = tmppt->x();
      double ya = tmppt->y();
      g[tmppt->gid - 1].sigmax += tmppt->x();
      g[tmppt->gid - 1].sigmay += tmppt->y();
      g[tmppt->gid - 1].n++;
      if (tmppt->x() < g[tmppt->gid - 1].xmin) g[tmppt->gid - 1].xmin = tmppt->x();
      if (tmppt->x() > g[tmppt->gid - 1].xmax) g[tmppt->gid - 1].xmax = tmppt->x();
      if (tmppt->y() < g[tmppt->gid - 1].ymin) g[tmppt->gid - 1].ymin = tmppt->y();
      if (tmppt->y() > g[tmppt->gid - 1].ymax) g[tmppt->gid - 1].ymax = tmppt->y();
    }
  }
  for (int c = 0; c < ngroups; c++) {
    g[c].xcenter = g[c].sigmax / (double)g[c].n;
    g[c].ycenter = g[c].sigmay / (double)g[c].n;
  }
  
  // Make calculations needed to adjust to correct alignment
  int d;
  switch (dir) {
  case ALIGN_HCENTER:
    // Align centers with center of first group
    for (d = 1; d < ngroups; d++) {
      g[d].dx = 0.0;
      g[d].dy = g[0].ycenter - g[d].ycenter;
    }
    break;
    
  case ALIGN_VCENTER:
    // Align centers with center of first group
    for (d = 1; d < ngroups; d++) {
      g[d].dx = g[0].xcenter - g[d].xcenter;
      g[d].dy = 0.0;
    }
    break;
  }
  
  // Apply dx and dy for each group to adjust
  for (	tmppt = selectPointList.first(); tmppt != 0; 
	tmppt = selectPointList.next() ) {
    if (tmppt->gid > 0) {
      tmppt->setX(tmppt->x() + g[tmppt->gid - 1].dx);
      tmppt->setY(tmppt->y() + g[tmppt->gid - 1].dy);
    }
  }

  // Detect (and fix) arrows and possible associated text
  if (dir == ALIGN_HCENTER) {
    customObject *tmp_obj_2;
    for (tmp_obj = selectList.first(); tmp_obj != NULL; 
	 tmp_obj = selectList.next()) {
      if (tmp_obj->Type() == MARROW) {
	double x1 = tmp_obj->getPoint1()->x();
	double y1 = tmp_obj->getPoint1()->y();
	double x2 = tmp_obj->getPoint2()->x();
	double y2 = tmp_obj->getPoint2()->y();
	double xc, yc;
	  yc = (y1 + y2) / 2.0;
	if (y1 != y2) {
	  tmp_obj->getPoint1()->setY(yc);
	  tmp_obj->getPoint2()->setY(yc);
	}
	if (x1 > x2) { xc = x1; x1 = x2; x2 = xc; }
	xc = (x1 + x2) / 2.0;
	for (tmp_obj_2 = selectList.first(); tmp_obj_2 != NULL; 
	     tmp_obj_2 = selectList.next()) {
	  if (tmp_obj_2->Type() == MTEXT) {
	    double xt1 = tmp_obj_2->getPoint1()->x();
	    if ( (xt1 < x1) || (xt1 > x2) ) continue;
	    double yt1 = tmp_obj_2->getPoint1()->y();
	    int sw = WidthOfString(tmp_obj_2->label, tmp_obj_2->font);
	    int sh = HeightOfString(tmp_obj_2->label, tmp_obj_2->font);
	    tmp_obj_2->getPoint1()->setX(xc - ((double)sw / 2.0));
	    if (yt1 < yc)
	      tmp_obj_2->getPoint1()->setY(yc - (double)sh);
	    else
	      tmp_obj_2->getPoint1()->setY(yc + 14.0);
	  }
	}
      }
    }
  }

  // Draw resize handle etc.
  double x1 = 9999.0, x2 = 0.0, y1 = 9999.0, y2 = 0.0;
  for (tmppt=selectPointList.first(); tmppt != 0; tmppt=selectPointList.next()) {
    if (tmppt->x() < x1) x1 = tmppt->x();
    if (tmppt->x() > x2) x2 = tmppt->x();
    if (tmppt->y() < y1) y1 = tmppt->y();
    if (tmppt->y() > y2) y2 = tmppt->y();
  }
  resizeOrigin = new DPoint(x1, y1);
  resizeHandle = new DPoint(x2, y2);
  
  // Draw the mess we just made!
  repaint();
}

// Resize(double sf, int global = false): resize a selection, or the 
// entire drawing (global = true) by factor sf.
// NOT TO BE CONFUSED WITH QWidget::resize()!!!
void Render2D::Resize(double sf, int global)
{
  double dx1, dy1;
  DPoint *tmppt;
  customObject *tmpobj;

  if (global == true) {
    // global resize
    for (tmppt = pointList.first(); tmppt != 0; tmppt = pointList.next() ) {
      tmppt->setX(tmppt->x() * sf);
      tmppt->setY(tmppt->y() * sf);
    }
  } else {
    // local resize
    for (tmppt = selectPointList.first(); tmppt != 0; tmppt = selectPointList.next() ) {
      dx1 = tmppt->x() - resizeOrigin->x();
      dy1 = tmppt->y() - resizeOrigin->y();
      dx1 *= sf;
      dy1 *= sf;
      tmppt->setX(resizeOrigin->x() + dx1);
      tmppt->setY(resizeOrigin->y() + dy1);
    }
  }
}

// Slots
void Render2D::FlipH(void) { Flip(HORIZONTAL_AXIS); }
void Render2D::FlipV(void) { Flip(VERTICAL_AXIS); }

// Flip(int flipdir): Flip a selection in the direction specified by flipdir.
void Render2D::Flip(int flipdir) 
{
// Something must be selected to flip (do not allow flipping entire drawing!)
	if (selectList.count() == 0) return;
	qWarning("Flip");
// as with many, many other applications, need to find min and max
	DPoint *tmp;
	double xmin = 9999.0, ymin = 9999.0, xmax = 0.0, ymax = 0.0;
	for (tmp = selectPointList.first(); tmp != 0; 
		tmp = selectPointList.next() ) {
		if (tmp->x() < xmin) xmin = tmp->x();
		if (tmp->x() > xmax) xmax = tmp->x();
		if (tmp->y() < ymin) ymin = tmp->y();
		if (tmp->y() > ymax) ymax = tmp->y();
	}
// Flip the points
	double t1;
	for (tmp = selectPointList.first(); tmp != 0; 
		tmp = selectPointList.next() ) {
		switch (flipdir) {
		case HORIZONTAL_AXIS:
			t1 = tmp->y() - ymin;
			tmp->setY(ymax - t1);
			break;

		case VERTICAL_AXIS:
			t1 = tmp->x() - xmin;
			tmp->setX(xmax - t1);
			break;
		}
	}
	repaint();
}

// Hydrogens:  help validate structure by making sure it has the correct
// number of hydrogens.
void Render2D::Hydrogens(void)
{
  int h = 0, flag = false;

  for (tmp_pt = pointList.first(); tmp_pt != 0; tmp_pt = pointList.next() ) {
    // Only atoms have attached hydrogens
    if (tmp_pt->isAtom() == false) continue;
    h = NumberOfBonds(tmp_pt->Name());
    if ( (tmp_pt->Name() == "S") && (h == 4) ) h = 0;
    for (tmp_obj = objectList.first(); tmp_obj != 0; 
	 tmp_obj = objectList.next() ) {
      // LINE types are 100-120
      if (tmp_obj->Type() > 150) continue;
      if (tmp_obj->isMember(tmp_pt) == true) {
	h -= tmp_obj->thickness;
      }
    }
    if ( h < 0 ) { h = 0; flag = true; }
    tmp_pt->setHydrogens(h);
  }

  //if (flag == true) {
  //  QMessageBox::warning(this, "Possible structure error",
  //		 "The molecular weight caclulator found a possible error in your drawing.\nRun 'Verify structure...' for more details.");
  //}
}

void Render2D::FindMW(void)
{
  Cleanup();

  int ngroups;
  double mw[16];  // there better not be more than 16 groups :)
  for (int xa = 0; xa < 15; xa++) mw[xa] = 0.0;

  qWarning("Finding Molecular Weights...");
  // If nothing selected, select all
  if (selectList.count() == 0) {
    selectAll();
  }
  // Group things together
  ngroups = Group();
  if (ngroups == 0) {
    qWarning("No structures found!");
    return;
  }
  // Add hydrogens
  Hydrogens();

  // calculate molecular weights
  for (	tmp_pt = selectPointList.first(); tmp_pt != 0; 
	tmp_pt = selectPointList.next() ) {
    if ( (tmp_pt->gid > 0) && (tmp_pt->isAtom() == true) ) {
      mw[tmp_pt->gid] += MolecularWeight(tmp_pt->Name());
      //            qWarning("MW = %lf", mw[tmp_pt->gid]);
      mw[tmp_pt->gid] += (tmp_pt->Hydrogens() * 1.00794);
      //            qWarning("MW(H) = %lf", mw[tmp_pt->gid]);
    }
  }

  // calculate position info
  GroupData g[255];
  for (	tmp_pt = selectPointList.first(); tmp_pt != 0; 
	tmp_pt = selectPointList.next() ) {
    if (tmp_pt->gid > 0) {
      double xa = tmp_pt->x();
      double ya = tmp_pt->y();
      g[tmp_pt->gid - 1].sigmax += tmp_pt->x();
      g[tmp_pt->gid - 1].sigmay += tmp_pt->y();
      g[tmp_pt->gid - 1].n++;
      if (tmp_pt->x() < g[tmp_pt->gid - 1].xmin) 
	g[tmp_pt->gid - 1].xmin = tmp_pt->x();
      if (tmp_pt->x() > g[tmp_pt->gid - 1].xmax) 
	g[tmp_pt->gid - 1].xmax = tmp_pt->x();
      if (tmp_pt->y() < g[tmp_pt->gid - 1].ymin) 
	g[tmp_pt->gid - 1].ymin = tmp_pt->y();
      if (tmp_pt->y() > g[tmp_pt->gid - 1].ymax) 
	g[tmp_pt->gid - 1].ymax = tmp_pt->y();
    }
  }

  // Draw labels with MW
  for ( xa = 0; xa < ngroups; xa++) {
    if (mw[xa+1] > 0.0) {
      tmp_pt = new DPoint(g[xa].xmin, g[xa].ymax + 20);
      tmp_pt->setName("");
      pointList.append(tmp_pt);
      tmp_obj = new customObject;
      tmp_obj->setType(MTEXT);
      QString t1; t1.setNum(mw[xa+1]);
      tmp_obj->setLabel(QString("MW = ") + t1);
      tmp_obj->setFont(currentFont);
      tmp_obj->addPoint(tmp_pt);
      tmp_pt->setColor(currentColor);
      objectList.append(tmp_obj);
    }
  }

  repaint();
}

// calculate width of formatted string using given font
int Render2D::WidthOfString(QString l, QFont f) {
  int t1, t2, max = 0;
  QFontMetrics fm(f);

  QTextStream t(&l, IO_ReadOnly);

  do {
    QString n = t.readLine();
    t1 = n.find(QString("<"));
    if (t1 >= 0) {
      do {
	t1 = n.find(QString("<"));
	t2 = n.find(QString(">"), t1);
	n.remove(t1, t2 - t1 + 1);
      } while (n.find(QString("<")) >= 0);
    }
    if (fm.width(n) > max) max = fm.width(n);
  } while (!t.atEnd());

  return max;
}

// calculate height of formatted string using given font
int Render2D::HeightOfString(QString l, QFont f) {
  int max = 0;
  QFontMetrics fm(f);

  QTextStream t(&l, IO_ReadOnly);

  do {
    QString n = t.readLine();
    max++;
  } while (!t.atEnd());

  return max * fm.height();
}
