#include <iostream.h>
#include <qpainter.h>
#include <qimage.h>
#include <qevent.h>
#include <qlist.h>
#include <math.h>
#include <qbitmap.h>
#include <qcursor.h>
#include <qdialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpointarray.h>
#include <qprinter.h>
#include <qpaintdevicemetrics.h>
#include <qmessagebox.h>
#include <qnamespace.h>

#include "render2d.h"
#include "customObject.h"
#include "defs.h"
#include "skullcursor.xbm"
#include "rotatecursor.xbm"
#include "textdialog.h"
#include "labeldialog.h"
#include "mclipboard.h"
#include "dpoint.h"

extern MClipboard clipboard;

// Convenience function: convert double to int, with correct rounding.
int Render2D::DoubleToInteger(double a) {
  int b = (int)a;
  if (((int)a / (int)fixedAngle) != (((int)a+1) / (int)fixedAngle))
    return (int)a+1;
  else
    return (int)a;
}

Render2D::Render2D(QWidget *parent, const char *name)
  : QWidget(parent, name)
{
  printer = new QPrinter;

  currentColor = QColor(0,0,0);
	
  button1down = FALSE;
  firsttime = TRUE;

  setPalette(QPalette(QColor(255,255,255)));

  DPoint *p1 = new DPoint(100,120);
  DPoint *p2 = new DPoint(200,220);

  opmode = MODE_SELECT;
  freehand = FREE_DRAWING;
  setCursor(arrowCursor);
  setMouseTracking(true);

  objectList.setAutoDelete(false);
  selectList.setAutoDelete(false);
  pointList.setAutoDelete(false);
  selectPointList.setAutoDelete(false);

  printer->setFullPage(true);
  printer->setPageSize(QPrinter::Letter);
  psize = PAGE_LETTER;
  printer->setOrientation(QPrinter::Portrait);
  porient = PAGE_PORTRAIT;
  printer->setColorMode(QPrinter::GrayScale);

  QPaintDeviceMetrics pm(printer);
  pagewidth = pm.width();
  pageheight = pm.height();

  pixelsPerMM = (double)pm.width() / (double)pm.widthMM();
  //	qWarning("%lf pixels per MM", pixelsPerMM);

  resize(pagewidth, pageheight);

  rendermap = new QPixmap(pagewidth, pageheight);
  rendermap->fill();

  currentFont = QFont("Helvetica", 12);
  linewidth = 1;

  Threshold = 5.0;
  fixedLength = 25.0;
  fixedAngle = 15.0;

  point_index = 0;
  is_point_highlighted = false;

  QBitmap skullBitmap = QBitmap(32, 32, skullcursor_xbm_bits, TRUE);
  skullCursor = QCursor(skullBitmap, skullBitmap, 1, 1);
  QBitmap rotateBitmap = QBitmap(32, 32, rotatecursor_xbm_bits, TRUE);
  rotateCursor = QCursor(rotateBitmap, rotateBitmap, 1, 1);

  highlighted_point = 0;
  highlighted_line = 0;

  vchanged = false;
	
  nearResizeHandle = false;
}

void Render2D::setSizeAndOrientation(int n, int o)
{
  psize = n;
  porient = o;

  if (psize == PAGE_LETTER)
    printer->setPageSize(QPrinter::Letter);
  if (psize == PAGE_LEGAL)
    printer->setPageSize(QPrinter::Legal);
  if (psize == PAGE_A4)
    printer->setPageSize(QPrinter::A4);
  if (porient == PAGE_PORTRAIT)
    printer->setOrientation(QPrinter::Portrait);
  else
    printer->setOrientation(QPrinter::Landscape);

  QPaintDeviceMetrics pm(printer);
  pagewidth = pm.width();
  pageheight = pm.height();

  pixelsPerMM = (double)pm.width() / (double)pm.widthMM();
  //	qWarning("%lf pixels per MM", pixelsPerMM);

  resize(pagewidth, pageheight);
}

void Render2D::setPaperSize(int w1, int h1)
{
  psize = PAGE_USERDEFINE;
  pagewidth = w1;
  pageheight = h1;
  resize(pagewidth, pageheight);
}

void Render2D::setFontCourier(void)
{
  if (selectList.count() == 0) {
    currentFont.setFamily("Courier");
    emit ChooseFont(FONT_COURIER);
  } else {
    customObject *tmp;
    for (tmp = selectList.first(); tmp != 0; tmp = selectList.next() )
      tmp->font.setFamily("Courier");
  }
}

void Render2D::setFontHelv(void)
{
  if (selectList.count() == 0) {
    currentFont.setFamily("Helvetica");
    emit ChooseFont(FONT_HELVETICA);
  } else {
    customObject *tmp;
    for (tmp = selectList.first(); tmp != 0; tmp = selectList.next() )
      tmp->font.setFamily("Helvetica");
  }
}

void Render2D::setFontTimes(void)
{
  if (selectList.count() == 0) {
    currentFont.setFamily("Times");
    emit ChooseFont(FONT_TIMES);
  } else {
    customObject *tmp;
    for (tmp = selectList.first(); tmp != 0; tmp = selectList.next() )
      tmp->font.setFamily("Times");
  }
}

void Render2D::setFont8(void) { setFontSize(8); emit ChooseSize(FONT_8); }
void Render2D::setFont10(void) { setFontSize(10); emit ChooseSize(FONT_10); }
void Render2D::setFont12(void) { setFontSize(12); emit ChooseSize(FONT_12); }
void Render2D::setFont14(void) { setFontSize(14); emit ChooseSize(FONT_14); }
void Render2D::setFont18(void) { setFontSize(18); emit ChooseSize(FONT_18); }
void Render2D::setFont24(void) { setFontSize(24); emit ChooseSize(FONT_24); }

void Render2D::setFontSize(int h) {
  if (selectList.count() == 0) {
    currentFont.setPointSize(h);
  } else {
    customObject *tmp;
    for (tmp = selectList.first(); tmp != 0; tmp = selectList.next() )
      tmp->font.setPointSize(h);
    repaint();
  }
}

void Render2D::setLW1(void) { setLW(1); }
void Render2D::setLW2(void) { setLW(2); }
void Render2D::setLW3(void) { setLW(3); }
void Render2D::setLW4(void) { setLW(4); }
void Render2D::setLW5(void) { setLW(5); }

void Render2D::setLW(int nl)
{
  if (selectList.count() == 0) {
    linewidth = nl;
  } else {
    customObject *tmp;
    for (tmp = selectList.first(); tmp != 0; tmp = selectList.next() )
      tmp->setLineWidth(nl);
    repaint();
  }
}

void Render2D::setFL25(void) { setFL(25.0); }
void Render2D::setFL50(void) { setFL(50.0); }
void Render2D::setFL_Quarter(void) { setFL(pixelsPerMM * 6.35); }
void Render2D::setFL_Half(void) { setFL(pixelsPerMM * 12.7); }

void Render2D::setFL(double n) { fixedLength = n; }

void Render2D::setFA10(void) { setFA(10.0); }
void Render2D::setFA15(void) { setFA(15.0); }
void Render2D::setFA30(void) { setFA(30.0); }

void Render2D::setFA(double n) { fixedAngle = n; }

int Render2D::changed(void) 
{
  return vchanged;
}

void Render2D::Print(void)
{
  emit UpdateStatusBar( QString("Printing...") );
  if (!printer->setup(this)) {
    emit UpdateStatusBar( QString("Printing canceled.") );
    return;
  }

  QPaintDeviceMetrics pm(printer);

  deselectAll();
  Render(2);

  emit UpdateStatusBar( QString("Done printing.") );
}

void Render2D::setMode(int newmode)
{
  opmode = newmode;

  // Deselect all when selecting a new tool
  customObject *tmp;
  for (tmp=selectList.first(); tmp != 0; tmp=selectList.next()) {
    tmp->Unhighlight();
  }
  repaint();
  selectList.clear();
  selectPointList.clear();

  switch (opmode) {
  case MODE_SELECT:
    emit UpdateStatusBar( QString("Left-click to select objects, click and drag to select multiple") );
    setCursor( arrowCursor );
    break;

  case MODE_DRAW_LINE:
  case MODE_DASH_LINE:
  case MODE_ARROW:
  case MODE_UPLINE:
  case MODE_DOWNLINE:
  case MODE_BRACKETS:
    emit UpdateStatusBar( QString("Left-click and drag to draw") );
    setCursor( crossCursor );
    break;

  case MODE_ERASE:
    emit UpdateStatusBar( QString("Highlight objects with cursor and left-click to delete") );
    setCursor( skullCursor );
    break;

  case MODE_TEXT:
    emit UpdateStatusBar( QString("Click to add text, click on a point to label the point") );
    setCursor( ibeamCursor );
    break;
  }
}

void Render2D::setParameter(int p)
{
  if ((p == FIXED_DRAWING) || (p == FREE_DRAWING)) {
    freehand = p;
  }
}

void Render2D::deselectAll(void)
{
  if (selectList.count() == 0) return;
  // Deselect all when selecting a new tool
  setCursor(arrowCursor);
  customObject *tmp;
  for (tmp=selectList.first(); tmp != 0; tmp=selectList.next()) {
    tmp->Unhighlight();
  }
  selectList.clear();
  selectPointList.clear();
  // clear variables used to resize
  nearResizeHandle = false;
  if (resizeOrigin) delete resizeOrigin;
  if (resizeHandle) delete resizeHandle;
}

void Render2D::selectAll(void)
{
  double x1, y1, x2, y2; 

  deselectAll();
  opmode = MODE_MULTIPLE_SELECTION;
  customObject *tmp;
  for (tmp=objectList.first(); tmp != 0; tmp=objectList.next()) {
    tmp->Highlight();
    selectList.append(tmp);
  }
  DPoint *ptmp;
  for (ptmp=pointList.first(); ptmp != 0; ptmp=pointList.next()) {
    selectPointList.append(ptmp);
  }
  // What if nothing selected?
  if (selectList.count() == 0)
    opmode = MODE_SELECT;

  // make resize handle
  x1 = 99999.0; x2 = 0.0; y1 = 99999.0; y2 = 0.0;
  for (ptmp=selectPointList.first(); ptmp != 0; ptmp=selectPointList.next()) {
    if (ptmp->x() < x1) x1 = ptmp->x();
    if (ptmp->x() > x2) x2 = ptmp->x();
    if (ptmp->y() < y1) y1 = ptmp->y();
    if (ptmp->y() > y2) y2 = ptmp->y();
  }
  resizeOrigin = new DPoint(x1, y1);
  resizeHandle = new DPoint(x2 + 10.0, y2 + 10.0);

  repaint();
  return;
}

void Render2D::Copy(void)
{
  // There must be something selected to copy
  if (opmode != MODE_MULTIPLE_SELECTION) return;
  // Dump current clipboard contents
  clipboard.objects.clear();
  clipboard.points.clear();

  customObject *tmpobj, *tmp1;
  DPoint *tmppt, *tmp2;

  // Copy points
  for (tmp2 = selectPointList.first(); tmp2 != 0; tmp2 = selectPointList.next()) {
    clipboard.points.append( new DPoint( tmp2->x(), tmp2->y() ) );
  }

  // Copy objects
  for (tmp1 = selectList.first(); tmp1 != 0; tmp1 = selectList.next()) {
    tmpobj = new customObject(tmp1);
    tmppt = clipboard.points.at( selectPointList.find( tmp1->getPoint1() ) );
    tmpobj->addPoint(tmppt);
    if (tmpobj->Type() < 150 || tmpobj->Type() == MARROW) {
      tmppt = clipboard.points.at( selectPointList.find( tmp1->getPoint2() ) );
      tmpobj->addPoint(tmppt);
    }
    clipboard.objects.append(tmpobj);
  }
	
  // Offset points by (30,30)
  // (Why?  So if you paste in the same document, the pasted objects
  // won't overlay the existing stuff)
  for (tmp2 = clipboard.points.first(); tmp2 != 0; tmp2 = clipboard.points.next()) {
    tmp2->setX(tmp2->x() + 30);
    tmp2->setY(tmp2->y() + 30);
  }
}

void Render2D::Cut(void)
{
  // There must be something selected to cut
  if (opmode != MODE_MULTIPLE_SELECTION) return;
  // Dump current clipboard contents
  clipboard.objects.clear();
  clipboard.points.clear();

  customObject *tmpobj, *tmp1;
  DPoint *tmppt, *tmp2;

  // Copy points
  for (tmp2 = selectPointList.first(); tmp2 != 0; tmp2 = selectPointList.next()) {
    clipboard.points.append( new DPoint( tmp2->x(), tmp2->y() ) );
  }

  // Cut (Copy objects and delete original)
  for (tmp1 = selectList.first(); tmp1 != 0; tmp1 = selectList.next()) {
    tmpobj = new customObject(tmp1);
    tmppt = clipboard.points.at( selectPointList.find( tmp1->getPoint1() ) );
    tmpobj->addPoint(tmppt);
    if (tmpobj->Type() == MLINE || tmpobj->Type() == MARROW) {
      tmppt = clipboard.points.at( selectPointList.find( tmp1->getPoint2() ) );
      tmpobj->addPoint(tmppt);
    }
    clipboard.objects.append(tmpobj);
    objectList.remove(tmp1);
  }
  // clear variables used to resize
  nearResizeHandle = false;
  if (resizeOrigin) delete resizeOrigin;
  if (resizeHandle) delete resizeHandle;

  opmode = MODE_SELECT;
  repaint();
}

void Render2D::Paste(void)
{
  // Deselect all when pasting
  customObject *tmp, *tmpobj;
  for (tmp=selectList.first(); tmp != 0; tmp=selectList.next()) {
    tmp->Unhighlight();
  }
  selectList.clear();
  selectPointList.clear();

  DPoint *tmp2, *tmppt;
  int curlen = pointList.count();
  for (tmp2 = clipboard.points.first(); tmp2 != 0; 
       tmp2 = clipboard.points.next()) {
    tmppt = new DPoint( tmp2->x(), tmp2->y() );
    pointList.append(tmppt);
    selectPointList.append(tmppt);
  }
  for (tmp = clipboard.objects.first(); tmp != 0; 
       tmp = clipboard.objects.next()) {
    tmpobj = new customObject(tmp);
    tmppt = pointList.at( clipboard.points.find( tmp->getPoint1() ) + curlen );
    tmpobj->addPoint(tmppt);
    if (tmpobj->Type() == MLINE || tmpobj->Type() == MARROW) {
      tmppt = pointList.at( clipboard.points.find( tmp->getPoint2() ) + curlen );
      tmpobj->addPoint(tmppt);
    }
    objectList.append(tmpobj);
    selectList.append(tmpobj);
  }
  for (tmp=selectList.first(); tmp != 0; tmp=selectList.next()) {
    tmp->Highlight();
  }
  setCursor(arrowCursor);	
  opmode = MODE_MULTIPLE_SELECTION;

  // Create resize handle
  double x1, y1, x2, y2;
  x1 = 99999.0; x2 = 0.0; y1 = 99999.0; y2 = 0.0;
  DPoint *ptmp;
  for (ptmp=selectPointList.first(); ptmp != 0; ptmp=selectPointList.next()) {
    if (ptmp->x() < x1) x1 = ptmp->x();
    if (ptmp->x() > x2) x2 = ptmp->x();
    if (ptmp->y() < y1) y1 = ptmp->y();
    if (ptmp->y() > y2) y2 = ptmp->y();
  }
  resizeOrigin = new DPoint(x1, y1);
  resizeHandle = new DPoint(x2, y2);

  repaint();
}

// Clear: delete selected objects.  This SLOT is called from the edit
// menu, defined in application.cpp
void Render2D::Clear(void)
{
  // something must be selected to delete
  if (selectList.count() == 0) return;
  customObject *tmp;

  // Execute this action via Action() to create undo record
  Action(DELETE_MULTIPLE);
  
  // empty select list
  selectList.clear();
  // clear variables used to resize
  nearResizeHandle = false;
  if (resizeOrigin) delete resizeOrigin;
  if (resizeHandle) delete resizeHandle;
  // the selection is gone, go back to MODE_SELECT
  opmode = MODE_SELECT;
  
  repaint();
}

// Rotate:  Rotate selected objects.
void Render2D::Rotate(void)
{
// if nothing selected, why rotate?
	if (opmode != MODE_MULTIPLE_SELECTION) return;
	opmode = MODE_ROTATE;
}

// color slots call Color(QColor) below
void Render2D::setColorBlack() { Color(QColor(0,0,0)); }
void Render2D::setColorRed() { Color(QColor(128,0,0)); }
void Render2D::setColorGreen() { Color(QColor(0,255,0)); }
void Render2D::setColorBlue() { Color(QColor(0,0,255)); }

// Color(QColor c):  Change color of selection, of if nothing selected,
// change current drawing color
void Render2D::Color(QColor c)
{
	if (selectList.count() == 0) {
		currentColor = c;
	} else {
		customObject *tmpobj;
		for (tmpobj = selectList.first(); tmpobj != 0; tmpobj = selectList.next())
			tmpobj->setColor(c);
		repaint();
	}
}

double Render2D::calcPointToLine(DPoint *e1, DPoint *e2, DPoint p)
{
	DPoint *holder;

	if (e2->x() < e1->x()) { holder = e1; e1 = e2; e2 = holder; }

	if (p.x() < e1->x() || p.x() > e2->x()) return 99999.0;	

	if (e1->x() == e2->x()) {
		// vertical line
		if (e2->y() < e1->y()) { holder = e1; e1 = e2; e2 = holder; }
		if (p.y() < e1->y()) return 99999.0;
		if (p.y() > e2->y()) return 99999.0;
		return (double)abs( (int)(p.x() - e1->x()) );
	}

	double slope = (double)(e2->y() - e1->y()) / (double)(e2->x() - e1->x());
	double yint = (double)e1->y() - slope*(double)e1->x();

	double ytest = slope*(double)p.x() + yint;

	ytest -= p.y();

	if (ytest < 0.0) ytest*=-1.0;

	return ytest;
}

customObject* Render2D::findNearestObjectToPoint(DPoint n)
{
	customObject *tmp;
	customObject *retval = 0;
	DPoint *p1, *p2, *p3, *p4;
	highlighted_line_index = false;
	int dx, dy;
	double dist, mindist;
	mindist = Threshold;

	for (tmp=objectList.first(); tmp != 0; tmp=objectList.next()) {
		if ( (tmp->Type() == MLINE) || (tmp->Type() == MARROW) ||
			(tmp->Type() == MDOWNLINE) || (tmp->Type() == MUPLINE) ) {
// get endpoints of line
			p1 = tmp->getPoint1();
			p2 = tmp->getPoint2();

// calculate distance by finding equation of line and evaluating...
			dist = calcPointToLine(p1, p2, n);

			if (dist < mindist) {
				mindist = dist;
				retval = tmp;
			}
		}
		if (tmp->Type() == MBRACKET) {
			p1 = tmp->getPoint1();
			p2 = tmp->getPoint2();
			p3 = new DPoint(p1->x(), p2->y());
			p4 = new DPoint(p2->x(), p1->y());
			dist = calcPointToLine(p1, p3, n);
			if (dist < mindist) { retval = tmp; mindist = dist; }
			dist = calcPointToLine(p2, p4, n);
			if (dist < mindist) { retval = tmp; mindist = dist; }
			delete p3, p4;
		}
		if ((tmp->Type() == MTEXT) || (tmp->Type() == MLABEL)) {
			if (tmp->onText(n)) return tmp;
		}
		if (tmp->Type() == MSYMBOL) {
			p1 = tmp->getPoint1();
//			p1->setX(p1->x() + 5.0);
//			p1->setY(p1->y() + 5.0);
			if (p1->distanceToPoint(n) < 5.0) return tmp;
		}
	}

	return retval;
}

DPoint * Render2D::findNearestPoint(DPoint n, double min)
{
	double dist, mindist;
	mindist = Threshold;
	DPoint* closestPoint;
	closestPoint = 0;
	int x1 = (int)n.x();
	int y1 = (int)n.y();
	int x2, y2, dx, dy;

	DPoint *tmp;
	for (tmp=pointList.first(); tmp != 0; tmp=pointList.next()) {
		x2 = (int)tmp->x(); y2 = (int)tmp->y();
		dx = x2 - x1; dy = y2 - y1;
		dist = sqrt((double)(dx*dx + dy*dy));
		if ((dist < mindist) && (dist > min)) {
			mindist = dist;
			closestPoint = tmp;
		}
	}
	return closestPoint;
}

int Render2D::selectedObjectNearPoint(DPoint n)
{
	double dist, mindist;
	int dx, dy;
	mindist = Threshold;
	DPoint *p1, *p2, *p3, *p4;

	customObject *tmp;
	for (tmp=selectList.first(); tmp != 0; tmp=selectList.next()) {
		if ( (tmp->Type() == MLINE) || (tmp->Type() == MARROW) ||
			(tmp->Type() == MUPLINE) || (tmp->Type() == MDOWNLINE) ) {
			p1 = tmp->getPoint1();
			p2 = tmp->getPoint2();
			// calculate distance by finding equation of line and evaluating...
			dist = calcPointToLine(p1, p2, n);

			if (dist < mindist)
				return true;
		}
		if (tmp->Type() == MBRACKET) {
			p1 = tmp->getPoint1();
			p2 = tmp->getPoint2();
			p3 = new DPoint(p1->x(), p2->y());
			p4 = new DPoint(p2->x(), p1->y());
			dist = calcPointToLine(p1, p3, n);
			if (dist < mindist) { return true; }
			dist = calcPointToLine(p2, p4, n);
			if (dist < mindist) { return true; }
			delete p3, p4;
		}
		if ((tmp->Type() == MTEXT) || (tmp->Type() == MLABEL)) {
			if (tmp->onText(n))
				return true;
		}
		if (tmp->Type() == MSYMBOL) {
			p1 = tmp->getPoint1();
//			p1->setX(p1->x() + 5.0);
//			p1->setY(p1->y() + 5.0);
			if (p1->distanceToPoint(n) < 5.0) return true;
		}
	}

	return false;
}

void Render2D::paintEvent(QPaintEvent *)
{
	is_point_highlighted = false;
	QPainter p(this);

	Render(0);
	// now highlight point (if applicable), so it will be on top on all!
	if (highlighted_point != 0)
		p.drawRect((int)highlighted_point->x()-3,(int)highlighted_point->y()-3,6,6);
	// draw resize handle if multiple selection or resize
	if (opmode == MODE_MULTIPLE_SELECTION)
		p.drawRect((int)resizeHandle->x()-3,(int)resizeHandle->y()-3,6,6);
	if (opmode == MODE_DRAG_MULTIPLE_SELECTION)
		p.drawRect((int)resizeHandle->x()-3,(int)resizeHandle->y()-3,6,6);
	if (opmode == MODE_RESIZE)
		p.drawRect((int)resizeHandle->x()-3,(int)resizeHandle->y()-3,6,6);

}

QSizePolicy Render2D::sizePolicy() const
{
	return QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
}

void Render2D::mousePressEvent(QMouseEvent *e1)
{
  vchanged = true;
  button1down = TRUE;
  if (e1->button() == RightButton) {
    button1down = false;
    button3down = true;
  }

  // Right button (button3) events
  if ((opmode == MODE_MULTIPLE_SELECTION) && button3down) {
    opmode = MODE_ROTATE;
    if (selectList.count() == 0) return;
    double x1 = 9999.0, x2 = 0.0, y1 = 9999.0, y2 = 0.0;
    DPoint *tmppt = new DPoint;
    for (tmppt=selectPointList.first(); tmppt!=0; tmppt=selectPointList.next()) {
      if (tmppt->x() < x1) x1 = tmppt->x();
      if (tmppt->x() > x2) x2 = tmppt->x();
      if (tmppt->y() < y1) y1 = tmppt->y();
      if (tmppt->y() > y2) y2 = tmppt->y();
    }
    center = new DPoint(x1 + (x2-x1)/2.0, y1 + (y2-y1)/2.0);
    origin = new DPoint(e1->pos());
    prevangle = getAngle(center, origin);
    setCursor(rotateCursor);
    emit UpdateStatusBar("Drag to rotate selection, release button when done");
    return;
  }
  
  // bail if button 1 not pressed
  if (button1down == false) return;
  
  // Left button (button1) events
  if ((opmode == MODE_MULTIPLE_SELECTION) && (nearResizeHandle)) {
    opmode = MODE_RESIZE;
    emit UpdateStatusBar("Drag to resize selection, release button when done");
  }
  
  if ((opmode == MODE_TEXT) && (highlighted_point == 0) &&
      (highlighted_line == 0)) {
    origin = new DPoint(e1->pos());
    button1down = false;
    TextDialog t(this, QString("mine!"), QString("Enter text"), 
		 currentFont);
    if ( !t.exec() ) return;
    if (t.getText().length() == 0) return;
    // in this case, origin not an element or group, set these to 0
    origin->setName("");
    pointList.append(origin);
    customObject *tmp = new customObject;
    tmp->setType(MTEXT);
    tmp->setLabel(t.getText());
    tmp->setFont(currentFont);
    tmp->addPoint(origin);
    tmp->setColor(currentColor);
    // Go via Action()...
    tmp_obj = tmp;
    Action(ADD_OBJECT);
    //    objectList.append(tmp);
    repaint();
    return;
  }

  if ((opmode == MODE_TEXT) && (highlighted_point != 0) &&
      (highlighted_line == 0)) {
    button1down = false;
    LabelDialog t(this, QString("mine!"), QString("Enter point label"));
    t.setFont(currentFont);
    if ( !t.exec() ) return;
    if (t.getText().length() == 0) return;
    // don't add carbon
    if (t.getText() == QString("C")) return;
    customObject *tmp = new customObject;
    tmp->setType(MLABEL);
    tmp->setLabel(t.getText());
    tmp->setFont(currentFont);
    tmp->addPoint(highlighted_point);
    tmp->setColor(currentColor);
    // Go via Action()...
    tmp_obj = tmp;
    Action(ADD_OBJECT);
    //    objectList.append(tmp);
    // set element/group info in point, if known
    highlighted_point->setName(t.getText());
    repaint();
    return;
  }

  if ((opmode == MODE_TEXT) && (highlighted_point == 0) &&
      (highlighted_line != 0)) {
    button1down = false;
    if (highlighted_line->Type() == MTEXT) {
      TextDialog t(this, QString("mine!"), QString("Edit text"),
		   highlighted_line->font);
      t.setFont(currentFont);
      t.setText(highlighted_line->Label());
      if ( !t.exec() ) return;
      if (t.getText().length() == 0) return;
      tmp_text = t.getText();
      tmp_obj = highlighted_line;
    }
    if (highlighted_line->Type() == MLABEL) {
      LabelDialog t(this, QString("mine!"), QString("Edit point label"));
      t.setFont(currentFont);
      t.setText(highlighted_line->Label());
      if ( !t.exec() ) return;
      if ( (t.getText().length() == 0) ||
	   (t.getText().length() == 0) ) {
	// delete object if carbon or blank
	Action(DELETE_SELECTED);
	return;
      }
      tmp_text = t.getText();
      tmp_obj = highlighted_line;
    }
    Action(UPDATE_TEXT);
    //    highlighted_line->setLabel(t.getText());
    highlighted_line->setFont(currentFont);
    if (highlighted_line->Type() == MLABEL) {
      tmp_pt = highlighted_line->getPoint1();
      tmp_pt->setName(tmp_text);
    }
    repaint();
    return;
  }

  if ( (opmode == MODE_DRAW_LINE) || (opmode == MODE_DASH_LINE) ) {
    if (highlighted_point == 0) {
      origin = new DPoint(e1->pos());
      originflag = NEW_POINT;
    } else {
      origin = highlighted_point;
      originflag = OLD_POINT;
    }
  }

	if (opmode == MODE_UPLINE) {
		if (highlighted_point == 0) {
			origin = new DPoint(e1->pos());
			originflag = NEW_POINT;
		} else {
			origin = highlighted_point;
			originflag = OLD_POINT;
		}
	}

	if (opmode == MODE_DOWNLINE) {
		if (highlighted_point == 0) {
			origin = new DPoint(e1->pos());
			originflag = NEW_POINT;
		} else {
			origin = highlighted_point;
			originflag = OLD_POINT;
		}
	}

	if (opmode == MODE_ARROW) {
		origin = new DPoint(e1->pos());
	}

	if (opmode == MODE_BRACKETS) {
		origin = new DPoint(e1->pos());
	}

	if (opmode == MODE_ROTATE) {
		if (selectList.count() == 0) return;
		double x1 = 9999.0, x2 = 0.0, y1 = 9999.0, y2 = 0.0;
		DPoint *tmppt = new DPoint;
		for (tmppt=selectPointList.first(); tmppt!=0; tmppt=selectPointList.next()) {
			if (tmppt->x() < x1) x1 = tmppt->x();
			if (tmppt->x() > x2) x2 = tmppt->x();
			if (tmppt->y() < y1) y1 = tmppt->y();
			if (tmppt->y() > y2) y2 = tmppt->y();
		}
		center = new DPoint(x1 + (x2-x1)/2.0, y1 + (y2-y1)/2.0);
		origin = new DPoint(e1->pos());
		prevangle = getAngleRad(center, origin);
		setCursor(rotateCursor);
	}

	if ((opmode == MODE_SELECT) && (highlighted_point == 0) &&
		(highlighted_line == 0)) {
		origin = new DPoint(e1->pos());
		opmode = MODE_SELECT_MULTIPLE;
		emit UpdateStatusBar("Drag to draw a box around objects you want to select");
	}

	if ((opmode == MODE_SELECT) && (highlighted_point == 0) &&
		(highlighted_line != 0)) {
		setCursor(sizeAllCursor);
		selectList.append(highlighted_line);
		selectPointList.append(highlighted_line->getPoint1());
		if (highlighted_line->Type() == MLINE)
			selectPointList.append(highlighted_line->getPoint2());
		if (highlighted_line->Type() == MUPLINE)
			selectPointList.append(highlighted_line->getPoint2());
		if (highlighted_line->Type() == MDOWNLINE)
			selectPointList.append(highlighted_line->getPoint2());
		if (highlighted_line->Type() == MARROW)
			selectPointList.append(highlighted_line->getPoint2());
		if (highlighted_line->Type() == MBRACKET)
			selectPointList.append(highlighted_line->getPoint2());
		opmode = MODE_MULTIPLE_SELECTION;
		// Draw resize handle etc.
		double x1 = 9999.0, x2 = 0.0, y1 = 9999.0, y2 = 0.0;
		DPoint *ptmp = new DPoint;
		for (ptmp=selectPointList.first(); ptmp != 0; ptmp=selectPointList.next()) {
			if (ptmp->x() < x1) x1 = ptmp->x();
			if (ptmp->x() > x2) x2 = ptmp->x();
			if (ptmp->y() < y1) y1 = ptmp->y();
			if (ptmp->y() > y2) y2 = ptmp->y();
		}
		resizeOrigin = new DPoint(x1, y1);
		resizeHandle = new DPoint(x2 + 10.0, y2 + 10.0);

		return;
	}

	if ((opmode == MODE_MULTIPLE_SELECTION) && 
		(!selectedObjectNearPoint( e1->pos() ) )) {
		deselectAll();
		opmode = MODE_SELECT;
		repaint();
		return;
	}

	if ((opmode == MODE_MULTIPLE_SELECTION) && 
		(selectedObjectNearPoint( e1->pos() ) )) {
		origin = new DPoint(e1->pos());
		opmode = MODE_DRAG_MULTIPLE_SELECTION;
		emit UpdateStatusBar("Drag to move selected object(s), release button when done");
	}

	if ((opmode == MODE_SELECT) && (highlighted_point != 0)) {
		opmode = MODE_SELECT_DRAGPOINT;
	}

	if (opmode == MODE_ERASE) {
			if (highlighted_line == 0) return;
		if (highlighted_line->Type() == MLINE) {
			if (highlighted_line->Thickness() == SINGLE_BOND) {
				Action(DELETE_SELECTED);
			} else {
				int t1 = highlighted_line->Thickness();
				highlighted_line->setThickness(t1-1);
				highlighted_line->setDash(false);
			}
			return;
		}
		if (highlighted_line->Type() == MUPLINE) {
			Action(DELETE_SELECTED);
			return;
		}
		if (highlighted_line->Type() == MDOWNLINE) {
			Action(DELETE_SELECTED);
			return;
		}
		if (highlighted_line->Type() == MARROW) {
			Action(DELETE_SELECTED);
			return;
		}
		if (highlighted_line->Type() == MLABEL) {
			Action(DELETE_SELECTED);
			return;
		}
		if (highlighted_line->Type() == MTEXT) {
			Action(DELETE_SELECTED);
			return;
		}
		if (highlighted_line->Type() == MSYMBOL) {
			Action(DELETE_SELECTED);
			return;
		}
	}

}

void Render2D::mouseReleaseEvent(QMouseEvent *e1)
{
  QPainter p(this);
  button1down = FALSE;
  int myflag = false; double x1, y1, x2, y2, swp;
  
  if (opmode == MODE_RESIZE) {
    opmode = MODE_MULTIPLE_SELECTION;
    emit UpdateStatusBar( " " );
  }
  
  if (opmode == MODE_SELECT_MULTIPLE) {
    opmode = MODE_MULTIPLE_SELECTION;
    target = new DPoint(e1->pos());
    // make (x1,y1) the top left and (x2,y2) the bottom right
    x1 = origin->x(); y1 = origin->y();
    x2 = target->x(); y2 = target->y();
    if (x1 > x2) { swp = x1; x1 = x2; x2 = swp; }
    if (y1 < y2) { swp = y1; y1 = y2; y2 = swp; }
    // Select objects in the box
    customObject *tmp;
    for (tmp=objectList.first(); tmp != 0; tmp=objectList.next()) {
      if (tmp->withinBounds(x1, y1, x2, y2)) {
	tmp->Highlight();
	selectList.append(tmp);
      }
    }
    DPoint *ptmp;
    for (ptmp=pointList.first(); ptmp != 0; ptmp=pointList.next()) {
      if ( (ptmp->x() > x1 ) && ( ptmp->x() < x2 ) && ( ptmp->y() < y1 ) &&
	   ( ptmp->y() > y2 ) ) {
	selectPointList.append(ptmp);
      }
    }
    // What if nothing selected?
    if (selectList.count() == 0) {
      opmode = MODE_SELECT;
      repaint();
      return;
    }
    // Draw resize handle etc.
    x1 = 99999.0; x2 = 0.0; y1 = 99999.0; y2 = 0.0;
    for (ptmp=selectPointList.first(); ptmp != 0; ptmp=selectPointList.next()) {
      if (ptmp->x() < x1) x1 = ptmp->x();
      if (ptmp->x() > x2) x2 = ptmp->x();
      if (ptmp->y() < y1) y1 = ptmp->y();
      if (ptmp->y() > y2) y2 = ptmp->y();
    }
    resizeOrigin = new DPoint(x1, y1);
    resizeHandle = new DPoint(x2 + 10.0, y2 + 10.0);
    repaint();
  }

  if (opmode == MODE_SELECT_DRAGPOINT) {
    opmode = MODE_SELECT;
  }

  if (opmode == MODE_DRAG_MULTIPLE_SELECTION) {
    opmode = MODE_MULTIPLE_SELECTION;
  }
  
  if (opmode == MODE_ERASE) {
    repaint();
  }
  
  if (opmode == MODE_ROTATE) {
    if (button3down) button3down = false;
    opmode = MODE_MULTIPLE_SELECTION;
    setCursor(arrowCursor);
    emit UpdateStatusBar(" ");
    repaint();
  }

  if ( (opmode == MODE_DRAW_LINE) || (opmode == MODE_DASH_LINE) ) {
    // to avoid problems, bail if either origin or target are null.
    if (origin == 0) return;
    if (target == 0) return;
    customObject *tmp = new customObject;
    tmp->setType(MLINE);
    tmp->setLineWidth(linewidth);
    if (originflag) {
      pointList.append(origin);
      tmp->addPoint(origin);		
    } else {
      tmp->addPoint(origin);
    }
    if (targetflag) {
      pointList.append(target);
      tmp->addPoint(target);
    } else {
      tmp->addPoint(target);
    }
    // Line points are atoms
    origin->isAtom(true);
    target->isAtom(true);
    // don't add new line if this one draws over another...
    if ((originflag == OLD_POINT) && (targetflag == OLD_POINT)) {
      customObject *tmp;
      DPoint *t1, *t2;
      for (tmp=objectList.first(); tmp != 0; tmp=objectList.next()) {
	if ((tmp->Type() == MLINE) || (tmp->Type() == MUPLINE)) {
	  t1 = tmp->getPoint1();
	  t2 = tmp->getPoint2();
	  if (((t1 == origin) || (t2 == origin)) && ((t1 == target) || (t2 == target))) {
	    int t3 = tmp->Thickness();
	    if (t3 < 3)  // can't draw higher than triple bond!
	      tmp->setThickness(t3+1);
	    if (opmode == MODE_DASH_LINE) tmp->setDash(true);
	    myflag = true;
	    break;
	  }
	}
      }
      if (myflag) { repaint(); return; }
    }
    tmp->setColor(currentColor);
    if (opmode == MODE_DASH_LINE) tmp->setDash(true);		
    // Add object via Action() so it can be undone
    tmp_obj = tmp;
    Action(ADD_OBJECT);
    //    objectList.append(tmp);
    repaint();
  }

  if ((opmode == MODE_UPLINE) || (opmode == MODE_DOWNLINE)) {
    customObject *tmp = new customObject;
    switch (opmode) {
    case MODE_UPLINE:
      tmp->setType(MUPLINE);
      break;
    case MODE_DOWNLINE:
      tmp->setType(MDOWNLINE);
      break;
    }
    if (originflag) {
      pointList.append(origin);
      tmp->addPoint(origin);		
    } else {
      tmp->addPoint(origin);
    }
    if (targetflag) {
      pointList.append(target);
      tmp->addPoint(target);
    } else {
      tmp->addPoint(target);
    }
    // Line points are atoms
    origin->isAtom(true);
    target->isAtom(true);
    // don't add new line if this one draws over another...
    if ((originflag == OLD_POINT) && (targetflag == OLD_POINT)) {
      repaint(); return;
    }
    tmp->setColor(currentColor);
    // Add object via Action() so it can be undone
    tmp_obj = tmp;
    Action(ADD_OBJECT);
    //objectList.append(tmp);
    repaint();
  }

  if (opmode == MODE_ARROW) {
    customObject *tmp = new customObject;
    tmp->setType(MARROW);
    // in this case, origin not an element or group, set these to 0
    origin->setName("");
    pointList.append(origin);
    tmp->addPoint(origin);		
    // in this case, target not an element or group, set these to 0
    target->setName("");
    pointList.append(target);
    tmp->addPoint(target);
    tmp->setColor(currentColor);
    // Add object via Action() so it can be undone
    tmp_obj = tmp;
    Action(ADD_OBJECT);
    //objectList.append(tmp);
    repaint();
  }
  
  if (opmode == MODE_BRACKETS) {
    customObject *tmp = new customObject;
    tmp->setType(MBRACKET);
    // in this case, origin not an element or group, set these to 0
    origin->setName("");
    pointList.append(origin);
    tmp->addPoint(origin);		
    // in this case, target not an element or group, set these to 0
    target->setName("");
    pointList.append(target);
    tmp->addPoint(target);
    tmp->setColor(currentColor);
    // Add object via Action() so it can be undone
    tmp_obj = tmp;
    Action(ADD_OBJECT);
    //    objectList.append(tmp);
    repaint();
  }
}

void Render2D::mouseMoveEvent(QMouseEvent *e1)
{
	QPainter p(this);
	int dx, dy;
	int repaintflag = false;

	p.setPen(currentColor);

	if ((opmode == MODE_MULTIPLE_SELECTION) &&
		(resizeHandle->distanceToPoint( e1->pos() ) < 5.0 )) {
		nearResizeHandle = true;
		setCursor( sizeFDiagCursor );
		emit UpdateStatusBar( "Click here and drag to resize selection" );
		return;
	}

	if ((opmode == MODE_MULTIPLE_SELECTION) &&
		(resizeHandle->distanceToPoint( e1->pos() ) > 5.0 )) {
		if (nearResizeHandle) {
			setCursor( arrowCursor );
			nearResizeHandle = false;
			emit UpdateStatusBar( " " );
		}
	}

	if ((opmode == MODE_MULTIPLE_SELECTION) && 
		(!selectedObjectNearPoint( e1->pos() ) )) {
		setCursor(arrowCursor);
		emit UpdateStatusBar( " " );
		return;
	}

	if ((opmode == MODE_MULTIPLE_SELECTION) && 
		(selectedObjectNearPoint( e1->pos() ) )) {
		setCursor(sizeAllCursor);
		emit UpdateStatusBar( "Click here and drag to move selection" );
		return;
	}

	if (opmode == MODE_DRAG_MULTIPLE_SELECTION) {
		target = new DPoint(e1->pos());
		dx = (int)( target->x() - origin->x() );
		dy = (int)( target->y() - origin->y() );
		if ( (dx == 0) && (dy == 0) ) return;
		DPoint *tmp;
		for (tmp=selectPointList.first(); tmp != 0; tmp=selectPointList.next()) {
			tmp->setX(tmp->x() + dx);
			tmp->setY(tmp->y() + dy);
		}
		resizeOrigin->setX(resizeOrigin->x() + dx);
		resizeOrigin->setY(resizeOrigin->y() + dy);
		resizeHandle->setX(resizeHandle->x() + dx);
		resizeHandle->setY(resizeHandle->y() + dy);
		repaint();
		origin = target;
		return;
	}

	if (opmode == MODE_RESIZE) {
		double dist1, dist2, sf, dx1, dy1;
		target = new DPoint(e1->pos());
		dx1 = resizeOrigin->x() - target->x();
		dy1 = resizeOrigin->y() - target->y();
		dist2 = sqrt(dx1*dx1 + dy1*dy1);
		dx1 = resizeOrigin->x() - resizeHandle->x();
		dy1 = resizeOrigin->y() - resizeHandle->y();
		dist1 = sqrt(dx1*dx1 + dy1*dy1);
		sf = dist2 / dist1;
		Resize(sf);
		dx1 *= sf;
		dy1 *= sf;
		if (dx1 < 0.0) dx1 *= -1.0;
		if (dy1 < 0.0) dy1 *= -1.0;
		resizeHandle->setX(resizeOrigin->x() + dx1);
		resizeHandle->setY(resizeOrigin->y() + dy1);
		repaint();
	}

	if (opmode == MODE_SELECT_MULTIPLE) {
		if (!button1down) return;
		target = new DPoint(e1->pos());
		int deltax = (int)( target->x() - origin->x() );
		int deltay = (int)( target->y() - origin->y() );
		repaint();
		p.drawRect(origin->x(), origin->y(), deltax, deltay);
	}

	if (opmode == MODE_BRACKETS) {
		if (!button1down) return;
		target = new DPoint(e1->pos());
		DPoint b1(origin->x(), target->y());
		DPoint b2(target->x(), origin->y());
		int s = 1;
		if (target->x() < origin->x()) s = -1;
		repaint();
		p.drawLine( origin->toQPoint(), b1.toQPoint() );
		p.drawLine(b2.toQPoint(), target->toQPoint() );
		p.drawLine(origin->toQPoint(), QPoint(origin->x() + s*5, origin->y()) );
		p.drawLine(QPoint(b2.x() - s*5, b2.y()), b2.toQPoint() );
		p.drawLine(b1.toQPoint(), QPoint(b1.x() + s*5, b1.y()) );
		p.drawLine(QPoint(target->x() - s*5, target->y()), target->toQPoint() );
	}

	if (opmode == MODE_SELECT_DRAGPOINT) {
		DPoint tmppos;
		tmppos = e1->pos();
		highlighted_point->setX(tmppos.x()); 
		highlighted_point->setY(tmppos.y()); 
		repaint();
	}

	if (opmode == MODE_ROTATE) {
		if ( (!button1down) && (!button3down) ) return;
		target = new DPoint(e1->pos());
		double thisangle = getAngleRad(center, target);
		double d_angle = prevangle - thisangle;
		if (d_angle < -M_PI) d_angle += (2*M_PI);
		if (d_angle > M_PI) d_angle -= (2*M_PI);
	// do the rotate and redraw
		DPoint *tmppt;
		for (tmppt=selectPointList.first(); tmppt!=0; tmppt=selectPointList.next()) {
			double thispointangle = getAngleRad(center, tmppt);
			double trad = sqrt( (tmppt->x() - center->x()) * (tmppt->x() - center->x()) + 
			(tmppt->y() - center->y()) * (tmppt->y() - center->y()) );
			double thisx = tmppt->x() - center->x();
			double thisy = tmppt->y() - center->y();
			double newx = thisx * cos(d_angle) + thisy * sin(d_angle);
			double newy = -thisx * sin(d_angle) + thisy * cos(d_angle);
			tmppt->setX(newx + center->x());
			tmppt->setY(newy + center->y());
		}
		repaint();
		prevangle = thisangle;
	}

	if (opmode == MODE_SELECT) {
		DPoint tmp1, *prevpt;
		tmp1 = e1->pos();
		customObject *prevline;
		prevline = highlighted_line;
		prevpt = highlighted_point;
		highlighted_line = findNearestObjectToPoint(tmp1);
		highlighted_point = findNearestPoint(tmp1, -0.01);
		if (highlighted_line != 0) {
			if (highlighted_line->Type() == MTEXT) highlighted_point = 0;
			if (highlighted_line->Type() == MLABEL) highlighted_point = 0;
			if (highlighted_line->Type() == MSYMBOL) highlighted_point = 0;
		}
		if (highlighted_point != prevpt) repaintflag = true;
		if ((highlighted_line == 0) && (prevline !=0)) {
			prevline->Unhighlight(); repaintflag = true;
		}
		if ((highlighted_point != 0) && (prevline !=0)) {
			prevline->Unhighlight(); repaintflag = true;
		}
		if ((highlighted_point == 0) && (highlighted_line != 0)) {
			if ((highlighted_line != 0) && (prevline != highlighted_line)) { 
				if (prevline != 0) prevline->Unhighlight();
				highlighted_line->Highlight();
				repaintflag = true;
			}
		}
		if (highlighted_line != 0 || highlighted_point != 0) {
			setCursor(pointingHandCursor);
			emit UpdateStatusBar("Left-click to select this object");
		} else {
			setCursor(arrowCursor);
			emit UpdateStatusBar(" ");
		}
		if (repaintflag) repaint();
		return;
	}

	// MODE_TEXT behavior:  if mouse is near an unlabeled point, highlight
  // the point.  If mouse is near a labeled point, highlight the label.
	// If the mouse is near any other object, it is not highlighted.
	// (Only objects which can be labeled should be highlighted)
	if (opmode == MODE_TEXT) {
		DPoint tmp1, *prevpt;
		tmp1 = e1->pos();
		customObject *prevline;
		prevline = highlighted_line;
		prevpt = highlighted_point;
		highlighted_line = findNearestObjectToPoint(tmp1);
		if (highlighted_line !=0) {
			if ((highlighted_line->Type() != MLABEL) &&
				(highlighted_line->Type() != MTEXT))
				{ highlighted_line = 0; }
		}
		highlighted_point = findNearestPoint(tmp1, -0.01);
		if (highlighted_line != 0) highlighted_point = 0;
		if ((highlighted_line == 0) && (prevline !=0)) {
			prevline->Unhighlight(); repaintflag = true;
		}
		if ((highlighted_point != 0) && (prevline !=0)) {
			prevline->Unhighlight(); repaintflag = true;
		}
		if ((highlighted_point == 0) && (highlighted_line != 0)) {
			if ((highlighted_line != 0) && (prevline != highlighted_line)) { 
				if (prevline != 0) prevline->Unhighlight();
				highlighted_line->Highlight(); repaintflag = true; 
			}
		}
		if (prevpt != highlighted_point) repaintflag = true;
		if (repaintflag) repaint();
		return;
	}

	if (opmode == MODE_ERASE) {
		DPoint tmp1 = e1->pos();
		customObject *prevline;
		prevline = highlighted_line;
		Threshold = 2.0;
		highlighted_line = findNearestObjectToPoint(tmp1);
		Threshold = 5.0;
		if ((highlighted_line == 0) && (prevline !=0)) {
			prevline->Unhighlight(); repaint();
		}
		if ((highlighted_line != 0) && (prevline != highlighted_line)) { 
			if (prevline != 0) prevline->Unhighlight();
			highlighted_line->Highlight(); repaint(); 
		}
	}

	if ( (opmode == MODE_DRAW_LINE) || (opmode == MODE_DASH_LINE) ) {
		if (!button1down) {
			DPoint tmp1, *tmp2;
			tmp1 = e1->pos();
			tmp2 = highlighted_point;
			highlighted_point = findNearestPoint(tmp1, -0.01);
			if (highlighted_point != 0) repaint();
			if ((highlighted_point == 0) && (tmp2 != 0)) repaint();
			return;
		}
		repaint();
		double newang;
		target = new DPoint(e1->pos());
		if (freehand == FIXED_DRAWING) {
			double dx = (double)target->x() - origin->x();
			double dy = (double)target->y() - origin->y();
			int mysign = 1;
			if (dx < 0.0) mysign = -1;
			double ang = atan(dy/dx)*MOL_ARAD;
			newang = (double)((int)( (ang + (fixedAngle/2.0)) / fixedAngle ) * fixedAngle);
//			qWarning("Fixed drawing mode - angle %lf - corrected %lf", ang, newang);
			if (abs((int)newang) != 90) {
				dx = mysign * fixedLength * cos(newang / MOL_ARAD);
				dy = mysign * fixedLength * sin(newang / MOL_ARAD);
				target->setX(origin->x() + (int)dx);
				target->setY(origin->y() + (int)dy);
			} else {
				if (dy < 0.0) mysign = -1;
				target->setX(origin->x());
				target->setY(origin->y() + mysign * (int)fixedLength);
			}
		}
		if (opmode == MODE_DRAW_LINE)
			p.setPen(QPen(currentColor, linewidth - 1, SolidLine));
		else
			p.setPen(QPen(currentColor, linewidth - 1, DashLine));
		p.drawLine( origin->toQPoint(), target->toQPoint() );
		QString a, b;
		a.setNum( DoubleToInteger( getAngle(origin, target) ) );
		b.setNum(origin->distanceToPoint(*target));
		emit UpdateStatusBar(QString("Length = ") + b + QString(" pixels, angle = ") + a
			+ QString(" degrees"));
		highlighted_point = findNearestPoint(*target, -0.01);
		if (highlighted_point == 0) {
			targetflag = NEW_POINT;
			highlighted_point = target;
		} else {
			targetflag = OLD_POINT;
			target = highlighted_point;
		}
	}

	if (opmode == MODE_UPLINE) {
		if (!button1down) {
			DPoint tmp1, *tmp2;
			tmp1 = e1->pos();
			tmp2 = highlighted_point;
			highlighted_point = findNearestPoint(tmp1, -0.01);
			if (highlighted_point != 0) repaint();
			if ((highlighted_point == 0) && (tmp2 != 0)) repaint();
			return;
		}
		repaint();
		target = new DPoint(e1->pos());
		if (freehand == FIXED_DRAWING) {
			double dx = (double)target->x() - origin->x();
			double dy = (double)target->y() - origin->y();
			int mysign = 1;
			if (dx < 0.0) mysign = -1;
			double ang = atan(dy/dx)*MOL_ARAD;
			double newang = (double)((int)( (ang + (fixedAngle/2.0)) / fixedAngle ) * fixedAngle);
//			qWarning("Fixed drawing mode - angle %lf - corrected %lf", ang, newang);
			if (abs((int)newang) != 90) {
				dx = mysign * fixedLength * cos(newang / MOL_ARAD);
				dy = mysign * fixedLength * sin(newang / MOL_ARAD);
				target->setX(origin->x() + (int)dx);
				target->setY(origin->y() + (int)dy);
			} else {
				if (dy < 0.0) mysign = -1;
				target->setX(origin->x());
				target->setY(origin->y() + mysign * (int)fixedLength);
			}
		}
		// get angle(s)
		double ang = getAngle(origin, target);
		double dx = (double)target->x() - origin->x();
		double dy = (double)target->y() - origin->y();
		double newang1 = ang + 8.0;
		double newang2 = ang - 8.0;
		double newlen = sqrt(dx*dx + dy*dy);
		double scalefactor = 1.0 / cos(8.0 / MOL_ARAD);
		double mysign = 1.0;
		dx = mysign * newlen * scalefactor * cos(newang1 / MOL_ARAD);
		dy = mysign * newlen * scalefactor * sin(newang1 / MOL_ARAD);
		DPoint t1(origin->x() + (int)dx, origin->y() + (int)dy);
		dx = mysign * newlen * scalefactor * cos(newang2 / MOL_ARAD);
		dy = mysign * newlen * scalefactor * sin(newang2 / MOL_ARAD);
		DPoint t2(origin->x() + (int)dx, origin->y() + (int)dy);

		p.setBrush(QBrush(currentColor));
		QPointArray triangle(3);
		triangle.setPoint(0, t1.toQPoint());
		triangle.setPoint(1, t2.toQPoint());
		triangle.setPoint(2, origin->x(), origin->y());
		p.drawPolygon(triangle);
		QString a, b;
		a.setNum(ang);
		b.setNum(origin->distanceToPoint(*target));
		emit UpdateStatusBar(QString("Length = ") + b + QString(" pixels, angle = ") + a
			+ QString(" degrees"));
		highlighted_point = findNearestPoint(*target, -0.01);
		if (highlighted_point == 0) {
			targetflag = NEW_POINT;
			highlighted_point = target;
		} else {
			targetflag = OLD_POINT;
			target = highlighted_point;
		}
	}

	if (opmode == MODE_DOWNLINE) {
		if (!button1down) {
			DPoint tmp1, *tmp2;
			tmp1 = e1->pos();
			tmp2 = highlighted_point;
			highlighted_point = findNearestPoint(tmp1, -0.01);
			if (highlighted_point != 0) repaint();
			if ((highlighted_point == 0) && (tmp2 != 0)) repaint();
			return;
		}
		repaint();
		double newang;
		target = new DPoint(e1->pos());
		if (freehand == FIXED_DRAWING) {
			double dx = (double)target->x() - origin->x();
			double dy = (double)target->y() - origin->y();
			int mysign = 1;
			if (dx < 0.0) mysign = -1;
			double ang = atan(dy/dx)*MOL_ARAD;
			newang = (double)((int)( (ang + (fixedAngle/2.0)) / fixedAngle ) * fixedAngle);
//			qWarning("Fixed drawing mode - angle %lf - corrected %lf", ang, newang);
			if (abs((int)newang) != 90) {
				dx = mysign * fixedLength * cos(newang / MOL_ARAD);
				dy = mysign * fixedLength * sin(newang / MOL_ARAD);
				target->setX(origin->x() + (int)dx);
				target->setY(origin->y() + (int)dy);
			} else {
				if (dy < 0.0) mysign = -1;
				target->setX(origin->x());
				target->setY(origin->y() + mysign * (int)fixedLength);
			}
		}
		// get angle(s)
		double ang = getAngle(origin, target);
		double dx = (double)target->x() - origin->x();
		double dy = (double)target->y() - origin->y();
		double newang1 = ang + 8.0;
		double newang2 = ang - 8.0;
		double newlen = sqrt(dx*dx + dy*dy);
		double s1 = 1.0 / cos(8.0 / MOL_ARAD);
		s1 *= newlen;

		for (double scalefactor = 1.0; scalefactor < s1; scalefactor = 
			scalefactor + 4.0) {
			dx = scalefactor * cos(newang1 / MOL_ARAD);
			dy = scalefactor * sin(newang1 / MOL_ARAD);
			DPoint t1(origin->x() + (int)dx, origin->y() + (int)dy);
			dx = scalefactor * cos(newang2 / MOL_ARAD);
			dy = scalefactor * sin(newang2 / MOL_ARAD);
			DPoint t2(origin->x() + (int)dx, origin->y() + (int)dy);
			p.drawLine( t1.toQPoint(), t2.toQPoint() );
		}
		dx = s1 * cos(newang1 / MOL_ARAD);
		dy = s1 * sin(newang1 / MOL_ARAD);
		DPoint t1(origin->x() + (int)dx, origin->y() + (int)dy);
		dx = s1 * cos(newang2 / MOL_ARAD);
		dy = s1 * sin(newang2 / MOL_ARAD);
		DPoint t2(origin->x() + (int)dx, origin->y() + (int)dy);
		p.drawLine( t1.toQPoint(), t2.toQPoint() );
		QString a, b;
		a.setNum(ang);
		b.setNum(origin->distanceToPoint(*target));
		emit UpdateStatusBar(QString("Length = ") + b + QString(" pixels, angle = ") + a
			+ QString(" degrees"));

		highlighted_point = findNearestPoint(*target, -0.01);
		if (highlighted_point == 0) {
			targetflag = NEW_POINT;
			highlighted_point = target;
		} else {
			targetflag = OLD_POINT;
			target = highlighted_point;
		}
	}

	if (opmode == MODE_ARROW) {
		if (!button1down) return;
		repaint();
		target = new DPoint(e1->pos());
		p.drawLine( origin->toQPoint(), target->toQPoint() );
		// draw arrowhead
		double dx = (double)origin->x() - target->x();
		double dy = (double)origin->y() - target->y();
		int mysign = 1;
		if (dx < 0.0) mysign = -1;
		double ang = atan(dy/dx)*MOL_ARAD;
		if (ang < 0.0) ang *= -1.0;

	if (origin->x() > target->x() && origin->y() > target->y()) ang = 0.0 + ang;
	if (origin->x() < target->x() && origin->y() > target->y()) ang = 180.0 - ang;
	if (origin->x() < target->x() && origin->y() < target->y()) ang = 180.0 + ang;
	if (origin->x() > target->x() && origin->y() < target->y()) ang = 360.0 - ang;
		if (dx == 0.0) {
			if (dy < 0.0)
				ang = 270.0;
			else
				ang = 90.0;
		}
		if (dy == 0.0) {
			if (dx < 0.0)
				ang = 180.0;
			else
				ang = 0.0;
		}

		double newang1 = ang + 30.0;
		double newang2 = ang - 30.0;

		DPoint a1(target->x() + (int)(cos(newang1/MOL_ARAD) * 10.0),
							target->y() + (int)(sin(newang1/MOL_ARAD) * 10.0));
		DPoint a2(target->x() + (int)(cos(newang2/MOL_ARAD) * 10.0),
							target->y() + (int)(sin(newang2/MOL_ARAD) * 10.0));
		p.drawLine( target->toQPoint(), a1.toQPoint() );
		p.drawLine( target->toQPoint(), a2.toQPoint() );
	}
}

double Render2D::getAngle(DPoint *a, DPoint *b)
{
	double dx = (double)b->x() - a->x();
	double dy = (double)b->y() - a->y();
	double ang = atan(dy/dx)*MOL_ARAD;
	if (ang < 0.0) ang *= -1.0;

	if (b->x() > a->x() && b->y() > a->y()) ang = 0.0 + ang;
	if (b->x() < a->x() && b->y() > a->y()) ang = 180.0 - ang;
	if (b->x() < a->x() && b->y() < a->y()) ang = 180.0 + ang;
	if (b->x() > a->x() && b->y() < a->y()) ang = 360.0 - ang;
	if (dx == 0.0) {
		if (dy < 0.0)
			ang = 270.0;
		else
			ang = 90.0;
	}
	if (dy == 0.0) {
		if (dx < 0.0)
			ang = 180.0;
		else
			ang = 0.0;
	}

	return ang;
}

double Render2D::getAngleRad(DPoint *a, DPoint *b)
{
	double dx = (double)b->x() - a->x();
	double dy = (double)b->y() - a->y();
	double ang = atan(dy/dx);
	if (ang < 0.0) ang *= -1.0;

	if (b->x() > a->x() && b->y() > a->y()) ang = 0.0 + ang;
	if (b->x() < a->x() && b->y() > a->y()) ang = M_PI - ang;
	if (b->x() < a->x() && b->y() < a->y()) ang = M_PI + ang;
	if (b->x() > a->x() && b->y() < a->y()) ang = 2.0 * M_PI - ang;
	if (dx == 0.0) {
		if (dy < 0.0)
			ang = 3.0 * M_PI / 2.0;
		else
			ang = M_PI / 2.0;
	}
	if (dy == 0.0) {
		if (dx < 0.0)
			ang = M_PI;
		else
			ang = 0.0;
	}

	return ang;
}

// ws = which symbol, x1 = x-coordinate, y1 = y-coordinate
void Render2D::addSymbol(int ws, int x1, int y1)
{
	DPoint *tpt;

	deselectAll();

	tpt = new DPoint(x1,y1);
	pointList.append(tpt); selectPointList.append(tpt);
	customObject *tmp1 = new customObject;
	tmp1->setType(MSYMBOL);
	tmp1->addPoint(tpt);
	tmp1->setSymbol(ws-1);
	tmp1->Highlight();
	objectList.append(tmp1); selectList.append(tmp1);

	opmode = MODE_MULTIPLE_SELECTION;
	resizeOrigin = new DPoint(x1, y1);
	resizeHandle = new DPoint(x1 + 15.0, y1 + 15.0);
	
	repaint();
}
