#ifndef RENDER2D_H
#define RENDER2D_H

#include <qwidget.h>
#include <qlist.h>
#include <qpixmap.h>

#include "customObject.h"

class QPainter;
class QPrinter;

class XNode {
public:
  DPoint *thisnode;
  QList<DPoint> connected;
};

class Render2D : public QWidget
{
	Q_OBJECT
public:
// These are defined in render2d.cpp
	Render2D(QWidget *parent=0, const char *name=0);
	QSizePolicy sizePolicy() const;
	DPoint* findNearestPoint(DPoint, double);
	customObject* findNearestObjectToPoint(DPoint);
	int load(const char *);
	int save(const char *);
	int savePicture(QString, int itype);
	int SaveEPS(QString);
	int SaveMDLMolfile(QString);
	int LoadMDLMolfile(QString);
	int SaveCMLFile(QString);
	int LoadCMLFile(QString);
	double calcPointToLine(DPoint *, DPoint *, DPoint);
	int selectedObjectNearPoint(DPoint);
	void Render(customObject *, int);
	void Render(int);
	int changed(void);
	double getAngle(DPoint *, DPoint *);
	double getAngleRad(DPoint *, DPoint *);
	void addRing(int, int, int, QString);  // defined in rings.cpp
	void addSymbol(int, int, int);
	void setLW(int);
	void setFL(double);
	void setFA(double);
	void setFontSize(int);
	int getPaperSize(void) { return psize; }
	int getOrientation(void) { return porient; }
	void setSizeAndOrientation(int, int);
	void setPaperSize(int, int);
// These are defined in processing.cpp
	void Cleanup(void);
	int Group(void);
	int GroupAll(void);
	void Align(int);
	void Resize(double, int global = false);
	void Flip(int);
	void Hydrogens(void);
// Defined in action.cpp
	void Action(int);
	void AddCoordinates(int);
	int DoubleToInteger(double);
	int pageWidth() { return pagewidth; }
	int pageHeight() { return pageheight; }
	int WidthOfString(QString, QFont);
	int HeightOfString(QString, QFont);
	bool TraverseXNodes(XNode *, XNode *);

public slots:
	void saveRing(void);
	void selectAll(void);
	void deselectAll(void);
	void Cut(void);
	void Copy(void);
	void Paste(void);
	void Clear(void);
	void Rotate(void);
	void Color(QColor);
	void setMode(int);
	void setParameter(int);
	void Print();
	void setFontHelv();
	void setFontTimes();
	void setFontCourier();
	void setFont8();
	void setFont10();
	void setFont12();
	void setFont14();
	void setFont18();
	void setFont24();
	void setLW1();
	void setLW2();
	void setLW3();
	void setLW4();
	void setLW5();
	void setFL25();
	void setFL50();
	void setFL_Quarter();
	void setFL_Half();
	void setFA10();
	void setFA15();
	void setFA30();
	void setColorBlack();
	void setColorRed();
	void setColorGreen();
	void setColorBlue();
	void FlipH();
	void FlipV();
	void AutoFormat();
	void Undo();
	// defined in processing.cpp; invoked from ApplicationWindow menu
	void FindMW(void);
	void showInfo(); // this is in infodialog.cpp
	void TEST();

signals:
	void UpdateStatusBar(QString);
	void ChooseFont(int);
	void ChooseSize(int);

protected:
	void mousePressEvent(QMouseEvent *);
	void mouseReleaseEvent(QMouseEvent *);
	void mouseMoveEvent(QMouseEvent *);
	void paintEvent(QPaintEvent *);

private:
	int vchanged;
	double Threshold;  // for snapping to points (findNearestPoint)
	double fixedLength;  // length of fixed-length line segments
  double fixedAngle;  // angle for fixed-angle drawing
	DPoint *points;
	DPoint *origin, *target;
	bool button1down, button3down;
	bool firsttime;
	RasterOp saveraster;
	customObject *test1;
	QList<customObject> objectList;
	QList<customObject> selectList;
	QList<DPoint> pointList;
	QList<DPoint> selectPointList;
	QList<customObject> groupList[16];
	int opmode;
	int point_index;
	int is_point_highlighted;
	int freehand;
	DPoint *highlighted_point;
	DPoint *snappoint;
	int highlighted_point_index;
	customObject *highlighted_line;
	int highlighted_line_index;
	int originflag, targetflag, dragindex, snapindex;
	QCursor skullCursor, rotateCursor;
	int has_moved;
	QPixmap *rendermap;
	QPrinter *printer;
	int pagewidth, pageheight;
	int psize, porient;
	QFont currentFont;
	QColor currentColor;
	int linewidth;
	double pixelsPerMM;
	DPoint *center;
	// For resizing selections
	DPoint *resizeOrigin, *resizeHandle;
	int nearResizeHandle;
	double prevangle;
	// for info window (also stored in save files)
	QString MoleculeTitle;
	QString MoleculeId;
	QString GenerationDate;
	QString GenerationSoftware;

	// consistent temporary objects
	customObject *tmp_obj;
	DPoint *tmp_pt;
	QString tmp_text;
};

#endif  // ifdef RENDER2D_H

