#include <qwidget.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include <qcolor.h>
#include <qfont.h>
#include <qnamespace.h>
#include <qpixmap.h>
#include <qcombobox.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qtooltip.h>

#include "ringdialog.h"
#include "defs.h"

#include "benzene.xpm"
#include "cyclohexane.xpm"
#include "cyclopentane.xpm"
#include "calix4.xpm"
#include "imidazole.xpm"

RingDialog::RingDialog(QWidget *parent, QString name):
	QDialog(parent, name, TRUE)
{
	// Init variables which tell which ring was picked
	fn = QString("noring");
	whichring = 0;

	QPixmap pix;
	QPushButton *ringbutton;
	QLabel *caption;
	QPushButton *dismiss;
	setBackgroundColor(lightGray);
	setCaption("Ring tool");

	QGridLayout *mygrid = new QGridLayout(this,3,5,5);

	caption = new QLabel("Choose a ring:", this);
	caption->setBackgroundColor(lightGray);
	mygrid->addMultiCellWidget(caption, 0, 0, 0, 3);

	pix = QPixmap(cyclopentane);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(lightGray));
	ringbutton->setPixmap(pix);
	QToolTip::add(ringbutton, "Cyclopentane");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Cyclopentane()) );
	mygrid->addWidget(ringbutton, 1, 0);

	pix = QPixmap(imidazole_xpm);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(lightGray));
	ringbutton->setPixmap(pix);
	QToolTip::add(ringbutton, "Imidazole");
	connect(ringbutton, SIGNAL(clicked()), SLOT(Imidazole()) );
	mygrid->addWidget(ringbutton, 1, 1);

	pix = QPixmap(cyclohexane);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(lightGray));
	ringbutton->setPixmap(pix);
	QToolTip::add(ringbutton, "Cyclohexane");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Cyclohexane()) );
	mygrid->addWidget(ringbutton, 1, 2);

	pix = QPixmap(benzene);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(lightGray));
	ringbutton->setPixmap(pix);
	QToolTip::add(ringbutton, "Benzene");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Benzene()) );
	mygrid->addWidget(ringbutton, 1, 3);

	pix = QPixmap(calix4);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(lightGray));
	ringbutton->setPixmap(pix);
	QToolTip::add(ringbutton, "Calix[4]arene");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Calix4()) );
	mygrid->addWidget(ringbutton, 1, 4);

  dismiss = new QPushButton("Dismiss", this);
  dismiss->setPalette(QPalette(lightGray));
  connect(dismiss, SIGNAL(clicked()), SLOT(reject()) );
	mygrid->addMultiCellWidget(dismiss, 2, 2, 1, 2);
}

int RingDialog::getRing(void)
{
	return whichring;
}

void RingDialog::Benzene(void)
{
	whichring = M_BENZENE;
	accept();
}

void RingDialog::Cyclohexane(void)
{
	whichring = M_CYCLOHEXANE;
	accept();
}

void RingDialog::Cyclopentane(void)
{
	whichring = M_CYCLOPENTANE;
	accept();
}

void RingDialog::Calix4(void)
{
	whichring = M_USERDEFINED;
	fn = "calix4.ring";
	accept();
}

void RingDialog::Imidazole(void)
{
	whichring = M_USERDEFINED;
	fn = "imidazole.ring";
	accept();
}

void RingDialog::fromComboBox(int i)
{
	whichring = M_USERDEFINED;
	switch (i) {
	case 0:
		return;
		break;  // just to be safe...
	case 1:
		fn = "calix4.ring";
		break;
	case 2:
		fn = "naphthalene.ring";
		break;
	}
	accept();
}
