// rwfile.cpp - code to read and write files

#include <iostream.h>
#include <qpainter.h>
#include <qimage.h>
#include <qevent.h>
#include <qlist.h>
#include <math.h>
#include <qbitmap.h>
#include <qcursor.h>
#include <qdialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpointarray.h>
#include <qprinter.h>
#include <qmessagebox.h>
#include <qpaintdevicemetrics.h>
#include <qnamespace.h>
#include <qregexp.h>

#include "defs.h"
#include "customObject.h"
#include "dpoint.h"
#include "render2d.h"

// Load native file format
int Render2D::load(const char *fn) {
  deselectAll();
  pointList.clear();
  objectList.clear();
  
  if (QString(fn).right(4).lower() == QString(".mol")) {
    return LoadMDLMolfile(QString(fn));
  }
  if (QString(fn).right(4).lower() == QString(".mdl")) {
    return LoadMDLMolfile(QString(fn));
  }

  if (QString(fn).right(4).lower() == QString(".cml")) {
    return LoadCMLFile(QString(fn));
  }
  if (QString(fn).right(4).lower() == QString(".xml")) {
    return LoadCMLFile(QString(fn));
  }

  QFile f(fn);
  if (!f.open(IO_ReadOnly)) {
    QMessageBox::warning(this, "Couldn't read file", 
			 "Unable to open the specified file.");
    return false;
  }
  QTextStream t(&f);
  QString line = t.readLine();
  QString tmpline, tfont, q1;
  DPoint *newpt;
  customObject *newobj;
  int d1, d2, d3, d4, d5, tpt;
  double l1, l2, l3, l4;
  if (line == "XDRAWCHEM-1.0") {
    // Reset internal data structures
    do {
      line = t.readLine();
      QTextIStream(&line) >> tmpline;
      if (tmpline == "DEFAULTFONT") {
	QTextIStream(&line) >> tmpline >> tfont >> d1;
	currentFont.setFamily(tfont);
	currentFont.setPointSize(d1);
      }
      if (tmpline == "FIXEDLENGTH") {
	QTextIStream(&line) >> tmpline >> l1;
	fixedLength = l1;
      }
      if (tmpline == "POINT") {
	QTextIStream(&line) >> tmpline >> d1 >> l2 >> l3 >> d2 >> d3;
	// qWarning("read POINT %d %d %d", d1, d2, d3);
	newpt = new DPoint(l2, l3);
	pointList.append(newpt);
      }
      if (tmpline == "LINE") {
	QTextIStream(&line) >> tmpline >> d1 >> d2 >> d3 >> d4 >> d5;
	// qWarning("read LINE %d %d %d", d1, d2, d3);
	newobj = new customObject;
	newobj->setType(MLINE);
	newobj->addPoint(pointList.at(d1));
	newobj->addPoint(pointList.at(d2));
	pointList.at(d1)->isAtom(true);
	pointList.at(d2)->isAtom(true);
	newobj->setThickness(d3);
	newobj->setLineWidth(d4);
	newobj->setDash(d5);
	objectList.append(newobj);
      }
      if (tmpline == "UPLINE") {
	QTextIStream(&line) >> tmpline >> d1 >> d2 >> d3;
	// qWarning("read UPLINE %d %d %d", d1, d2, d3);
	newobj = new customObject;
	newobj->setType(MUPLINE);
	newobj->addPoint(pointList.at(d1));
	newobj->addPoint(pointList.at(d2));
	pointList.at(d1)->isAtom(true);
	pointList.at(d2)->isAtom(true);
	newobj->setThickness(d3);
	objectList.append(newobj);
      }
      if (tmpline == "DOWNLINE") {
	QTextIStream(&line) >> tmpline >> d1 >> d2 >> d3;
	// qWarning("read DOWNLINE %d %d %d", d1, d2, d3);
	newobj = new customObject;
	newobj->setType(MDOWNLINE);
	newobj->addPoint(pointList.at(d1));
	newobj->addPoint(pointList.at(d2));
	pointList.at(d1)->isAtom(true);
	pointList.at(d2)->isAtom(true);
	newobj->setThickness(d3);
	objectList.append(newobj);
      }
      if (tmpline == "SYMBOL") {
	QTextIStream(&line) >> tmpline >> d1 >> d2;
	newobj = new customObject;
	newobj->setType(MSYMBOL);
	newobj->addPoint(pointList.at(d1));
	pointList.at(d1)->isAtom(false);
	newobj->setSymbol(d2);
	objectList.append(newobj);
      }
      if (tmpline == "ARROW") {
	QTextIStream(&line) >> tmpline >> d1 >> d2;
	// qWarning("read ARROW %d %d", d1, d2);
	newobj = new customObject;
	newobj->setType(MARROW);
	newobj->addPoint(pointList.at(d1));
	newobj->addPoint(pointList.at(d2));
	objectList.append(newobj);
      }
      if (tmpline == "BRACKET") {
	QTextIStream(&line) >> tmpline >> d1 >> d2;
	newobj = new customObject;
	newobj->setType(MBRACKET);
	newobj->addPoint(pointList.at(d1));
	newobj->addPoint(pointList.at(d2));
	objectList.append(newobj);
      }
      if (tmpline == "TEXT") {
	QTextIStream(&line) >> tmpline >> d1 >> tfont >> tpt;
	tmpline = line.mid(line.find('`') + 1);
	// qWarning("read TEXT %d %s", d1, tmpline.ascii());
	newobj = new customObject;
	newobj->setType(MTEXT);
	newobj->addPoint(pointList.at(d1));
	tmpline.replace(QRegExp("~"), QString("\n"));
	newobj->setLabel(tmpline);
	newobj->setFont(QFont(tfont, tpt));
	objectList.append(newobj);
      }
      if (tmpline == "LABEL") {
	QTextIStream(&line) >> tmpline >> d1 >> tfont >> tpt;
	tmpline = line.mid(line.find('`') + 1);
	// qWarning("read LABEL %d %s", d1, tmpline.ascii());
	newobj = new customObject;
	newobj->setType(MLABEL);
	newobj->addPoint(pointList.at(d1));
	newobj->setLabel(tmpline);
	newobj->setFont(QFont(tfont, tpt));
	objectList.append(newobj);
	pointList.at(d1)->setName(tmpline);
      }
    } while (!t.atEnd());
    f.close();
    repaint();
    emit UpdateStatusBar(QString("Read file ") + QString(fn));
    return true;
  }
  f.close();
  QMessageBox::warning(this, "Can't read file", "Unknown file format.");
  return false;
}

// Save native file format
int Render2D::save(const char *fn) {
  // clean up data structures to avoid saving useless stuff
  Cleanup();

  if (QString(fn).right(4) == QString(".mol")) {
    return SaveMDLMolfile(QString(fn));
  }

  if (QString(fn).right(4) == QString(".cml")) {
    return SaveCMLFile(QString(fn));
  }

  int tmp;
  QFile f(fn);
  if (f.open(IO_WriteOnly)) {
    QTextStream t(&f);
    t << "XDRAWCHEM-1.0" << endl;
    DPoint *tmp1;
    t << "DEFAULTFONT " << currentFont.family() << " ";
    t << currentFont.pointSize() << endl;
    t << "FIXEDLENGTH " << fixedLength << endl;
    for (tmp1 = pointList.first(); tmp1 != 0; tmp1 = pointList.next()) {
      t << "POINT " << (int)pointList.at() << " ";
      t << tmp1->x() << " " << tmp1->y() << " ";
      // Write element and group info
      t << "0 0" << endl;
    }
    customObject *tmp2;
    for (tmp2 = objectList.first(); tmp2 != 0; tmp2 = objectList.next()) {
      if (tmp2->Type() == MLINE) {
	t << "LINE " << pointList.find(tmp2->getPoint1()) << " ";
	t << pointList.find(tmp2->getPoint2()) << " " << tmp2->Thickness();
	t << " " << tmp2->LineWidth() << " " << tmp2->Dash() << endl;
      }
      if (tmp2->Type() == MUPLINE) {
	t << "UPLINE " << pointList.find(tmp2->getPoint1()) << " ";
	t << pointList.find(tmp2->getPoint2()) << " " << tmp2->Thickness();
	t << endl;
      }
      if (tmp2->Type() == MDOWNLINE) {
	t << "DOWNLINE " << pointList.find(tmp2->getPoint1()) << " ";
	t << pointList.find(tmp2->getPoint2()) << " " << tmp2->Thickness();
	t << endl;
      }
      if (tmp2->Type() == MSYMBOL) {
	t << "SYMBOL " << pointList.find(tmp2->getPoint1()) << " ";
	t << tmp2->Symbol();
	t << endl;
      }
      if (tmp2->Type() == MARROW) {
	t << "ARROW " << pointList.find(tmp2->getPoint1()) << " ";
	t << pointList.find(tmp2->getPoint2());
	t << endl;
      }
      if (tmp2->Type() == MBRACKET) {
	t << "BRACKET " << pointList.find(tmp2->getPoint1()) << " ";
	t << pointList.find(tmp2->getPoint2());
	t << endl;
      }
      if (tmp2->Type() == MTEXT) {
	t << "TEXT ";
	t << pointList.find(tmp2->getPoint1());
	t << " ";
	t << tmp2->font.family();
	t << " ";
	t << tmp2->font.pointSize();
	t << " `";
	QString nonbreaking = tmp2->Label();
	// convert CR's to another character to save on one line
	nonbreaking.replace(QRegExp("\n"), QString("~"));
	t << nonbreaking;
	t << endl;
      }
      if (tmp2->Type() == MLABEL) {
	t << "LABEL ";
	t << pointList.find(tmp2->getPoint1());
	t << " ";
	t << tmp2->font.family();
	t << " ";
	t << tmp2->font.pointSize();
	t << " `";
	t << tmp2->Label();
	t << endl;
      }
    }
    f.close();
    vchanged = false;
    emit UpdateStatusBar(QString("Saved file ") + QString(fn));
  } else {
    QMessageBox::warning(this, "Couldn't save file",
			 "Couldn't save file.");
    return false;
  }

  return true;
}

// Save a ring template (essentailly a simplified form of save() above)
void Render2D::saveRing(void)
{
  QString fn("1.ring");
  // clean up data structures to avoid saving useless stuff
  Cleanup();

  // Find minimum x and y -- have to translate this object so it's at (0,0)
  double dx = 9999.0, dy = 9999.0;
  DPoint *tmp1;
  for (tmp1 = pointList.first(); tmp1 != 0; tmp1 = pointList.next()) {
    if (tmp1->x() < dx) dx = tmp1->x();
    if (tmp1->y() < dy) dy = tmp1->y();
  }
  // calculate scale factor
  double sf = fixedLength / 10.0;

  int tmp;
  QFile f(fn);
  if (f.open(IO_WriteOnly)) {
    QTextStream t(&f);
    t << "XDRAWCHEM-TEMPLATE-1.0" << endl;
    for (tmp1 = pointList.first(); tmp1 != 0; tmp1 = pointList.next()) {
      t << "POINT " << (int)pointList.at() << " ";
      // Remember to translate point to origin
      t << (tmp1->x() - dx)/sf << " " << (tmp1->y() - dy)/sf << endl;
    }
    customObject *tmp2;
    for (tmp2 = objectList.first(); tmp2 != 0; tmp2 = objectList.next()) {
      // only save line types here
      if (tmp2->Type() == MLINE) {
	t << "LINE " << pointList.find(tmp2->getPoint1()) << " ";
	t << pointList.find(tmp2->getPoint2()) << " " << tmp2->Thickness();
	t << " " << tmp2->LineWidth() << endl;
      }
      if (tmp2->Type() == MUPLINE) {
	t << "UPLINE " << pointList.find(tmp2->getPoint1()) << " ";
	t << pointList.find(tmp2->getPoint2()) << " " << tmp2->Thickness();
	t << endl;
      }
      if (tmp2->Type() == MDOWNLINE) {
	t << "DOWNLINE " << pointList.find(tmp2->getPoint1()) << " ";
	t << pointList.find(tmp2->getPoint2()) << " " << tmp2->Thickness();
	t << endl;
      }
      if (tmp2->Type() == MLABEL) {
	t << "LABEL ";
	t << pointList.find(tmp2->getPoint1());
	t << " ";
	t << tmp2->font.family();
	t << " ";
	t << tmp2->font.pointSize();
	t << " `";
	t << tmp2->Label();
	t << endl;
      }
    }
    f.close();
    emit UpdateStatusBar(QString("Saved ring ") + QString(fn));
  } else {
    QMessageBox::warning(this, "Couldn't save template",
			 "Couldn't save template.");
  }
}

// Save picture (format specified by itype, default Windows bitmap (*.bmp))
int Render2D::savePicture(QString filename, int itype)
{
  if (itype == IMG_EPS) {
    return SaveEPS(filename);
  }

  rendermap->fill();

  int xmin = 99999, xmax = 0, ymin = 99999, ymax = 0, w, h;
  DPoint *tmp;
  customObject *tmp2;
  if (selectList.count() > 0) {
    for (tmp = selectPointList.first(); tmp != 0; 
	 tmp = selectPointList.next() ) {
      if (tmp->x() < xmin) xmin = (int)tmp->x();
      if (tmp->x() > xmax) xmax = (int)tmp->x();
      if (tmp->y() < ymin) ymin = (int)tmp->y();
      if (tmp->y() > ymax) ymax = (int)tmp->y();
    }
    for (tmp2 = selectList.first(); tmp2 != 0; tmp2 = selectList.next() ) {
      tmp2->Unhighlight();
    }
    for (tmp2=selectList.first(); tmp2 != 0; tmp2=selectList.next()) {
      if (tmp2->Type() != MLABEL) Render(tmp2, 1);
    }
    // now draw labels, so they will appear on top
    for (tmp2=selectList.first(); tmp2 != 0; tmp2=selectList.next()) {
      if (tmp2->Type() == MLABEL) Render(tmp2, 1);
    }
    for (tmp2 = selectList.first(); tmp2 != 0; tmp2 = selectList.next() ) {
      tmp2->Highlight();
    }
  } else {
    for (tmp = pointList.first(); tmp != 0; tmp = pointList.next() ) {
      if (tmp->x() < xmin) xmin = (int)tmp->x();
      if (tmp->x() > xmax) xmax = (int)tmp->x();
      if (tmp->y() < ymin) ymin = (int)tmp->y();
      if (tmp->y() > ymax) ymax = (int)tmp->y();
    }
    for (tmp2=objectList.first(); tmp2 != 0; tmp2=objectList.next()) {
      if (tmp2->Type() != MLABEL) Render(tmp2, 1);
    }
    // now draw labels, so they will appear on top
    for (tmp2=objectList.first(); tmp2 != 0; tmp2=objectList.next()) {
      if (tmp2->Type() == MLABEL) Render(tmp2, 1);
    }
  }
  xmin -= 50; if (xmin < 0) xmin = 0;
  xmax += 50; if (xmax > 849) xmax = 849;
  ymin -= 25; if (ymin < 0) ymin = 0;
  ymax += 25; if (ymax > 1099) ymax = 1099;
  w = xmax - xmin;
  h = ymax - ymin;

  // Copy and save desired region
  QPixmap saveimg(w,h);
  bitBlt(&saveimg, 0, 0, rendermap, xmin, ymin, w, h);
  switch (itype) {
  case IMG_BMP:
    if (!saveimg.save(filename, "BMP")) {
      QMessageBox::warning(this, "Couldn't save picture",
			   "Could not save the picture to the specified file.");
      return true;
    }
    break;

  case IMG_PNG:
    if (!saveimg.save(filename, "PNG")) {
      QMessageBox::warning(this, "Couldn't save picture",
			   "Could not save the picture to the specified file.");
      return true;
    }
    break;
  }
  return false;
}

// Save EPS file.
int Render2D::SaveEPS(QString f1) 
{
  qWarning("Don't know how to save EPS files yet.");
  return true;
}

// MDL Molfile format meets December 1998 "CTFile Formats" publication by MDL.
// This document is available at http://www.mdli.com in the free downloads
// section
int Render2D::SaveMDLMolfile(QString fn) 
{
  int acount = 0, bcount = 0;
  DPoint *tmppt;
  customObject *tmpobj;
  QList<DPoint> atoms;
  QString tmpline;
  QFile f(fn);
  if (!f.open(IO_WriteOnly)) {
    QMessageBox::warning(this, "Can't save file", 
			 "Can't save the specified file.");		
    return false;
  }
  QTextStream t(&f);
  // write first three lines -- blank, unless we want to add comments etc.
  t << endl;
  t << endl;
  t << endl;
  // find counts...
  for (tmppt = pointList.first(); tmppt != 0; tmppt = pointList.next() ) {
    if (tmppt->isAtom()) { acount++; atoms.append(tmppt); }
  }
  for (tmpobj = objectList.first(); tmpobj != 0; tmpobj = objectList.next() ) {
    if (tmpobj->Type() == MLINE) bcount++;
    if (tmpobj->Type() == MUPLINE) bcount++;
    if (tmpobj->Type() == MDOWNLINE) bcount++;
  }
  // write counts line
  t.width(3);
  t << acount;
  t.width(3);
  t << bcount << "  0  0  0  0  0  0  0  0  1" << endl;
  // write atoms 
  for (tmppt = atoms.first(); tmppt != 0; tmppt = atoms.next()) {
    tmpline = tmppt->Name();
    if (tmpline.length() < 3) tmpline.append(' ');
    if (tmpline.length() < 3) tmpline.append(' ');
    t.width(10);
    t << tmppt->x();
    t.width(10);
    t << -tmppt->y() << "    0.0000 " << tmpline;
    t << " 0  0  0  0  0  0  0  0  0  0  0  0" << endl;
  }
  // write bonds
  for (tmpobj = objectList.first(); tmpobj != 0; tmpobj = objectList.next() ) {
    if ((tmpobj->Type() < 100) || (tmpobj->Type() > 120)) continue;
    t.width(3);
    t << atoms.find( tmpobj->getPoint1() ) + 1;
    t.width(3);
    t << atoms.find( tmpobj->getPoint2() ) + 1;
    t.width(3);
    t << tmpobj->Thickness();
    if (tmpobj->Type() == MLINE)
      t << "  0  0  0  0" << endl;
    if (tmpobj->Type() == MUPLINE)
      t << "  1  0  0  0" << endl;
    if (tmpobj->Type() == MDOWNLINE)
      t << "  6  0  0  0" << endl;
  }

  // end of file -- write M END record
  t << "M  END" << endl;
  f.close();
  return true;
}

// MDL Molfile format meets December 1998 "CTFile Formats" publication by MDL.
// This document is available at http://www.mdli.com in the free downloads
// section
int Render2D::LoadMDLMolfile(QString fn)
{
  QFile f(fn);
  if (!f.open(IO_ReadOnly)) {
    QMessageBox::warning(this, "Can't read file", 
			 "Can't open the specified file.");
    return false;
  }
  // molfiles are formatted text
  QTextStream t(&f);

  QString tmpline, tfont;
  DPoint *newpt, *tmppt;
  customObject *newobj, *tmpobj;
  int atomcount, bondcount;
  int d1, d2, d3, d4, tpt, n;
  double l1, l2, l3, l4;
  // first line contains molecule name.  If it starts with $, that probably
  // indicates an MDL file format other than molfile.
  QString line = t.readLine();
  if (line.left(1) == QString("$")) {
    QMessageBox::warning(this, "Invalid file format", 
			 "Can't read this type of MDL file!");
    return false;
  }
  // Second and third lines contain optional info.  Discard for now
  line = t.readLine();
  line = t.readLine();
  // Fourth line contains counts -- how many atoms, bonds, etc.  Parse values
  line = t.readLine();
  atomcount = line.mid(1, 3).toShort();
  bondcount = line.mid(4, 3).toShort();

  // read in atom lines
  for (n = 0; n < atomcount; n++) {
    line = t.readLine();
    newpt = new DPoint();
    newpt->setX( line.mid( 1, 10).toDouble() );
    newpt->setY( -line.mid(11, 10).toDouble() );
    newpt->isAtom(true);
    tmpline = line.mid(31,3);
    tmpline.stripWhiteSpace();
    newpt->setName(tmpline);
    pointList.append(newpt);
    if (newpt->Name() != QString("C")) {
      newobj = new customObject;
      newobj->setType(MLABEL);
      newobj->setLabel(tmpline);
      newobj->setFont(QFont("Helvetica", 12));
      newobj->addPoint(newpt);
      objectList.append(newobj);
    }
  }
  // now read in bonds - link to atoms read above
  for (n = 0; n < bondcount; n++) {
    line = t.readLine();
    newobj = new customObject;
    newobj->setType(MLINE);
    d1 = line.mid(1,3).toShort();
    d2 = line.mid(4,3).toShort();
    d3 = line.mid(7,3).toShort();
    d4 = line.mid(10,3).toShort();
    // remember, QList starts at 0, but atom list in molfile assumed to start
    // at 1, so offset accordingly!
    newobj->addPoint(pointList.at(d1 - 1));
    newobj->addPoint(pointList.at(d2 - 1));
    newobj->setThickness(d3);
    if (d4 == 1) newobj->setType(MUPLINE);
    if (d4 == 6) newobj->setType(MDOWNLINE);
    objectList.append(newobj);
  }

  // scale the drawing to match current fixed length
  double x1 = newobj->getPoint1()->x();
  double y1 = newobj->getPoint1()->y();
  double x2 = newobj->getPoint2()->x();
  double y2 = newobj->getPoint2()->y();
  double dx = x2 - x1; double dy = y2 - y1;
  double length1 = sqrt(dx*dx + dy*dy);
  double sf = fixedLength / length1;

  for (tmppt = pointList.first(); tmppt != 0; tmppt = pointList.next() ) {
    tmppt->setX(tmppt->x() * sf);
    tmppt->setY(tmppt->y() * sf);
  }
  // translate the drawing so it fits in the page!
  double xmin = 99999.0, xmax = 0.0, ymin = 99999.0, ymax = 0.0;
  for (tmppt = pointList.first(); tmppt != 0; tmppt = pointList.next() ) {
    if (tmppt->x() < xmin) xmin = tmppt->x();
    if (tmppt->x() > xmax) xmax = tmppt->x();
    if (tmppt->y() < ymin) ymin = tmppt->y();
    if (tmppt->y() > ymax) ymax = tmppt->y();
  }
  xmin *= -1.0; ymin *= -1.0;
  if (xmin < 0.0) xmin = 0.0;
  if (ymin < 0.0) ymin = 0.0;
  for (tmppt = pointList.first(); tmppt != 0; tmppt = pointList.next() ) {
    // add 20 to offsets for good measure (and to make selecting easier)
    tmppt->setX(tmppt->x() + xmin + 20.0);
    tmppt->setY(tmppt->y() + ymin + 20.0);
  }

  f.close();
  repaint();
  return true;
}

