#include <unistd.h>
#include <iostream.h>
#include <qpainter.h>
#include <qimage.h>
#include <qevent.h>
#include <qlist.h>
#include <qstack.h>
#include <math.h>
#include <qbitmap.h>
#include <qcursor.h>
#include <qdialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpointarray.h>
#include <qprinter.h>
#include <qmessagebox.h>
#include <qpaintdevicemetrics.h>
#include <qnamespace.h>
#include <qregexp.h>
#include <qdatetime.h>

#include "defs.h"
#include "customObject.h"
#include "dpoint.h"
#include "render2d.h"
#include "data.h"

// temporary data structures to hold data in CML formats
class CML_Atom {
public:
  QString id;
  QString element;
  int formalCharge;
  double x, y, z;
  DPoint *toDPoint() {
    DPoint * d = new DPoint(x, y);
    d->setName(element);
    d->id = id;
    d->isAtom(TRUE);
    return d;
  }
};

class CML_Bond {
public:
  QString id;
  QString a1;
  QString a2;
  int order;
};

QList<CML_Atom> CML_Atoms;
QList<CML_Atom> CML_Atoms_Sub;
QList<CML_Bond> CML_Bonds;
QList<CML_Bond> CML_Bonds_Sub;

CML_Atom *tmp_atom;
CML_Bond *tmp_bond;

// track what tag(s) we're in
#define CML_MOLECULE 10
#define CML_ATOM   2
#define CML_BOND   3
#define CML_ATOMARRAY 4
#define CML_BONDARRAY 5
QList<int> TagStack;

// Static function to extract tags from stream
QString ReadTag(QString f, int &ptr) {
  int t1 = f.find("<", ptr);
  ptr = t1;
  int t2 = f.find(">", ptr);
  ptr = t2 + 1;
  return f.mid(t1, ptr - t1).ascii();
}

// Static function to extract data between tags
QString ReadData(QString f, int &ptr) {
  int t1 = f.find("<", ptr);
  return f.mid(ptr, t1 - ptr);
}

// Static function to extract attributes from tag
// Return format:  list item 0=first attribute, 1=value of first attribute,
//   2=second attribute, 3=value of second attribute, you get it, right?
QStringList ReadAttr(QString tag) {
  QStringList l1;
  QString attr;
  QString a1, v1;
  int t1, eq1, te;
  // where to start tokenizing
  int ptr = tag.find(QString(" "));
  // where to end tokenizing
  int eol = tag.find(QString(">"));
  
  do {
    t1 = tag.find(QString("\""), ptr);
    te = tag.find(QString("\""), t1 + 1);
    attr = tag.mid(ptr + 1, te - ptr);
    ptr = te + 1;
    eq1 = attr.find(QString("="));
    a1 = attr.left(eq1);
    v1 = attr.right(attr.length() - eq1 - 1);
    v1 = v1.mid(1, v1.length() - 2);
    l1.append(a1); l1.append(v1);
  } while (ptr < eol);

  return l1;
}

// Static function to tokenize
QStringList Tokenize(QString tag) {
  QStringList l1;
  QString a1, tag1;
  int t1;
  // strip leading and trailing white space
  tag1 = tag.simplifyWhiteSpace();
  // where to start tokenizing
  int ptr = 0;
  // where to end tokenizing
  int eol = tag1.findRev(QString(" "));
  
  do {
    t1 = tag1.find(QString(" "), ptr);
    a1 = tag1.mid(ptr, t1 - ptr);
    ptr = t1 + 1;
    l1.append(a1);
    cout << "[" << a1 << "]" << endl;
  } while (ptr < eol);
  
  a1 = tag1.mid(ptr);
  l1.append(a1);

  return l1;
}

// Load CML file
// Assume it follows CML.DTD (JCICS 39(6):928-942)
// Extensively check for errors, bail if it doesn't comply
int Render2D::LoadCMLFile(QString fn) {
  QFile f(fn);
  if (!f.open(IO_ReadOnly)) {
    QMessageBox::warning(this, "Couldn't read file", 
			 "Unable to open the specified file.");
    return false;
  }
  QTextStream t(&f);
  QString line;
  QString tmpline, tfont, tdata, a1, v1;
  DPoint *newpt;
  customObject *newobj;
  QString doctype, thistag, nexttag;
  QStringList tokens, refs;
  QStringList elementlist, fclist, x2list, y2list, idlist;
  QStringList a1list, a2list, olist;
  QString str_id, str_title, str_builtin, atomrefs1, atomrefs2, str_order,
    str_element;
  QString arr_fc, arr_x2, arr_y2;
  int d1, d2, d3, d4, d5, ptr = 0, cml_type = 0;
  int *st1;
  double l1, l2, l3, l4;

  // Read entire file to string
  QString wholefile = t.read();
  // We're done with the file.  Close it
  f.close();
  
  do {
    thistag = ReadTag(wholefile, ptr);
    cout << thistag.ascii() << endl;
    // ReadTag returns a null at EOF (ideally).
    if (thistag.isNull()) break;
    // check for 'special' tags...
    // continue past <?xml...> tag
    if (thistag.find(QString("?xml")) >= 0) continue;
    // but we really want the <!DOCTYPE...> tag
    if (thistag.find(QString("!DOCTYPE")) >= 0) {
      // read the actual document type
      d1 = thistag.find(QString("!DOCTYPE"));
      d2 = thistag.find(QString(" "), d1);
      d3 = thistag.find(QString(" "), d2 + 1);
      doctype = thistag.mid(d2 + 1, d3 - d2 - 1);
      if (thistag.upper().find("CML.DTD") >= 0) cml_type = 1;
      if (thistag.upper().find("CML-1999-05-15") >= 0) cml_type = 1;
      if (cml_type == 0) {
	QMessageBox::warning(this, "Error reading file", 
			     "Format (DTD) is unrecognized.\n"+thistag);
	return false;
      }
      continue;
    }
    // Look for beginning of document
    if (thistag.find(QString("<") + doctype) >= 0) {
      // push this tag on stack
      st1 = new int;
      *st1 = CML_MOLECULE;
      TagStack.append(st1);
      // parse attributes in tag
      tokens = ReadAttr(thistag);
      for (int c1 = 0; c1 < tokens.count(); c1 += 2) {
	a1 = *(tokens.at(c1));
	v1 = *(tokens.at(c1 + 1));
	cout << "-" << a1.ascii() << "|" << v1.ascii() << "-" << endl;
	// compare attribute a1 with list of relevant attr's
	if (a1.upper() == QString("ID")) MoleculeId = v1;
	if (a1.upper() == QString("TITLE")) MoleculeTitle = v1;
      }
    }
    // handle <atom> records
    if (thistag.find(QString("<atom ")) >= 0) {
      st1 = new int;
      *st1 = CML_ATOM;
      TagStack.append(st1);
      tmp_atom = new CML_Atom;
      tokens = ReadAttr(thistag);
      for (int c1 = 0; c1 < tokens.count(); c1 += 2) {
	a1 = *(tokens.at(c1));
	v1 = *(tokens.at(c1 + 1));
	cout << "-" << a1.ascii() << "|" << v1.ascii() << "-" << endl;
	// compare attribute a1 with list of relevant attr's
	if (a1.upper() == QString("ID")) tmp_atom->id = v1;
      }
    }
    // handle </atom>
    if (thistag == QString("</atom>")) {
      st1 = TagStack.last();
      if (*st1 == CML_ATOM) {
	TagStack.removeLast();
      } else {
	cout << "Imbalanced <atom> tags!" << endl;
      }
      CML_Atoms.append(tmp_atom);
    }
    // handle <bond> records
    if (thistag.find(QString("<bond ")) >= 0) {
      st1 = new int;
      *st1 = CML_BOND;
      TagStack.append(st1);
      tmp_bond = new CML_Bond;
      tmp_bond->a1 = "";
      tokens = ReadAttr(thistag);
      for (int c1 = 0; c1 < tokens.count(); c1 += 2) {
	a1 = *(tokens.at(c1));
	v1 = *(tokens.at(c1 + 1));
	cout << "-" << a1.ascii() << "|" << v1.ascii() << "-" << endl;
	// compare attribute a1 with list of relevant attr's
	if (a1.upper() == QString("ID")) tmp_bond->id = v1;
	if (a1.upper() == QString("ATOMREFS")) {
	  d5 = v1.find(QString(" "));
	  tmp_bond->a1 = v1.left(d5);
	  tmp_bond->a2 = v1.mid(d5 + 1);
	  cout << "bond[" << tmp_bond->a1.ascii() << "|";
	  cout << tmp_bond->a2.ascii() << "]" << endl;
	}
      }
    }
    // handle </bond>
    if (thistag == QString("</bond>")) {
      st1 = TagStack.last();
      if (*st1 == CML_BOND) {
	TagStack.removeLast();
      } else {
	cout << "Imbalanced <bond> tags!" << endl;
      }
      CML_Bonds.append(tmp_bond);
    }
    // handle <atomArray> records
    if (thistag == QString("<atomArray>")) {
      st1 = new int;
      *st1 = CML_ATOMARRAY;
      TagStack.append(st1);
      str_id.remove(0,9999);
      str_element.remove(0,9999);
    }
    // handle </atomArray>
    if (thistag == QString("</atomArray>")) {
      st1 = TagStack.last();
      if (*st1 == CML_ATOMARRAY) {
	TagStack.removeLast();
      } else {
	cout << "Imbalanced <atomArray> tags!" << endl;
      }
      // create lists of tokens in arr_x2, arr_y2, str_id, str_element, arr_fc
      idlist = Tokenize(str_id);
      elementlist = Tokenize(str_element);
      x2list = Tokenize(arr_x2);
      y2list = Tokenize(arr_y2);
      fclist = Tokenize(arr_fc);
      for (d1 = 0; d1 < idlist.count(); d1++) {
	tmp_atom = new CML_Atom;
	tmp_atom->id = idlist[d1];
	tmp_atom->element = elementlist[d1];
	tmp_atom->formalCharge = fclist[d1].toInt();
	tmp_atom->x = x2list[d1].toDouble();
	tmp_atom->y = y2list[d1].toDouble();
	CML_Atoms.append(tmp_atom);
      }
    }
    // handle <bondArray> records
    if (thistag == QString("<bondArray>")) {
      st1 = new int;
      *st1 = CML_BONDARRAY;
      TagStack.append(st1);
      atomrefs1.remove(0,9999);
      atomrefs2.remove(0,9999);
      str_order.remove(0,9999);
    }
    // handle </bondArray>
    if (thistag == QString("</bondArray>")) {
      cout << atomrefs1 << endl;
      cout << atomrefs2 << endl;
      cout << str_order << endl;
      st1 = TagStack.last();
      if (*st1 == CML_BONDARRAY) {
	TagStack.removeLast();
      } else {
	cout << "Imbalanced <bondArray> tags!" << endl;
      }
      // tokenize data
      a1list = Tokenize(atomrefs1);
      a2list = Tokenize(atomrefs2);
      olist = Tokenize(str_order);
      for (d1 = 0; d1 < a1list.count(); d1++) {
	tmp_bond = new CML_Bond;
	tmp_bond->a1 = a1list[d1];
	tmp_bond->a2 = a2list[d1];
	tmp_bond->order = olist[d1].toInt();
	CML_Bonds.append(tmp_bond);
      }
    }
    // handle <coordinate2>-</coordinate2>
    if (thistag.find(QString("<coordinate2 ")) >= 0) {
      st1 = TagStack.last();
      if (*st1 != CML_ATOM)
	cout << "Improperly placed <coordinate2> tag!" << endl;
      tokens = ReadAttr(thistag);
      for (int c1 = 0; c1 < tokens.count(); c1 += 2) {
	a1 = *(tokens.at(c1));
	v1 = *(tokens.at(c1 + 1));
	cout << "COORD2-" << a1.ascii() << "|" << v1.ascii() << "-" << endl;
	// compare attribute a1 with list of relevant attr's
	if (a1.upper() == QString("TITLE")) str_title = v1;
	if (a1.upper() == QString("BUILTIN")) str_builtin = v1;
      }
      tdata = ReadData(wholefile, ptr);
      nexttag = ReadTag(wholefile, ptr);
      if (nexttag != QString("</coordinate2>"))
	cout << "Imbalanced <coordinate2> tags!" << endl;
      if (str_builtin != QString("xy2"))
	cout << "Unrecognized builtin type in <coordinate2>!" << endl;
      d5 = tdata.find(QString(" "));
      tmp_atom->x = tdata.left(d5).toDouble();
      tmp_atom->y = tdata.mid(d5).toDouble();
    }
    // handle <coordinate3>-</coordinate3>
    if (thistag.find(QString("<coordinate3 ")) >= 0) {
      st1 = TagStack.last();
      if (*st1 != CML_ATOM)
	cout << "Improperly placed <coordinate3> tag!" << endl;
      tokens = ReadAttr(thistag);
      for (int c1 = 0; c1 < tokens.count(); c1 += 2) {
	a1 = *(tokens.at(c1));
	v1 = *(tokens.at(c1 + 1));
	cout << "COORD3-" << a1.ascii() << "|" << v1.ascii() << "-" << endl;
	// compare attribute a1 with list of relevant attr's
	if (a1.upper() == QString("TITLE")) str_title = v1;
	if (a1.upper() == QString("BUILTIN")) str_builtin = v1;
      }
      tdata = ReadData(wholefile, ptr);
      nexttag = ReadTag(wholefile, ptr);
      if (nexttag != QString("</coordinate3>"))
	cout << "Imbalanced <coordinate3> tags!" << endl;
      if ( (str_builtin != QString("xyz3")) &&
	   (str_builtin != QString("xyzFract")) )
	cout << "Unrecognized builtin type in <coordinate3>!" << endl;
      tokens = Tokenize(tdata);
      tmp_atom->x = tokens[0].toDouble();
      tmp_atom->y = tokens[1].toDouble();
      tmp_atom->z = tokens[2].toDouble();
    }
    // handle <string>-</string> AND <stringArray>-</stringArray>
    if ( (thistag.find(QString("<string ")) >= 0) ||
	 (thistag.find(QString("<stringArray ")) >= 0) ) {
      tokens = ReadAttr(thistag);
      for (int c1 = 0; c1 < tokens.count(); c1 += 2) {
	a1 = *(tokens.at(c1));
	v1 = *(tokens.at(c1 + 1));
	cout << "STRING-" << a1.ascii() << "|" << v1.ascii() << "-" << endl;
	// compare attribute a1 with list of relevant attr's
	if (a1.upper() == QString("TITLE")) str_title = v1;
	if (a1.upper() == QString("BUILTIN")) str_builtin = v1;
      }
      tdata = ReadData(wholefile, ptr);
      nexttag = ReadTag(wholefile, ptr);
      cout << "DATA<string>[" << tdata << "]" << endl;
      if (nexttag.left(8) != QString("</string"))
	cout << "Imbalanced <string> tags!" << endl;
      // known cases of <string>
      st1 = TagStack.last();
      if ( (*st1 == CML_BOND) && (str_builtin == QString("atomRef")) ) {
	if (tmp_bond->a1 == "") 
	  tmp_bond->a1 = tdata;
	else
	  tmp_bond->a2 = tdata;
      }
      if ( (*st1 == CML_BOND) && (str_builtin == QString("order")) )
	tmp_bond->order = tdata.toInt();
      if ( (*st1 == CML_ATOM) && (str_builtin == QString("elementType")) )
	tmp_atom->element = tdata;
      if ( (*st1 == CML_ATOMARRAY) && (str_builtin == QString("elementType")) )
	str_element = tdata;
      if ( (*st1 == CML_ATOMARRAY) && (str_builtin == QString("id")) )
	str_id = tdata;
      if ( (*st1 == CML_BONDARRAY) && (str_builtin == QString("atomRefs")) ) {
	if (atomrefs1.length() == 0)
	  atomrefs1 = tdata;
	else
	  atomrefs2 = tdata;
      }
      if ( (*st1 == CML_BONDARRAY) && (str_builtin == QString("order")) )
	str_order = tdata;
    }
    // handle <integerArray>-</integerArray> AND 
    // <floatArray>-</floatArray>
    if ( (thistag.find(QString("<integerArray ")) >= 0) ||
	 (thistag.find(QString("<floatArray ")) >= 0) ) {
      tokens = ReadAttr(thistag);
      for (int c1 = 0; c1 < tokens.count(); c1 += 2) {
	a1 = *(tokens.at(c1));
	v1 = *(tokens.at(c1 + 1));
	// compare attribute a1 with list of relevant attr's
	if (a1.upper() == QString("TITLE")) str_title = v1;
	if (a1.upper() == QString("BUILTIN")) str_builtin = v1;
      }
      tdata = ReadData(wholefile, ptr);
      nexttag = ReadTag(wholefile, ptr);
      cout << "DATA<numArray>[" << tdata << "]" << endl;
      if ( (nexttag != QString("</integerArray>")) &&
	   (nexttag != QString("</floatArray>")) )
	cout << "Imbalanced <integerArray> or <floatArray> tags!" << endl;
      // known cases of <integerArray> and <floatArray>
      st1 = TagStack.last();
      if ( (*st1 == CML_ATOMARRAY) && (str_builtin==QString("formalCharge")) )
	arr_fc = tdata;
      if ( (*st1 == CML_ATOMARRAY) && (str_builtin == QString("x2")) )
	arr_x2 = tdata;
      if ( (*st1 == CML_ATOMARRAY) && (str_builtin == QString("y2")) )
	arr_y2 = tdata;
      if ( (*st1 == CML_ATOMARRAY) && (str_builtin == QString("x3")) )
	arr_x2 = tdata;
      if ( (*st1 == CML_ATOMARRAY) && (str_builtin == QString("y3")) )
	arr_y2 = tdata;
      if ( (*st1 == CML_BONDARRAY) && (str_builtin == QString("order")) )
	str_order = tdata;
    }
    // Look for end of document
    if (thistag.find(QString("</") + doctype) >= 0) {
      st1 = TagStack.last();
      if (*st1 == CML_MOLECULE) {
	TagStack.removeLast();
	// should be last tag...
	if (!TagStack.isEmpty()) cout << "Imbalanced tags!" << endl;
	continue;
      }
    }
  } while (ptr < wholefile.length());

  // convert CML_Atom and CML_Bond to DPoint and customObject, respectively
  deselectAll();
  pointList.clear();
  objectList.clear();

  cout << "count:" << CML_Atoms.count() << endl;
  // Copy CML_Atoms in order to pointList
  for (tmp_atom = CML_Atoms.first(); tmp_atom != NULL; 
       tmp_atom = CML_Atoms.next() ) {
    tmp_pt = tmp_atom->toDPoint();
    pointList.append(tmp_pt);
    if (tmp_pt->Name() != QString("C")) {
      tmp_obj = new customObject;
      tmp_obj->setType(MLABEL);
      tmp_obj->setLabel(tmp_atom->element);
      tmp_obj->setFont(currentFont);
      tmp_obj->addPoint(tmp_pt);
      objectList.append(tmp_obj);
    }
  }
  // Now build bonds
  for (tmp_bond = CML_Bonds.first(); tmp_bond != NULL;
       tmp_bond = CML_Bonds.next() ) {
    tmp_obj = new customObject;
    tmp_obj->setType(MLINE);
    // find first atom
    for (tmp_pt = pointList.first(); tmp_pt != NULL;
	 tmp_pt = pointList.next() ) {
      if (tmp_bond->a1 == tmp_pt->id) {
	tmp_obj->addPoint(tmp_pt);
	break;
      }
    }
    // find second atom
    for (tmp_pt = pointList.first(); tmp_pt != NULL;
	 tmp_pt = pointList.next() ) {
      if (tmp_bond->a2 == tmp_pt->id) {
	tmp_obj->addPoint(tmp_pt);
	break;
      }
    }
    tmp_obj->setThickness(tmp_bond->order);
    objectList.append(tmp_obj);
  }
  // Fix the structure to add coordinates and look pretty
  //int ng = GroupAll();
  //for (int cc = 1; cc <= ng; cc++)
  //  AddCoordinates(cc);

  repaint();
  emit UpdateStatusBar(QString("Read file ") + QString(fn));
  return true;
}

// Save CML file
// Use CML.DTD (JCICS 39(6):928-942)
int Render2D::SaveCMLFile(QString fn) {
  Cleanup();  // see processing.cpp

  int acount = 0, bcount = 0, idx = 0;
  DPoint *tmppt;
  customObject *tmpobj;
  QList<DPoint> atoms;
  QString tmpline, i1, i2;
  QFile f(fn);
  if (!f.open(IO_WriteOnly)) {
    QMessageBox::warning(this, "Can't save file", 
			 "Can't save the specified file.");		
    return false;
  }
  QTextStream t(&f);
  // introductory XML stuff -- declare DTD, start molecule
  t << "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" << endl;
  t << "<!DOCTYPE molecule SYSTEM \"cml.dtd\" []>" << endl;
  t << "<molecule ";
  if (!MoleculeTitle.isNull())
    t << "title=\"" << MoleculeTitle.ascii() << "\" ";
  if (MoleculeId.isNull())
    MoleculeId = fn.right(fn.length() - fn.findRev("/") - 1);
  t << "id=\"" << MoleculeId.ascii();
  t << "\" convention=\"CML-1999-05-15\">" << endl;
  // add miscellaneous info, a la JChemPaint
  t << "  <string title=\"GenerationDate\">" << QDateTime::currentDateTime().toString() << "</string>" << endl;
  t<<"  <string title=\"GenerationSoftware\">XDrawChem 0.84alpha</string>"<<endl;
  
  // store atoms
  i1 = QString("a");
  idx = 1;
  for (tmp_pt = pointList.first(); tmp_pt != 0; tmp_pt = pointList.next()) {
    if (tmp_pt->isAtom() == false) continue;  // skip non-atoms
    i2.setNum(idx);
    t << "  <atom id=\"" << (i1 + i2).ascii() << "\">" << endl;
    t << "    <string builtin=\"elementType\">";
    t << tmp_pt->Name() << "</string>" << endl;
    t << "    <coordinate2 builtin=\"xy2\">";
    t << tmp_pt->x();
    t << " ";
    t << tmp_pt->y();
    t << "</coordinate2>" << endl;
    t << "    <integer builtin=\"formalCharge\">0</integer>" << endl; 
    t << "  </atom>" << endl;
    idx++;
  }

  // now store bonds
  i1 = QString("b");
  idx = 1;
  for (tmp_obj = objectList.first(); tmp_obj != 0; 
       tmp_obj = objectList.next()) {
    // only lines get stored in this flavor of CML.
    if ( (tmp_obj->Type() == MLINE) ||
	 (tmp_obj->Type() == MUPLINE) ||
	 (tmp_obj->Type() == MDOWNLINE) ) {
      i2.setNum(idx);
      t << "  <bond id=\"" << (i1 + i2).ascii() << "\" atomRefs=\"";
      i2.setNum(pointList.find(tmp_obj->getPoint1()) + 1);
      t << "a" << i2 << " ";
      i2.setNum(pointList.find(tmp_obj->getPoint2()) + 1);
      t << "a" << i2 << "\">" << endl;
      i2.setNum(tmp_obj->Thickness());
      t << "    <string builtin=\"order\">" << i2 << "</string>" << endl;
      t << "  </bond>" << endl;
      idx++;
    }
  }

  // close out molecule
  t << "</molecule>" << endl;

  // Clean up
  f.close();
  vchanged = false;

  //emit UpdateStatusBar(QString("Can't really save CML yet!"));
  return false;
}






