// Structure Diagram Generator
// Add coordinates to a connectivity table.
// Based on JMDraw

#include <iostream.h>
#include <qpainter.h>
#include <qimage.h>
#include <qevent.h>
#include <qlist.h>
#include <qstack.h>
#include <math.h>
#include <qbitmap.h>
#include <qcursor.h>
#include <qdialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpointarray.h>
#include <qprinter.h>
#include <qmessagebox.h>
#include <qpaintdevicemetrics.h>
#include <qnamespace.h>
#include <qregexp.h>
#include <qdatetime.h>

#include "defs.h"
#include "customObject.h"
#include "dpoint.h"
#include "render2d.h"

QList<XNode> nodelist;
QList<DPoint> path;  // for depth-first ring detection

bool Render2D::TraverseXNodes(XNode *t_from, XNode *t_to) {
  DPoint *tmppt;
  if (t_to == 0) { // first node
    for (tmppt = t_from->connected.first(); tmppt != 0;
	 tmppt = t_from->connected.next()) {
      return TraverseXNodes(t_from, nodelist.at(pointList.find(tmppt)));
    }
  } else { // not the first node
    // check if we traveled in a loop
    if (t_from->thisnode == path.first()) return true;
    // prevent infinite looping (if there's a ring but didn't start on it)
    if (path.find(t_from->thisnode) < 0) path.append(t_from->thisnode);
    for (tmppt = t_to->connected.first(); tmppt != 0;
	 tmppt = t_to->connected.next()) {
      if (tmppt == t_from->thisnode) continue; // prevent backtrace
      return TraverseXNodes(t_to, nodelist.at(pointList.find(tmppt)));
    }
  }
  return false;
}

void Render2D::AddCoordinates(int g1) {
  QList<DPoint> rings[32];
  int nrings = 0;
  QList<DPoint> polys[8];
  int npolys = 0;

  // collect DPoints and customObjects in group g1
  QList<DPoint> groupAtoms;
  QList<customObject> groupBonds;

  for (tmp_pt = pointList.first(); tmp_pt != 0; tmp_pt = pointList.next()) {
    tmp_pt->nrings = 0;
    tmp_pt->npolys = 0;
    if (tmp_pt->gid == g1) groupAtoms.append(tmp_pt);
  }
  for (tmp_obj = objectList.first(); tmp_obj != 0; 
       tmp_obj = objectList.next()) {
    tmp_obj->nrings = 0;
    tmp_obj->npolys = 0;
    if (tmp_obj->getPoint1()->gid == g1) groupBonds.append(tmp_obj);
  }
  // build simple connection table
  for (tmp_pt = groupAtoms.first(); tmp_pt != 0; tmp_pt = groupAtoms.next()) {
    XNode *newnode = new XNode;
    newnode->thisnode = tmp_pt;
    for (tmp_obj = groupBonds.first(); tmp_obj != 0; 
	 tmp_obj = groupBonds.next()) {
      if (tmp_obj->isMember(tmp_pt) == true)
	newnode->connected.append(tmp_obj->getOther(tmp_pt));
    }
    nodelist.append(newnode);
  }
  // find rings
  int cc, iflag = false;
  XNode *tmp_x;
  DPoint *tmp_y;
  for (tmp_x = nodelist.first(); tmp_x != 0; tmp_x = nodelist.next()) {
    if (nrings > 0) iflag = true;
    if (TraverseXNodes(tmp_x, 0) == true) {
      for (cc = 0; cc < nrings; cc++) {
	for (tmp_y = path.first(); tmp_y != 0; tmp_y = path.next())
	  if (rings[cc].find(tmp_y) < 0) iflag = false;
      }
      if (iflag == false) {
	for (tmp_y = path.first(); tmp_y != 0; tmp_y = path.next()) {
	  rings[nrings].append(tmp_y);
	}
	nrings++;
      }
    }
    path.clear();
    nodelist.find(tmp_x); // reset noelist iterator
  }
}



