#include <qwidget.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include <qcolor.h>
#include <qfont.h>
#include <qnamespace.h>
#include <qpixmap.h>
#include <qcombobox.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qtooltip.h>

#include "symboldialog.h"
#include "defs.h"

#include "sym_delta_minus.xpm"
#include "sym_delta_plus.xpm"
#include "sym_minus.xpm"
#include "sym_plus.xpm"
#include "sym_radical1.xpm"
#include "sym_radical2.xpm"
#include "sym_radical_plus.xpm"
#include "sym_radical_minus.xpm"

SymbolDialog::SymbolDialog(QWidget *parent, QString name):
	QDialog(parent, name, TRUE)
{
	// Init variables which tell which ring was picked
	whichsymbol = 0;

	QPixmap pix;
	QPushButton *ringbutton;
	QLabel *caption;
	QPushButton *dismiss;
	setBackgroundColor(lightGray);
	setCaption("Symbol tool");

	QGridLayout *mygrid = new QGridLayout(this,3,4,5);

	caption = new QLabel("Choose a symbol:", this);
	caption->setBackgroundColor(lightGray);
	mygrid->addMultiCellWidget(caption, 0, 0, 0, 3);

	pix = QPixmap(sym_delta_minus);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(white));
	ringbutton->setPixmap(pix);
	ringbutton->setGeometry(0,0,20,20);
	QToolTip::add(ringbutton, "Partial negative charge");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Sym1()) );
	mygrid->addWidget(ringbutton, 1, 0);

	pix = QPixmap(sym_delta_plus);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(white));
	ringbutton->setPixmap(pix);
	ringbutton->setGeometry(0,0,20,20);
	QToolTip::add(ringbutton, "Partial positive charge");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Sym2()) );
	mygrid->addWidget(ringbutton, 1, 1);

	pix = QPixmap(sym_minus);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(white));
	ringbutton->setPixmap(pix);
	ringbutton->setGeometry(0,0,20,20);
	QToolTip::add(ringbutton, "Negative charge");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Sym3()) );
	mygrid->addWidget(ringbutton, 1, 2);

	pix = QPixmap(sym_plus);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(white));
	ringbutton->setPixmap(pix);
	ringbutton->setGeometry(0,0,20,20);
	QToolTip::add(ringbutton, "Positive charge");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Sym4()) );
	mygrid->addWidget(ringbutton, 1, 3);

	pix = QPixmap(sym_radical1);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(white));
	ringbutton->setPixmap(pix);
	ringbutton->setGeometry(0,0,20,20);
	QToolTip::add(ringbutton, "Radical");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Sym5()) );
	mygrid->addWidget(ringbutton, 2, 0);

	pix = QPixmap(sym_radical2);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(white));
	ringbutton->setPixmap(pix);
	ringbutton->setGeometry(0,0,20,20);
	QToolTip::add(ringbutton, "Lone pair");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Sym6()) );
	mygrid->addWidget(ringbutton, 2, 1);

	pix = QPixmap(sym_radical_minus);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(white));
	ringbutton->setPixmap(pix);
	ringbutton->setGeometry(0,0,20,20);
	QToolTip::add(ringbutton, "Radical - negative charge");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Sym7()) );
	mygrid->addWidget(ringbutton, 2, 2);

	pix = QPixmap(sym_radical_plus);
	ringbutton = new QPushButton(this);
	ringbutton->setPalette(QPalette(white));
	ringbutton->setPixmap(pix);
	ringbutton->setGeometry(0,0,20,20);
	QToolTip::add(ringbutton, "Radical - positive charge");
  connect(ringbutton, SIGNAL(clicked()), SLOT(Sym8()) );
	mygrid->addWidget(ringbutton, 2, 3);

  dismiss = new QPushButton("Dismiss", this);
  dismiss->setPalette(QPalette(lightGray));
  connect(dismiss, SIGNAL(clicked()), SLOT(reject()) );
	mygrid->addMultiCellWidget(dismiss, 3, 3, 1, 2);
}

int SymbolDialog::getSymbol(void)
{
	return whichsymbol;
}

void SymbolDialog::Sym1(void)
{
	whichsymbol = SYM_DELTA_MINUS;
	accept();
}

void SymbolDialog::Sym2(void)
{
	whichsymbol = SYM_DELTA_PLUS;
	accept();
}

void SymbolDialog::Sym3(void)
{
	whichsymbol = SYM_MINUS;
	accept();
}

void SymbolDialog::Sym4(void)
{
	whichsymbol = SYM_PLUS;
	accept();
}

void SymbolDialog::Sym5(void)
{
	whichsymbol = SYM_RADICAL1;
	accept();
}

void SymbolDialog::Sym6(void)
{
	whichsymbol = SYM_RADICAL2;
	accept();
}

void SymbolDialog::Sym7(void)
{
	whichsymbol = SYM_RADICAL_MINUS;
	accept();
}

void SymbolDialog::Sym8(void)
{
	whichsymbol = SYM_RADICAL_PLUS;
	accept();
}
