#include <qwidget.h>
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include <qcolor.h>
#include <qfont.h>
#include <qnamespace.h>
#include <qtextstream.h>
#include <qfontmetrics.h>
#include <qrect.h>
#include <qpainter.h>

#include <iostream.h>

#include "textdialog.h"

// Text dialog code
TextDialog::TextDialog(QWidget *parent,QString name,QString caption,QFont fs): 
	QDialog(parent, name, TRUE)
{
	QPushButton *sup, *sub, *pview, *ok, *cancel;
	QLabel *l1, *textmsg;
	setBackgroundColor(lightGray);
	setCaption(QString("Text tool"));
	l1 = new QLabel(QString("Preview"), this);
	l1->setGeometry(10,10,200,30);
	l1->setBackgroundColor(lightGray);
	tpw = new TextPreviewWidget(fs, this);
	tpw->setGeometry(10,40,210,50);
	textmsg = new QLabel(caption, this);
	textmsg->setGeometry(10,100,200,30);
	textmsg->setBackgroundColor(lightGray);
	textfield = new MyEditor(this);
	textfield->setGeometry(10,135,400,80);
	sup = new QPushButton("Superscript", this);
	sup->setGeometry(10,220,90,30);
	sup->setPalette(QPalette(lightGray));
	connect(sup, SIGNAL(clicked()), SLOT(setSuper()) );
	sub = new QPushButton("Subscript", this);
	sub->setGeometry(100,220,90,30);
	sub->setPalette(QPalette(lightGray));
	connect(sub, SIGNAL(clicked()), SLOT(setSub()) );
	pview = new QPushButton("Preview", this);
	pview->setGeometry(210,220,70,30);
	pview->setPalette(QPalette(lightGray));
	connect(pview, SIGNAL(clicked()), SLOT(PreviewText()) );
	ok = new QPushButton("OK", this);
	ok->setGeometry(280,220,70,30);
	ok->setPalette(QPalette(lightGray));
	connect(ok, SIGNAL(clicked()), SLOT(accept()) );
	cancel = new QPushButton("Cancel", this);
	cancel->setGeometry(350,220,70,30);
	cancel->setPalette(QPalette(lightGray));
	connect(cancel, SIGNAL(clicked()), SLOT(reject()) );
	// connect returnPressed to PreviewText
	connect(textfield, SIGNAL(returnPressed()),
		this, SLOT(PreviewText()) );
}

void TextDialog::setText(QString t)
{
	textfield->setText(t);
}

void TextDialog::setFont(QFont t)
{
	textfield->setFont(t);
}

QString TextDialog::getText(void)
{
	return textfield->text();
}

void TextDialog::setSuper(void)
{
  textfield->Format(1);
  tpw->Update(textfield->text());
}

void TextDialog::setSub(void)
{
  textfield->Format(2);
  tpw->Update(textfield->text());
}

void TextDialog::PreviewText(void)
{
  tpw->Update(textfield->text());
}

// ####################
// MyEditor widget code
// ####################

MyEditor::MyEditor(QWidget *parent, const char *name) :
  QMultiLineEdit(parent, name) {
}

bool MyEditor::viewMarkedRegion(int *x1, int *y1, int *x2, int *y2) {
  return getMarkedRegion(x1, y1, x2, y2);
}

void MyEditor::Format(int fcode)
{
  int r1, c1, r2, c2;

  if (getMarkedRegion(&r1, &c1, &r2, &c2) == FALSE) return;

  switch (fcode) {
  case 1:  // superscript
    insertAt(QString("</super>"), r2, c2);
    insertAt(QString("<super>"), r1, c1);
    break;
  case 2:  // subscript
    insertAt(QString("</sub>"), r2, c2);
    insertAt(QString("<sub>"), r1, c1);
    break;
  }
}

// ###################
// Preview widget code
// ###################

TextPreviewWidget::TextPreviewWidget(QFont fs, QWidget *parent, 
				     const char *name):QWidget(parent, name) {
  setPalette(QPalette(lightGray));

  myfont = fs;
}

void TextPreviewWidget::Update(QString n) {
  erase(0,0,399,79);

  int nextlineoff = 0, xpos = 0, ypos = 0, yoff = 0;
  ypos = myfont.pointSize() + 1;

  if (!n) return;  // if n is null (entry field is empty), return

  QPainter p;
  p.begin(this);
  p.setFont(myfont);
  p.setPen(QColor(0,0,0));
  QFontMetrics fm = p.fontMetrics();

  QTextStream t(&n, IO_ReadOnly);

  int superflag = 0, subflag = 0, newsize, off, voff;
  switch (myfont.pointSize()) {
  case 8:
    newsize = 8; off = 2; voff = -1;
    break;
  case 10:
    newsize = 8; off = 2; voff = -3;
    break;
  case 12:
    newsize = 8; off = 2; voff = -4;
    break;
  case 14:
    newsize = 10; off = 2; voff = -4;
    break;
  case 18:
    newsize = 12; off = 3; voff = -5;
    break;
  case 24:
    newsize = 12; off = 3; voff = -10;
    break;
  default:
    newsize = 8; off = 4; voff = -2;
    break;
  }
  QFont newfont(myfont.family(), newsize);

  do {
    QString l = t.readLine();
    int superflag = 0, subflag = 0;
    if (l.contains(QString("<super>"), FALSE) > 0) {
      superflag = 1;
      ypos += 1;
      if (newsize == 12) ypos += 1;
    }
    if (l.contains(QString("<sub>"), FALSE) > 0) subflag = 1;
      
    xpos = 0;
      
    for (int i = 0; i < l.length(); i++) {
      QChar n1 = l.at(i);
      yoff = 0;
      // handle super/sub
      if (n1 == "<") {
	QString s1;
	int lflag = 0;
	if (l.at(i+3).upper() == "P") {
	  yoff = voff;
	  i += 7;
	  lflag = 1;
	} else {
	  yoff = off;
	  i += 5;
	}
	do {
	  s1.append(l.at(i));
	  i++;
	  n1 = l.at(i);
	} while (n1 != "<");
	p.setFont(newfont);
	p.drawText(xpos, ypos + yoff, s1);
	xpos += fm.width(s1);
	if (lflag == 1)
	  i += 7;
	else
	  i += 5;
      } else {
	p.setFont(myfont);
	p.drawText(xpos, ypos, QString(n1));
	xpos += fm.width(n1);
      }
    }
      
    if (subflag > 0) 
      ypos += 1;
    if ( (newsize == 12) && (subflag > 0) ) ypos += 1;
    ypos += myfont.pointSize();
  } while (!t.atEnd());
}

